
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <SDL2/SDL.h>

//embedded data like graphics
#include "embed.c"

// ****INTERNAL STUFF****

#define identifier_max 31 //WARNING: I strongly advise not making this value smaller
#define arguments_max 10 //this can be made bigger, edit funccall.c/.s to do so
#define members_max 64
#define return_size_max 65536 //WARNING: this is possibly not large enough

/*

**Documentation**

The following three types define most of the functionality of seashell:

var  : a pointer with a type (we just freely get memory and cast based on type to get gcc to do it)
func : a pointer with typed args, we call into the pointer using some asm in funcCall.c
type : a list of types (just by size and signed-ness); doesn't do types-of-types, should generalize okay

*/

typedef struct {
    char name[identifier_max];
    unsigned char* pntr;
    int amnt;
    short typeUse;
} seashell_var;

typedef struct {
    char name[identifier_max];
    char args;
    unsigned char* pntr;
    short argTypes[arguments_max];
    int argAmnts[arguments_max]; //previously unused (0-th now inhabits object type for obj funcs)
    short typeReturn;
} seashell_func;

typedef struct {
    char name[identifier_max];
    char members;
    int sizes[members_max]; //NOTE: internally, size values also store signed-ness of variables (negative = signed)
    char floaty[members_max];
    short sizeTot;
} seashell_type;

//calls a function given the above information, implemented w/ C/asm in funcCall.c / .s
void INTERNAL_funcCall(seashell_var* vars, seashell_func* funcs, seashell_type* types, int funcNumb, seashell_var* returnLoc, seashell_var* args);

/*

**Documentation**

The following three are for the whole program: all vars, funcs, and types.

*/

seashell_var   *seashell_vars;
seashell_func  *seashell_funcs;
seashell_type  *seashell_types;

unsigned char* seashell_defaultArgs[arguments_max]; //default locations of arguments (for const args); qwords
seashell_var   seashell_args[arguments_max];
seashell_var   seashell_return;

int   seashell_varsAmnt,  seashell_varsAllocd;
short seashell_funcsAmnt, seashell_funcsAllocd;
short seashell_typesAmnt, seashell_typesAllocd;

char seashell_sortTypes = 0;
char seashell_sortVars = 0;
char seashell_sortFuncs = 0;
char seashell_sortAll = 1;

#define io_max 131072
#define input_max 512
#define cmds_max 128

/*

**Documentation**

io is the actual text of the shell, iolen defines where we write and so on
userInput is a copy of what the user has recently typed in (also in io), inputlen is similar to iolen
io temp is for copying and stuff (dont modify!)

*/


char seashell_io[io_max + input_max]; //for safety w/ cmd copying system
int  seashell_ioLen = 0;
char seashell_userInput[input_max+1]; //add one for null
int  seashell_inputLen = 0;
char seashell_ioTemp[io_max];

/*

**Documentation**

Scrolled defines by how many lines the user has scrolled
recent input, recent commands, inputsel determines which is to be re-written to userinput

*/


int  seashell_scrolled = 0;

char seashell_recentInput[cmds_max][input_max];
int  seashell_recentInputSel = 0;

char seashell_initted = 0; //has the user asked us to waste their clocks? (did we turn on?)
char seashell_quitted = 0; //has the user told us to stop wasting their clocks? (did we turn off?)
SDL_Window* seashell_window;
char seashell_title[256]; //WINDOW title
SDL_Renderer* seashell_render;
SDL_Event seashell_event;
const unsigned char *seashell_keystate; //for handling shift-eaten events.
char seashell_fixer[94]; //general key fixing thing cause linux sucks
char seashell_hasInput = 0; //do we currently have the input stream?
char seashell_recentInputMod = 0; //we do this to resolve toggle

SDL_Event seashell_feedout[64]; //takes input events not belonging to us and feeds it back into the event que
int seashell_feedLen = 0; //how many events to feed
char seashell_holding = 0, seashell_stepping = 0; //stop the program on update/steps?
unsigned char seashell_bgR, seashell_bgG, seashell_bgB; //background colors, calculated in seashell_init
char seashell_shifting = 0; //is the user pressing a shift key
char seashell_controlling = 0; //is the user pressing a control key
int seashell_cursorNegative = 0; //the position to the left from current input maximum
char seashell_promptOnUpdate = 0; //next update, print out "seashell~ "
char seashell_renderEvent = 2; //used for event-based rendering, making the program use as less clocks
int seashell_linesRecent = 0; //measures the lines recently calculated/rendered, if changes, orders full re-render
char seashell_stringGet[input_max]; //used for parsing, not done on the stack for speed
char seashell_calledFromStepping = 0; //DO NOT EDIT: marks when update is called from the stepping function
int seashell_varClipboard = -1; //which variable is in the clipboard, to be copied later?

int seashell_holdsLeft = 0, seashell_measuringSteps = 0, seashell_holdsTotal = 1;
unsigned long long seashell_holdStart, seashell_holdEnd;

char seashell_oper; //are we holding or not? (step currently)

#define defaultStepsAllocd 32

int seashell_stepsAllocd = 0, seashell_stepOn = 0;
unsigned long long *seashell_stepsAdd, *seashell_stepsRec;
unsigned long long seashell_ticksTotal;

/* runtime consts : don't edit at runtime! */

int seashell_windowHeight = 752;
int seashell_windowWidth = 768;
int seashell_textHeight = 16;
int seashell_textWidth = 8;

/* render stuff : some bars for drawing */

SDL_Rect seashell_renderBar; //this is the bar at the middle, almost purely asthetic
SDL_Rect seashell_defBar;  //same as above, but for default text view; @ top.
SDL_Rect seashell_fontLoc; //where is the font in renderspace
SDL_Rect seashell_fontGet; //where to get the font out of the bitmap
SDL_Rect seashell_identifyBar; //iterated to all locations of '@' '-' '*'

#define seashell_grayDraw 215
#define seashell_dgrayDraw 105 //make sure less than 215

/*

**Documentation**

the following things are for bar graphing, seashell_gbars indicates the actual graphical data
the amount we have allocated is gbarsallocd, gbarsdraw defines how much we are drawing

*/

char seashell_renderVisualMode = -1; // 0 bar/count, 1 cart (deprecated), -1 text
int seashell_gBarsAllocd = 0, seashell_gBarsDraw = 0;
int seashell_gBarsHeight, seashell_gBarsWidth;
float seashell_gBarsHVal;
int seashell_gBarsHeightHalf, seashell_gBarsWidthHalf;
SDL_Rect *seashell_gBars;
char *seashell_gBarsNegative;
unsigned long long seashell_gCount[256];

char seashell_watching = 0; //how many are we watching (>= 1 draws)
int  seashell_watch[6]; //watched vars (accesses seashell_vars indexes)
int  seashell_watchTxtLen = 20; //longest identifier to render
int  seashell_watchBytLen = 10; //width in chars (added to txtlen + 3 for full size)
unsigned long long seashell_watchRec[6] = {0,0,0,0,0,0}; //store values for delta analysis
char seashell_watchStrings[64];
SDL_Rect seashell_watchBar;

#define default_linesClear 128

/*
**Documentation**

Cleans a defined amount of lines from the io stream, we use this when things get big in seashell.
*/

void INTERNAL_clearIOLines(int numb)
{
    int loc = 0, count = 0;

    while (count < numb) {
	if (seashell_io[loc] == '\n' && loc < seashell_ioLen) {
	    count++;
        } else if (loc >= seashell_ioLen) {
	    break;
	}
	loc++;
    }

    memcpy(seashell_ioTemp,seashell_io,seashell_ioLen);
    memcpy(seashell_io, &(seashell_ioTemp[loc]), seashell_ioLen-loc);
    seashell_ioLen -= loc;
}

/*
**Documentation**

This calculates how many graphical lines we will need to render on ascii line.
Takes the starting character and a fill location for the final x stop, returns total lines.
*/

int INTERNAL_calculateLines(int whereFrom, int* fillX)
{
    int xCur = 0, yCur = 0, tempAdd;
    int charsMax = seashell_windowWidth/seashell_textWidth;
    
    while (seashell_io[whereFrom] != '\n' && whereFrom < seashell_ioLen) {
	if (xCur >= charsMax) {
	    yCur += 1;
	    xCur = 0;
	} else if (xCur > charsMax-identifier_max && seashell_io[whereFrom] == ' ' && seashell_io[whereFrom+1] != ' ') {
	    tempAdd = 1;
	    while (seashell_io[whereFrom+tempAdd] != ' ' && seashell_io[whereFrom+tempAdd] != '\n' && whereFrom+tempAdd < seashell_ioLen) {
		tempAdd += 1;
	    }
	    if (xCur+tempAdd-1 >= charsMax) {
		yCur += 1;
		xCur = 0;
	    } else {
		xCur += 1;
	    }
	} else {
	    xCur += 1;
	}
	whereFrom += 1;
    }

    *fillX = xCur;
    return yCur + 1;
}

/*
**Documentation**

This function calculates visual lines with a stopping point, similarly to the previous.
*/

int INTERNAL_calculateLinesDefinedStop(int whereFrom, int howMany, int* fillX)
{
    int xCur = 0, yCur = 0, tempAdd;
    int charsMax = seashell_windowWidth/seashell_textWidth;
    int iter = 0;
    
    while (seashell_io[whereFrom] != '\n' && whereFrom < howMany && whereFrom < seashell_ioLen) {
	if (xCur >= charsMax) {
	    yCur += 1;
	    xCur = 0;
	} else if (xCur > charsMax-identifier_max && seashell_io[whereFrom] == ' ' && seashell_io[whereFrom+1] != ' ') {
	    tempAdd = 1;
	    while (seashell_io[whereFrom+tempAdd] != ' ' && seashell_io[whereFrom+tempAdd] != '\n' && whereFrom+tempAdd < seashell_ioLen) {
		tempAdd += 1;
	    }
	    if (xCur+tempAdd-1 >= charsMax) {
		yCur += 1;
		xCur = 0;
	    } else {
		xCur += 1;
	    }
	} else {
	    xCur += 1;
	}
	whereFrom += 1;
	iter += 1;
    }

    *fillX = xCur;
    return yCur + 1;
}

/*
**Documentation**

Injects a variable into the watch, not much craziness.
*/

int INTERNAL_addWatch(int numb)
{
    if (seashell_watching < 6) {
	for (int iter = 0; iter < seashell_watching; iter++)
	{
	    if (seashell_watch[iter] == numb) {
		return 2; //already being watched
	    }
	}
	
        seashell_watch[seashell_watching] = numb;
	seashell_watching++;
	return 0; //added
    }

    return 1; //watch is full
}

/*
**Documentation**

Deletes a variable from the watch, not much insanity.
*/

int INTERNAL_removeWatch(int numb)
{
    int get = -1;

    for (int iter = 0; iter < seashell_watching; iter++)
    {
	if (seashell_watch[iter] == numb) {
	    get = iter;
	    break;
	}
    }

    if (get == -1) {
	return 1; //not in watch
    } else {
	seashell_watching--;
	for (int iter = get; iter < seashell_watching; iter++)
	{
	    seashell_watch[iter] = seashell_watch[iter+1];
	}
	return 0;
    }
}

/*
**Documentation**

This is the main function with which we append to the stream (seashell_io).

It handles some of the details of writing, just pass the character to append
and the cursor's negative point (to inject in the middle of the user stuff)
*/

void INTERNAL_apply(char with, int negativePos)
{
    if (seashell_fixer[with - ' '] == 0) {
	if (seashell_ioLen + 1 > io_max) {
	    INTERNAL_clearIOLines(default_linesClear);
	}

	if (seashell_inputLen < input_max) {
	    if (negativePos) {
		memmove(&(seashell_userInput[seashell_inputLen - negativePos + 1]),
			&(seashell_userInput[seashell_inputLen - negativePos]),
			negativePos);
	    }
	    seashell_userInput[seashell_inputLen - negativePos] = with;
	    seashell_inputLen++;
	
	    if (negativePos) {
		memmove(&(seashell_io[seashell_ioLen - negativePos + 1]),
			&(seashell_io[seashell_ioLen - negativePos]),
			negativePos);
	    }
	    seashell_io[seashell_ioLen - negativePos] = with;
	    seashell_ioLen++;
	}
    
	seashell_renderEvent = 1;

	if (seashell_scrolled) {
	    seashell_scrolled = 0;
	    seashell_renderEvent = 2;
	}

	seashell_fixer[with - ' '] = 1;
	if (with >= 'A' && with <= 'Z') {
	    seashell_fixer[with + 32 - ' '] = 1;
	} else if (with >= 'a' && with <= 'z') {
	    seashell_fixer[with - 32 - ' '] = 1;
	} else if (with == '!') {
	    seashell_fixer['1' - ' '] = 1;
	} else if (with == '@') {
	    seashell_fixer['2' - ' '] = 1;
	} else if (with == '#') {
	    seashell_fixer['3' - ' '] = 1;
	} else if (with == '$') {
	    seashell_fixer['4' - ' '] = 1;
	} else if (with == '%') {
	    seashell_fixer['5' - ' '] = 1;
	} else if (with == '^') {
	    seashell_fixer['6' - ' '] = 1;
	} else if (with == '&') {
	    seashell_fixer['7' - ' '] = 1;
	} else if (with == '*') {
	    seashell_fixer['8' - ' '] = 1;
	} else if (with == '(') {
	    seashell_fixer['9' - ' '] = 1;
	} else if (with == ')') {
	    seashell_fixer['0' - ' '] = 1;
	    
	} else if (with == '1') {
	    seashell_fixer['!' - ' '] = 1;
	} else if (with == '2') {
	    seashell_fixer['@' - ' '] = 1;
	} else if (with == '3') {
	    seashell_fixer['#' - ' '] = 1;
	} else if (with == '4') {
	    seashell_fixer['$' - ' '] = 1;
	} else if (with == '5') {
	    seashell_fixer['%' - ' '] = 1;
	} else if (with == '6') {
	    seashell_fixer['^' - ' '] = 1;
	} else if (with == '7') {
	    seashell_fixer['&' - ' '] = 1;
	} else if (with == '8') {
	    seashell_fixer['*' - ' '] = 1;
	} else if (with == '9') {
	    seashell_fixer['(' - ' '] = 1;
	} else if (with == '0') {
	    seashell_fixer[')' - ' '] = 1;
	}
    }
}

/*
**Documentation**

backspacing, simple as that.
*/

void INTERNAL_back(int negativePos)
{
    if (seashell_inputLen-negativePos) {
        if (seashell_inputLen > 0) {
	    if (negativePos) {
		memmove(&(seashell_userInput[seashell_inputLen - negativePos - 1]),
			&(seashell_userInput[seashell_inputLen - negativePos]),
			negativePos);
	    }

	    seashell_inputLen--;
	}
	if (seashell_ioLen > 0) {
	    if (negativePos) {
		memmove(&(seashell_io[seashell_ioLen - negativePos - 1]),
			&(seashell_io[seashell_ioLen - negativePos]),
			negativePos);
	    }
	    
	    seashell_ioLen--;
	}
    }

    seashell_renderEvent = 1;

    if (seashell_scrolled) {
	seashell_scrolled = 0;
	seashell_renderEvent = 2;
    }
}

/*
**Documentation**

applies a string to stream
*/

void INTERNAL_appendToIO(char* what)
{
    int lenAppend = strlen(what);

    while (seashell_ioLen + lenAppend > io_max) {
	INTERNAL_clearIOLines(default_linesClear);
    }

    memcpy(&(seashell_io[seashell_ioLen]), what, lenAppend);
    seashell_ioLen += lenAppend;
}

/*
**Documentation**

pushes a string to the stream one line back from top
*/

int INTERNAL_deferToIO(char* what)
{
    if (seashell_ioLen == 0) { //such as: stepping BEFORE update
	printf("Seashell Error: this should not be happening, and you should know what!\n");
    }
    
    int lenDefer = strlen(what), lenStore = 0;
    char tempStore[input_max + 12];

    while (seashell_io[seashell_ioLen-lenStore] != '\n' && seashell_ioLen-lenStore)
    {
	lenStore++;
    }
    
    if (lenStore == 0) {
        memset(seashell_io,0,io_max);
	seashell_ioLen = 0;
	INTERNAL_appendToIO("**Error: a weird string-size error occured, the view has been cleared to fix it\n");
	seashell_promptOnUpdate = 1;
    } else {
	memcpy(tempStore,&(seashell_io[seashell_ioLen-lenStore]),lenStore);

        while (seashell_ioLen + lenDefer >= io_max) {
	    INTERNAL_clearIOLines(default_linesClear);
	}
	
	memcpy(&(seashell_io[seashell_ioLen-lenStore]),what,lenDefer);
	seashell_ioLen += lenDefer;
        seashell_io[seashell_ioLen-lenStore] = '\n';
	seashell_ioLen += 1;
	memcpy(&(seashell_io[seashell_ioLen-lenStore]),tempStore,lenStore);
    }

    return 0;
}

/*
**Documentation**

comparison funcs for using with qsort and bsearch
*/

//following three needn't be seperate, but whatevs (more future/edit-safe)
int INTERNAL_compareTypes(const void* a, const void* b)
{
    seashell_type *thing0, *thing1;
    thing0 = (seashell_type*) a;
    thing1 = (seashell_type*) b;

    return strcmp(thing0->name,thing1->name);
}

int INTERNAL_compareVars(const void* a, const void* b)
{
    seashell_var *thing0, *thing1;
    thing0 = (seashell_var*) a;
    thing1 = (seashell_var*) b;

    return strcmp(thing0->name,thing1->name);
}

int INTERNAL_compareFuncs(const void* a, const void* b)
{
    seashell_func *thing0, *thing1;
    thing0 = (seashell_func*) a;
    thing1 = (seashell_func*) b;

    return strcmp(thing0->name,thing1->name);
}

/*
**Documentation**

Important functions!! These allow passing strings to return shorts to types, funcs, and also vars.
*/

short INTERNAL_getTypeByName(char* name)
{
    //if needed, do a sort (if we want to optimize this area + other get funcs, do an insertion)
    
    if (seashell_sortTypes) {
	qsort(seashell_types,seashell_typesAmnt,sizeof(seashell_type),INTERNAL_compareTypes);
	seashell_sortTypes = 0;
    }

    //search + !math for indice

    seashell_type key;
    strcpy(key.name,name);
    
    seashell_type* thing = bsearch(&key,seashell_types,seashell_typesAmnt,
				   sizeof(seashell_type),INTERNAL_compareTypes);

    if (thing == NULL) {
	return -1;
    } else {
	short loc = thing-seashell_types; //automatically does indice math (concerning!)
	return loc;
    }
}

short INTERNAL_getFuncByName(char* name)
{
    if (seashell_sortFuncs) {
	qsort(seashell_funcs,seashell_funcsAmnt,sizeof(seashell_func),INTERNAL_compareFuncs);
	seashell_sortFuncs = 0;
    }

    seashell_func key;
    strcpy(key.name,name);

    seashell_func* thing = bsearch(&key,seashell_funcs,seashell_funcsAmnt,
				   sizeof(seashell_func),INTERNAL_compareFuncs);

    if (thing == NULL) {
	return -1;
    } else {
	short loc = thing-seashell_funcs; //scury!
	return loc;
    }
}

int INTERNAL_getVarByName(char* name)
{
    if (seashell_sortVars) {
	qsort(seashell_vars,seashell_varsAmnt,sizeof(seashell_var),INTERNAL_compareFuncs);
	seashell_sortVars = 0;
    }

    seashell_var key;
    strcpy(key.name,name);

    seashell_var* thing = bsearch(&key,seashell_vars,seashell_varsAmnt,
				  sizeof(seashell_var),INTERNAL_compareVars);

    if (thing == NULL) {
	return -1;
    } else {
	short loc = thing-seashell_vars; //scurvy?!
	return loc;
    }
}

/*
**Documentation**

gets a word from a string.
*/

//inlining would be nice, we use this a lot! (if u wanna optimize, inline this if your compiler likes it)
void INTERNAL_parseWord(char* get, int* getLen, int len, char* store, int* storeLen)
{
    *storeLen = 0;
    
    //rid spaces
    
    while (get[*getLen] == ' ' && *getLen < len)
    {
        *getLen = (*getLen) + 1;
    }

    //get text

    while (get[*getLen] != ' ' && *getLen < len)
    {
        store[*storeLen] = get[*getLen];
	*storeLen = (*storeLen) + 1;
	*getLen = (*getLen) + 1;
    }
    store[*storeLen] = 0;
}

/*
**Documentation**

Don't know why I decided to functionize this.
*/

void INTERNAL_cleanFeed()
{
    seashell_feedLen = 0;
}

/*
**Documentation**

this is the event system, which allows us to parse and yet not touch the event system.
*/

void INTERNAL_feedEvent(SDL_Event with)
{
    if (seashell_feedLen < 64) {
	seashell_feedout[seashell_feedLen] = with;
	seashell_feedLen++;
    }
}

void INTERNAL_pushFeed()
{
    for (int iter = 0; iter < seashell_feedLen; iter++)
    {
	SDL_PushEvent(&(seashell_feedout[iter]));
    }
}

/*
**Documentation**

These are the io stuff, we detect if a key is not pressed and allow further presses if so.
*/

void INTERNAL_cleanFixer()
{
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_a)]) {
        seashell_fixer['A' - ' '] = 0; seashell_fixer['a' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_b)]) {
        seashell_fixer['B' - ' '] = 0; seashell_fixer['b' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_c)]) {
        seashell_fixer['C' - ' '] = 0; seashell_fixer['c' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_d)]) {
        seashell_fixer['D' - ' '] = 0; seashell_fixer['d' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_e)]) {
        seashell_fixer['E' - ' '] = 0; seashell_fixer['e' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_f)]) {
        seashell_fixer['F' - ' '] = 0; seashell_fixer['f' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_g)]) {
        seashell_fixer['G' - ' '] = 0; seashell_fixer['g' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_h)]) {
        seashell_fixer['H' - ' '] = 0; seashell_fixer['h' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_i)]) {
        seashell_fixer['I' - ' '] = 0; seashell_fixer['i' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_j)]) {
        seashell_fixer['J' - ' '] = 0; seashell_fixer['j' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_k)]) {
        seashell_fixer['K' - ' '] = 0; seashell_fixer['k' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_l)]) {
        seashell_fixer['L' - ' '] = 0; seashell_fixer['l' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_m)]) {
        seashell_fixer['M' - ' '] = 0; seashell_fixer['m' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_n)]) {
        seashell_fixer['N' - ' '] = 0; seashell_fixer['n' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_o)]) {
        seashell_fixer['O' - ' '] = 0; seashell_fixer['o' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_p)]) {
        seashell_fixer['P' - ' '] = 0; seashell_fixer['p' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_q)]) {
        seashell_fixer['Q' - ' '] = 0; seashell_fixer['q' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_r)]) {
        seashell_fixer['R' - ' '] = 0; seashell_fixer['r' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_s)]) {
	seashell_fixer['S' - ' '] = 0; seashell_fixer['s' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_t)]) {
        seashell_fixer['T' - ' '] = 0; seashell_fixer['t' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_u)]) {
        seashell_fixer['U' - ' '] = 0; seashell_fixer['u' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_v)]) {
        seashell_fixer['V' - ' '] = 0; seashell_fixer['v' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_w)]) {
        seashell_fixer['W' - ' '] = 0; seashell_fixer['w' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_x)]) {
        seashell_fixer['X' - ' '] = 0; seashell_fixer['x' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_y)]) {
        seashell_fixer['Y' - ' '] = 0; seashell_fixer['y' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_z)]) {
        seashell_fixer['Z' - ' '] = 0; seashell_fixer['z' - ' '] = 0;
    }
	
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_SPACE)]) {
        seashell_fixer[' ' - ' '] = 0;
    }

    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_BACKQUOTE)]) {
	seashell_fixer['~' - ' '] = 0; seashell_fixer['`' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_MINUS)]) {
        seashell_fixer['_' - ' '] = 0; seashell_fixer['-' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_EQUALS)]) {
	seashell_fixer['+' - ' '] = 0; seashell_fixer['=' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_LEFTBRACKET)]) {
        seashell_fixer['{' - ' '] = 0; seashell_fixer['[' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_RIGHTBRACKET)]) {
        seashell_fixer['}' - ' '] = 0; seashell_fixer[']' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_BACKSLASH)]) {
        seashell_fixer['|' - ' '] = 0; seashell_fixer['\\' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_SEMICOLON)]) {
        seashell_fixer[':' - ' '] = 0; seashell_fixer[';' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_QUOTE)]) {
        seashell_fixer['"' - ' '] = 0; seashell_fixer['\'' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_COMMA)]) {
        seashell_fixer['<' - ' '] = 0; seashell_fixer[',' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_PERIOD)] && !seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_PERIOD)]) {
        seashell_fixer['>' - ' '] = 0; seashell_fixer['.' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_SLASH)]) {
        seashell_fixer['?' - ' '] = 0; seashell_fixer['/' - ' '] = 0;
    }

    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_1)] &&
	!seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_1)]) {
        seashell_fixer['!' - ' '] = 0; seashell_fixer['1' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_2)] &&
	!seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_2)]) {
	seashell_fixer['@' - ' '] = 0; seashell_fixer['2' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_3)] &&
	!seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_3)]) {
        seashell_fixer['#' - ' '] = 0; seashell_fixer['3' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_4)] &&
	!seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_4)]) {
        seashell_fixer['$' - ' '] = 0; seashell_fixer['4' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_5)] &&
	!seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_5)]) {
	seashell_fixer['%' - ' '] = 0; seashell_fixer['5' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_6)] &&
	!seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_6)]) {
        seashell_fixer['^' - ' '] = 0; seashell_fixer['6' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_7)] &&
	!seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_7)]) {
	seashell_fixer['&' - ' '] = 0; seashell_fixer['7' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_8)] &&
	!seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_8)]) {
        seashell_fixer['*' - ' '] = 0; seashell_fixer['8' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_9)] &&
	!seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_9)]) {
        seashell_fixer['(' - ' '] = 0; seashell_fixer['9' - ' '] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_0)] &&
	!seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_0)]) {
        seashell_fixer[')' - ' '] = 0; seashell_fixer['0' - ' '] = 0;
    }
}

void INTERNAL_useShiftFixer()
{
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_a)]) {
	INTERNAL_apply('A' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_b)]) {
	INTERNAL_apply('B' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_c)]) {
	INTERNAL_apply('C' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_d)]) {
	INTERNAL_apply('D' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_e)]) {
	INTERNAL_apply('E' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_f)]) {
	INTERNAL_apply('F' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_g)]) {
	INTERNAL_apply('G' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_h)]) {
	INTERNAL_apply('H' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_i)]) {
	INTERNAL_apply('I' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_j)]) {
	INTERNAL_apply('J' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_k)]) {
	INTERNAL_apply('K' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_l)]) {
	INTERNAL_apply('L' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_m)]) {
	INTERNAL_apply('M' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_n)]) {
	INTERNAL_apply('N' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_o)]) {
	INTERNAL_apply('O' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_p)]) {
	INTERNAL_apply('P' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_q)]) {
	INTERNAL_apply('Q' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_r)]) {
	INTERNAL_apply('R' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_s)]) {
	INTERNAL_apply('S' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_t)]) {
	INTERNAL_apply('T' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_u)]) {
	INTERNAL_apply('U' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_v)]) {
	INTERNAL_apply('V' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_w)]) {
	INTERNAL_apply('W' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_x)]) {
	INTERNAL_apply('X' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_y)]) {
	INTERNAL_apply('Y' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_z)]) {
	INTERNAL_apply('Z' + 32*(!seashell_shifting),seashell_cursorNegative);
    }
	
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_SPACE)]) {
	INTERNAL_apply(' ',seashell_cursorNegative);
    }

    if (seashell_shifting) {
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_BACKQUOTE)]) {
	    INTERNAL_apply('~',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_MINUS)]) {
	    INTERNAL_apply('_',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_EQUALS)]) {
	    INTERNAL_apply('+',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_LEFTBRACKET)]) {
	    INTERNAL_apply('{',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_RIGHTBRACKET)]) {
	    INTERNAL_apply('}',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_BACKSLASH)]) {
	    INTERNAL_apply('|',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_SEMICOLON)]) {
	    INTERNAL_apply(':',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_QUOTE)]) {
	    INTERNAL_apply('\"',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_COMMA)] || seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_PERIOD)]) {
	    INTERNAL_apply('<',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_PERIOD)]) {
	    INTERNAL_apply('>',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_SLASH)]) {
	    INTERNAL_apply('?',seashell_cursorNegative);
	}
    } else {
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_BACKQUOTE)]) {
	    INTERNAL_apply('`',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_MINUS)]) {
	    INTERNAL_apply('-',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_EQUALS)]) {
	    INTERNAL_apply('=',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_LEFTBRACKET)]) {
	    INTERNAL_apply('[',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_RIGHTBRACKET)]) {
	    INTERNAL_apply(']',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_BACKSLASH)]) {
	    INTERNAL_apply('\\',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_SEMICOLON)]) {
	    INTERNAL_apply(';',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_QUOTE)]) {
	    INTERNAL_apply('\'',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_COMMA)]) {
	    INTERNAL_apply(',',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_PERIOD)] || seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_PERIOD)]) {
	    INTERNAL_apply('.',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_SLASH)]) {
	    INTERNAL_apply('/',seashell_cursorNegative);
	}
    }
    
    if (!seashell_shifting) {
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_1)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_1)]) {
	    INTERNAL_apply('1',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_2)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_2)]) {
	    INTERNAL_apply('2',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_3)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_3)]) {
	    INTERNAL_apply('3',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_4)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_4)]) {
	    INTERNAL_apply('4',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_5)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_5)]) {
	    INTERNAL_apply('5',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_6)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_6)]) {
	    INTERNAL_apply('6',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_7)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_7)]) {
	    INTERNAL_apply('7',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_8)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_8)]) {
	    INTERNAL_apply('8',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_9)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_9)]) {
	    INTERNAL_apply('9',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_0)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_0)]) {
	    INTERNAL_apply('0',seashell_cursorNegative);
	}
    } else {
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_1)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_1)]) {
	    INTERNAL_apply('!',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_2)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_2)]) {
	    INTERNAL_apply('@',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_3)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_3)]) {
	    INTERNAL_apply('#',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_4)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_4)]) {
	    INTERNAL_apply('$',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_5)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_5)]) {
	    INTERNAL_apply('%',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_6)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_6)]) {
	    INTERNAL_apply('^',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_7)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_7)]) {
	    INTERNAL_apply('&',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_8)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_8)]) {
	    INTERNAL_apply('*',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_9)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_9)]) {
	    INTERNAL_apply('(',seashell_cursorNegative);
	}
	if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_0)] ||
	    seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_0)]) {
	    INTERNAL_apply(')',seashell_cursorNegative);
	}
    }
}

/*
**Documentation**

Linux input is bad. We need a custom system to fix its shit. IKR?
*/

int INTERNAL_enter(int printThis); //damn it

char seashell_linuxFix[50];

void INTERNAL_cleanLinuxFixer()
{
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_LEFT)]) {
	seashell_linuxFix[0] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_RIGHT)]) {
	seashell_linuxFix[1] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_UP)]) {
	seashell_linuxFix[2] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_DOWN)]) {
	seashell_linuxFix[3] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_RETURN)] &&
	!seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_ENTER)]) {
	seashell_linuxFix[4] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_SPACE)]) {
	seashell_linuxFix[5] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_BACKSPACE)]) {
	seashell_linuxFix[6] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_DELETE)]) {
	seashell_linuxFix[7] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_F11)]) {
	seashell_linuxFix[8] = 0;
    }
    if (!seashell_keystate[SDL_GetScancodeFromKey(SDLK_F12)]) {
	seashell_linuxFix[9] = 0;
    }
}

void INTERNAL_doLinuxFix()
{
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_LEFT)] && !seashell_linuxFix[0]) {

	if (seashell_cursorNegative < seashell_inputLen) { seashell_cursorNegative++; seashell_renderEvent = 1; }
	
	seashell_linuxFix[0] = 1;
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_RIGHT)] && !seashell_linuxFix[1]) {

	if (seashell_cursorNegative > 0) { seashell_cursorNegative--; seashell_renderEvent = 1; }
	
	seashell_linuxFix[1] = 1;
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_UP)] && !seashell_linuxFix[2]) {

	if (seashell_controlling) {
	    seashell_scrolled++;
	    seashell_renderEvent = 2;
	} else {
	    if (seashell_recentInputSel == 0) {
		seashell_userInput[seashell_inputLen] = 0;
		memcpy(seashell_recentInput[cmds_max-1],seashell_userInput,input_max);
		seashell_recentInputSel = 1;
	    }

	    if (seashell_recentInputSel < cmds_max-1 && seashell_recentInput[cmds_max - seashell_recentInputSel - 1][0]) {
		seashell_recentInputSel++;
			    
		int len = strlen(seashell_recentInput[cmds_max - seashell_recentInputSel]);

		seashell_ioLen -= seashell_inputLen;
			    
		memcpy(seashell_userInput,seashell_recentInput[cmds_max - seashell_recentInputSel],len);
		memcpy(&seashell_io[seashell_ioLen],seashell_recentInput[cmds_max - seashell_recentInputSel],len);

		seashell_ioLen += len; seashell_inputLen = len; seashell_cursorNegative = 0;
		seashell_userInput[seashell_inputLen] = 0;

		//printf("%s\n",seashell_recentInput[cmds_max-1]);
		seashell_renderEvent = 2;
	    }
	}
	
	seashell_linuxFix[2] = 1;
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_DOWN)] && !seashell_linuxFix[3]) {

	if (seashell_controlling) {
	    if (seashell_scrolled > 0) {
		seashell_scrolled--;
		seashell_renderEvent = 2;
	    }
	} else {
	    if (seashell_recentInputSel > 0) {
		seashell_recentInputSel--;

		int len2 = strlen(seashell_recentInput[cmds_max - seashell_recentInputSel]);

		seashell_ioLen -= seashell_inputLen;
			    
		memcpy(seashell_userInput,seashell_recentInput[cmds_max - seashell_recentInputSel],len2);
		memcpy(&seashell_io[seashell_ioLen],seashell_recentInput[cmds_max - seashell_recentInputSel],len2);

		seashell_ioLen += len2; seashell_inputLen = len2; seashell_cursorNegative = 0;
		seashell_userInput[seashell_inputLen] = 0;

		seashell_renderEvent = 2;
	    }
	}
	
	seashell_linuxFix[3] = 1;
    }
    if ((seashell_keystate[SDL_GetScancodeFromKey(SDLK_RETURN)] ||
	 seashell_keystate[SDL_GetScancodeFromKey(SDLK_KP_ENTER)]) && !seashell_linuxFix[4]) {

	INTERNAL_enter(0);
	
	seashell_linuxFix[4] = 1;
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_SPACE)] && !seashell_linuxFix[5]) {

	INTERNAL_apply(' ',seashell_cursorNegative);
	
	seashell_linuxFix[5] = 1;
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_BACKSPACE)] && !seashell_linuxFix[6]) {

        INTERNAL_back(seashell_cursorNegative);
	
	seashell_linuxFix[6] = 1;
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_DELETE)] && !seashell_linuxFix[7]) {

        if (seashell_cursorNegative > 0) { seashell_cursorNegative--; INTERNAL_back(seashell_cursorNegative); }
	
	seashell_linuxFix[7] = 1;
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_F11)] && !seashell_linuxFix[8]) {

        seashell_oper = 0;
	
	seashell_linuxFix[8] = 1;
    }
    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_F12)] && !seashell_linuxFix[9]) {

        seashell_oper = 0; seashell_stepping = 0; seashell_holding = 0; INTERNAL_deferToIO("-exit-"); seashell_renderEvent = 2;
	
	seashell_linuxFix[9] = 1;
    }
}

/*
**Documentation**

Draws a graph of bytes from memory. Decided to functionize because ez.
*/

void INTERNAL_bargraph(void* from, int numb) //force numb to be <= 768 && >= 2
{
    unsigned char* pntr = from;
    int scale = (seashell_windowWidth - 4*seashell_textWidth)/numb;

    int maximum = 0;
    for (int iter = 0; iter < numb; iter++)
    {
	if (*(pntr+iter) > maximum) {
	    maximum = *(pntr+iter);
	}
    }
    int get = maximum;
    maximum += (maximum*0.1 + 1.0); //1.0 to ensure not absolute tallest in case of small values (looks bad)
    
    for (int iter = 0; iter < numb; iter++)
    {
	seashell_gBars[iter].x = seashell_textWidth*2 + iter*scale + 1;
	seashell_gBars[iter].w = scale;
	
	double measure = (*pntr + 0.0)/(maximum + 0.0);
	seashell_gBars[iter].y = (seashell_windowHeight-(14*seashell_textHeight)) - (measure*512);
	seashell_gBars[iter].h = (measure*512) + 1;

	seashell_gBarsNegative[iter] = 0;
	
	pntr++;
    }

    seashell_gBarsDraw = numb;
    seashell_gBarsHVal = get;
    seashell_gBarsHeight = (seashell_windowHeight-(14*seashell_textHeight)) - ((get + 0.0)/(maximum + 0.0))*512;
    seashell_gBarsHeightHalf = (seashell_windowHeight-(14*seashell_textHeight)) - ((get + 0.0)/(maximum + 0.0))*256;
    seashell_gBarsWidth = numb*scale + seashell_textWidth*2;
    seashell_gBarsWidthHalf = (numb/2)*scale + seashell_textWidth*2;
}

void INTERNAL_bargraphf(void* from, int numb, float scalef, int padadd)
{
    float* pntr = from;
    unsigned char *pntr1;
    int scale = (seashell_windowWidth - 4*seashell_textWidth)/numb;
    if (scalef < 0.0) {
	scalef *= -1;
    }
    float oscale = scalef;
    scalef *= 1.1;

    for (int iter = 0; iter < numb; iter++)
    {
	seashell_gBars[iter].x = seashell_textWidth*2 + iter*scale + 1;
	seashell_gBars[iter].w = scale;
	
	double measure = *pntr / scalef;
	if (measure < 0.0) {
	    measure *= -1;
	}
	if (measure > 1.0) {
	    measure = 1.0;
	}
	seashell_gBars[iter].y = (seashell_windowHeight-(14*seashell_textHeight)) - (measure*512.0);
	seashell_gBars[iter].h = (measure*512.0) + 1;

	if (*pntr < 0.0) {
	    seashell_gBarsNegative[iter] = 1;
	} else {
	    seashell_gBarsNegative[iter] = 0;
	}
	
	pntr1 = (void*) pntr;
        pntr1 += 4;
	pntr1 += padadd;
	pntr = (void*) pntr1;
    }

    seashell_gBarsDraw = numb;
    seashell_gBarsHVal = oscale;
    seashell_gBarsHeight = (seashell_windowHeight-(14*seashell_textHeight)) - (0.909)*512;
    seashell_gBarsHeightHalf = (seashell_windowHeight-(14*seashell_textHeight)) - (0.909)*256;
    seashell_gBarsWidth = numb*scale + seashell_textWidth*2;
    seashell_gBarsWidthHalf = (numb/2)*scale + seashell_textWidth*2;
}

void INTERNAL_bargraphd(void* from, int numb, double scalef, int padadd)
{
    double* pntr = from;
    unsigned char *pntr1;
    int scale = (seashell_windowWidth - 4*seashell_textWidth)/numb;
    if (scalef < 0.0) {
	scalef *= -1;
    }
    double oscale = scalef;
    scalef *= 1.1;
    
    for (int iter = 0; iter < numb; iter++)
    {
	seashell_gBars[iter].x = seashell_textWidth*2 + iter*scale + 1;
	seashell_gBars[iter].w = scale;
	
	double measure = *pntr / scalef;
	if (measure < 0.0) {
	    measure *= -1;
	}
	if (measure > 1.0) {
	    measure = 1.0;
	}
	seashell_gBars[iter].y = (seashell_windowHeight-(14*seashell_textHeight)) - (measure*512.0);
	seashell_gBars[iter].h = (measure*512.0) + 1;

	if (*pntr < 0.0) {
	    seashell_gBarsNegative[iter] = 1;
	} else {
	    seashell_gBarsNegative[iter] = 0;
	}
	
	pntr1 = (void*) pntr;
        pntr1 += 8;
	pntr1 += padadd;
	pntr = (void*) pntr1;
    }

    seashell_gBarsDraw = numb;
    seashell_gBarsHVal = oscale;
    seashell_gBarsHeight = (seashell_windowHeight-(14*seashell_textHeight)) - (0.909)*512;
    seashell_gBarsHeightHalf = (seashell_windowHeight-(14*seashell_textHeight)) - (0.909)*256;
    seashell_gBarsWidth = numb*scale + seashell_textWidth*2;
    seashell_gBarsWidthHalf = (numb/2)*scale + seashell_textWidth*2;
}

/*
**Documentation**

Just does counting, isn't complete countgraph.
*/

void INTERNAL_count(void* from, int numb)
{
    register unsigned char* pntr = from;
    register int iter;

    for (iter = 0; iter < 256; iter++)
    {
	seashell_gCount[iter] = 0;
    }

    for (iter = 0; iter < numb; iter++)
    {
	seashell_gCount[*pntr]++;
	pntr++;
    }
}

void INTERNAL_countgraph(void* from, int numb)
{
    INTERNAL_count(from,numb);

    unsigned long long tallest = 0;
    for (int iter = 0; iter < 256; iter++)
    {
	if (seashell_gCount[iter] > tallest) {
	    tallest = seashell_gCount[iter];
	}
    }

    int maximum = 0;
    for (int iter = 0; iter < 256; iter++)
    {
	if (seashell_gCount[iter]) {
	    maximum = iter;
	}
    }

    int scale = 512/maximum;
    unsigned long long get = tallest;
    tallest += (tallest*0.1 + 1.0);
    
    //printf("%llu",tallest);
    
    for (int iter = 0; iter < maximum; iter++)
    {
	seashell_gBars[iter].x = seashell_textWidth*2 + iter*scale + 1;
	seashell_gBars[iter].w = scale;

	double measure = (seashell_gCount[iter] + 0.0)/(tallest + 0.0);
	seashell_gBars[iter].y = (seashell_windowHeight-(14*seashell_textHeight)) - (measure*512);
	seashell_gBars[iter].h = (measure*512) + 1;

	seashell_gBarsNegative[iter] = 0;
    }

    seashell_gBarsDraw = maximum;
    seashell_gBarsHVal = get;
    seashell_gBarsHeight = (seashell_windowHeight-(14*seashell_textHeight)) - ((get + 0.0)/(tallest + 0.0))*512;
    seashell_gBarsHeightHalf = (seashell_windowHeight-(14*seashell_textHeight)) - ((get + 0.0)/(tallest + 0.0))*256;
    seashell_gBarsWidth = maximum*scale + seashell_textWidth*2;
    seashell_gBarsWidthHalf = (maximum/2)*scale + seashell_textWidth*2;
}

//enter function dependencies
#define cmdUnrecd "**Error: command unrecognized\n-Type 'help cmds' for a list of commands\n"
#define varUnrecd "**Error: variable identifier unrecognized\n"
void seashell_quit();

/*
**Documentation**

Writes a var based on type.
*/

void INTERNAL_printVar(seashell_var* with)
{
    int type = with->typeUse;
    int amnt = with->amnt;
    int members = seashell_types[type].members;
    int sizeTot = seashell_types[type].sizeTot;
    int sizeSpec, tempAmnt;
    char* pntr = (char*) with->pntr;
    unsigned short getUchars[8]; unsigned char tempUchar;
    char stringOut[identifier_max + 50]; //adds some padding bytes for various cmds
    char floaty = seashell_types[type].floaty[0];
    
    if (members == 1) {
	sizeSpec = seashell_types[type].sizes[0];

	switch (sizeSpec) {
	case -1:
	    if (amnt != 1) {
		sprintf(stringOut,"        00   01   02   03  ");
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut," 04   05   06   07   08   09  ");
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut," 0a   0b   0c   0d   0e   0f  ");
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut," 10 \n\n");
		INTERNAL_appendToIO(stringOut);
			    
		sprintf(stringOut,"[%04x]  ",0);
		INTERNAL_appendToIO(stringOut);
	    }
	    for (int iter = 0; iter < amnt; iter++)
	    {
		sprintf(stringOut,"%-4i ",*((char*) pntr));
		INTERNAL_appendToIO(stringOut);
		pntr += 1;
		if (iter % 17 == 16) {
		    INTERNAL_appendToIO("\n");
		    if (iter != amnt-1) {
			sprintf(stringOut,"[%04x]  ",iter+1);
			INTERNAL_appendToIO(stringOut);
		    }
		}
	    }
	    if ((amnt-1) % 17 != 16) {
		INTERNAL_appendToIO("\n");
	    }
	    break;
	case 1:
	    if (amnt != 1) {
		sprintf(stringOut,"        00  01  02  03  ");
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut,"04  05  06  07  08  09  ");
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut,"0a  0b  0c  0d  0e  0f  10 ");
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut," 11  12  13  14  15\n\n");
		INTERNAL_appendToIO(stringOut);
			    
		sprintf(stringOut,"[%04x]  ",0);
		INTERNAL_appendToIO(stringOut);
	    }
	    for (int iter = 0; iter < amnt; iter++)
	    {
		tempUchar = *((unsigned char*) pntr);
		getUchars[0] = tempUchar;
		sprintf(stringOut,"%-3hu ",getUchars[0]);
		INTERNAL_appendToIO(stringOut);
		pntr += 1;
		if (iter % 22 == 21) {
		    INTERNAL_appendToIO("\n");
		    if (iter != amnt-1) {
			sprintf(stringOut,"[%04x]  ",iter+1);
			INTERNAL_appendToIO(stringOut);
		    }
		}
	    }
	    if ((amnt-1) % 22 != 21) {
		INTERNAL_appendToIO("\n");
	    }
	    break;
	case -2:
	    if (amnt != 1) {
		sprintf(stringOut,"        [00]   [01]   ");
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut,"[02]   [03]   [04]   [05]   ");
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut,"[06]   [07]   [08]   [09]   ");
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut,"[0a]   [0b]\n\n");
		INTERNAL_appendToIO(stringOut);
			    
		sprintf(stringOut,"[%04x]  ",0);
		INTERNAL_appendToIO(stringOut);
	    }
	    for (int iter = 0; iter < amnt; iter++)
	    {
		sprintf(stringOut,"%-6hi ",*((short*) pntr));
		INTERNAL_appendToIO(stringOut);
		pntr += 2;
		if (iter % 12 == 11) {
		    INTERNAL_appendToIO("\n");
		    if (iter != amnt-1) {
			sprintf(stringOut,"[%04x]  ",iter+1);
			INTERNAL_appendToIO(stringOut);
		    }
		}
	    }
	    if ((amnt-1) % 12 != 11) {
		INTERNAL_appendToIO("\n");
	    }
	    break;
	case 2:
	    if (amnt != 1) {
		sprintf(stringOut,"        [00]  [01]  ");
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut,"[02]  [03]  [04]  [05]  ");
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut,"[06]  [07]  [08]  [09]  ");
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut,"[0a]  [0b]  [0c]  [0d]\n\n");
		INTERNAL_appendToIO(stringOut);
			    
		sprintf(stringOut,"[%04x]  ",0);
		INTERNAL_appendToIO(stringOut);
	    }
	    for (int iter = 0; iter < amnt; iter++)
	    {
		sprintf(stringOut,"%-5hu ",*((unsigned short*) pntr));
		INTERNAL_appendToIO(stringOut);
		pntr += 2;
		if (iter % 14 == 13) {
		    INTERNAL_appendToIO("\n");
		    if (iter != amnt-1) {
			sprintf(stringOut,"[%04x]  ",iter+1);
			INTERNAL_appendToIO(stringOut);
		    }
		}
	    }
	    if ((amnt-1) % 14 != 13) {
		INTERNAL_appendToIO("\n");
	    }
	    break;
	case -4:
	    if (amnt != 1) {
		sprintf(stringOut,"        [00]        [01]");
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut,"        [02]        [03]");
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut,"        [04]        [05]");
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut,"        [06]\n\n");
		INTERNAL_appendToIO(stringOut);
			    
		sprintf(stringOut,"[%04x]  ",0);
		INTERNAL_appendToIO(stringOut);
	    }
	    for (int iter = 0; iter < amnt; iter++)
	    {
		sprintf(stringOut,"%-11li ",*((int*) pntr));
		INTERNAL_appendToIO(stringOut);
		pntr += 4;
		if (iter % 7 == 6) {
		    INTERNAL_appendToIO("\n");
		    if (iter != amnt-1) {
			sprintf(stringOut,"[%04x]  ",iter+1);
			INTERNAL_appendToIO(stringOut);
		    }
		}
	    }
	    if ((amnt-1) % 7 != 6) {
		INTERNAL_appendToIO("\n");
	    }
	    break;
	case 4:
	    if (floaty) {
		if (amnt != 1) {
		  sprintf(stringOut,"[%04x]  ",0);
		    INTERNAL_appendToIO(stringOut);
		}
		for (int iter = 0; iter < amnt; iter++)
		{
		    sprintf(stringOut,"%0.6f ",*((float*) pntr));
		    INTERNAL_appendToIO(stringOut);
		    pntr += 4;
		    if (iter % 7 == 6) {
			INTERNAL_appendToIO("\n");
			if (iter != amnt-1) {
			    sprintf(stringOut,"[%04x]  ",iter+1);
			    INTERNAL_appendToIO(stringOut);
			}
		    }
		}
		if ((amnt-1) % 7 != 6) {
		    INTERNAL_appendToIO("\n");
		}
	    } else {
		if (amnt != 1) {
		    sprintf(stringOut,"        [00]       [01]");
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"       [02]       [03]");
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"       [04]       [05]");
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"       [06]\n\n");
		    INTERNAL_appendToIO(stringOut);
			    
		    sprintf(stringOut,"[%04x]  ",0);
		    INTERNAL_appendToIO(stringOut);
		}
		for (int iter = 0; iter < amnt; iter++)
		{
		    sprintf(stringOut,"%-10lu ",*((unsigned int*) pntr));
		    INTERNAL_appendToIO(stringOut);
		    pntr += 4;
		    if (iter % 7 == 6) {
			INTERNAL_appendToIO("\n");
			if (iter != amnt-1) {
			    sprintf(stringOut,"[%04x]  ",iter+1);
			    INTERNAL_appendToIO(stringOut);
			}
		    }
		}
		if ((amnt-1) % 7 != 6) {
		    INTERNAL_appendToIO("\n");
		}
	    }
	    break;
	case -8:
	    if (amnt != 1) {
		sprintf(stringOut,"        [00]                 [01]");
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut,"                 [02]                 [03]\n\n");
		INTERNAL_appendToIO(stringOut);
			    
		sprintf(stringOut,"[%04x]  ",0);
		INTERNAL_appendToIO(stringOut);
	    }
	    for (int iter = 0; iter < amnt; iter++)
	    {
		sprintf(stringOut,"%-20lli ",*((long long*) pntr));
		INTERNAL_appendToIO(stringOut);
		pntr += 8;
		if (iter % 4 == 3) {
		    INTERNAL_appendToIO("\n");
		    if (iter != amnt-1) {
			sprintf(stringOut,"[%04x]  ",iter+1);
			INTERNAL_appendToIO(stringOut);
		    }
		}
	    }
	    if ((amnt-1) % 4 != 3) {
		INTERNAL_appendToIO("\n");
	    }
	    break;
	case 8:
	    if (floaty) {
	        if (amnt != 1) {
		    sprintf(stringOut,"[%04x]   ",0);
		    INTERNAL_appendToIO(stringOut);
		}
		for (int iter = 0; iter < amnt; iter++)
		{
		    sprintf(stringOut,"%0.8f   ",*((double*) pntr));
		    INTERNAL_appendToIO(stringOut);
		    pntr += 8;
		    if (iter % 4 == 3) {
			INTERNAL_appendToIO("\n");
			if (iter != amnt-1) {
			    sprintf(stringOut,"[%04x]   ",iter+1);
			    INTERNAL_appendToIO(stringOut);
			}
		    }
		}
		if ((amnt-1) % 4 != 3) {
		    INTERNAL_appendToIO("\n");
		}
	    } else {
		if (amnt != 1) {
		    sprintf(stringOut,"        [00]                 [01]");
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"                 [02]                 [03]\n\n");
		    INTERNAL_appendToIO(stringOut);
			    
		    sprintf(stringOut,"[%04x]  ",0);
		    INTERNAL_appendToIO(stringOut);
		}
		for (int iter = 0; iter < amnt; iter++)
		{
		    sprintf(stringOut,"%-20llu ",*((unsigned long long*) pntr));
		    INTERNAL_appendToIO(stringOut);
		    pntr += 8;
		    if (iter % 4 == 3) {
			INTERNAL_appendToIO("\n");
			if (iter != amnt-1) {
			    sprintf(stringOut,"[%04x]  ",iter+1);
			    INTERNAL_appendToIO(stringOut);
			}
		    }
		}
		if ((amnt-1) % 4 != 3) {
		    INTERNAL_appendToIO("\n");
		}
	    }
	    break;
	default:
	    amnt *= abs(sizeSpec);
	    tempAmnt = amnt/8;
	    for (int iter = 0; iter = tempAmnt; iter++)
	    {
		for (int iter1 = 0; iter1 < 8; iter1++)
		{
		    tempUchar = *((unsigned char*) pntr);
		    getUchars[iter1] = tempUchar;
		    pntr += 1;
		}
		sprintf(stringOut,"%3hu %3hu %3hu %3hu %3hu %3hu %3hu %3hu \n",
			getUchars[0],getUchars[1],getUchars[2],getUchars[3],
			getUchars[4],getUchars[5],getUchars[6],getUchars[7]);
		INTERNAL_appendToIO(stringOut);
	    }
	    tempAmnt *= 8;
	    for (int iter = tempAmnt; iter < amnt; iter++)
	    {
		tempUchar = *((unsigned char*) pntr);
		getUchars[0] = tempUchar;
		pntr += 1;
		sprintf(stringOut,"%3hu ",getUchars[0]);
		INTERNAL_appendToIO(stringOut);
	    }
	    INTERNAL_appendToIO("\n");
	    break;
	}
    } else {
        unsigned long long offset;
	
	for (int iter = 0; iter < amnt; iter++)
	{
	    sprintf(stringOut,"--%04x--\n",iter);
	    INTERNAL_appendToIO(stringOut);
	    offset = 0;

	    for (int iter1 = 0; iter1 < members; iter1++)
	    {
		if (seashell_types[type].floaty[iter1] && seashell_types[type].sizes[iter1] == 4) {
		    sprintf(stringOut,"%f\n",*((float*) (pntr + offset)));
		    INTERNAL_appendToIO(stringOut);
		} else if (seashell_types[type].floaty[iter1] && seashell_types[type].sizes[iter1] == 8) {
		    sprintf(stringOut,"%f\n",*((double*) (pntr + offset)));
		    INTERNAL_appendToIO(stringOut);
		} else if (!seashell_types[type].floaty[iter1] && seashell_types[type].sizes[iter1] == -1) {
		    char get = *((char*) (pntr + offset));
		    sprintf(stringOut,"%i\n",get);
		    INTERNAL_appendToIO(stringOut);
		} else if (!seashell_types[type].floaty[iter1] && seashell_types[type].sizes[iter1] == 1) {
		    unsigned char get = *((unsigned char*) (pntr + offset));
		    unsigned short make = get;
		    sprintf(stringOut,"%hu\n",make);
		    INTERNAL_appendToIO(stringOut);
		} else if (!seashell_types[type].floaty[iter1] && seashell_types[type].sizes[iter1] == -2) {
		    short get = *((short*) (pntr + offset));
		    sprintf(stringOut,"%hi\n",get);
		    INTERNAL_appendToIO(stringOut);
		} else if (!seashell_types[type].floaty[iter1] && seashell_types[type].sizes[iter1] == 2) {
		    unsigned short get = *((unsigned short*) (pntr + offset));
		    sprintf(stringOut,"%hu\n",get);
		    INTERNAL_appendToIO(stringOut);
		} else if (!seashell_types[type].floaty[iter1] && seashell_types[type].sizes[iter1] == -4) {
		    int get = *((int*) (pntr + offset));
		    sprintf(stringOut,"%li\n",get);
		    INTERNAL_appendToIO(stringOut);
		} else if (!seashell_types[type].floaty[iter1] && seashell_types[type].sizes[iter1] == 4) {
		    unsigned int get = *((unsigned int*) (pntr + offset));
		    sprintf(stringOut,"%lu\n",get);
		    INTERNAL_appendToIO(stringOut);
		} else if (!seashell_types[type].floaty[iter1] && seashell_types[type].sizes[iter1] == -8) {
		    long long get = *((long long*) (pntr + offset));
		    sprintf(stringOut,"%lli\n",get);
		    INTERNAL_appendToIO(stringOut);
		} else if (!seashell_types[type].floaty[iter1] && seashell_types[type].sizes[iter1] == 8) {
		    unsigned long long get = *((unsigned long long*) (pntr + offset));
		    sprintf(stringOut,"%llu\n",get);
		    INTERNAL_appendToIO(stringOut);
		} else {
		    for (int iter2 = 0; iter2 < abs(seashell_types[type].sizes[iter1]); iter2++)
		    {
			sprintf(stringOut,"%02x ",*(pntr + offset + iter2));
			INTERNAL_appendToIO(stringOut);
		    }
		}

		offset += abs(seashell_types[type].sizes[iter1]);
	    }

	    pntr += seashell_types[type].sizeTot;
	}
    }
}

/*
**Documentation**

This is what happens when enter is pressed. Essentially the major proportion of the program.
*/

//MAIN INTERACTION IN DIS ONE
int INTERNAL_enter(int printThis)
{
    //newline + null-term stuff

    if (seashell_inputLen < input_max) {
	seashell_io[seashell_ioLen] = '\n';
	seashell_ioLen++;
    }

    seashell_userInput[seashell_inputLen] = 0;

    //print out the thing

    if (printThis == 1) {
	INTERNAL_appendToIO("@seashell~ ");
	INTERNAL_appendToIO(seashell_userInput);
	INTERNAL_appendToIO("\n");
    }
    
    //store ish

    if (seashell_inputLen) {
	memmove(seashell_recentInput[0],seashell_recentInput[1],(cmds_max-2) * input_max);
	memcpy(seashell_recentInput[cmds_max-2],seashell_userInput,input_max);
	seashell_recentInputSel = 0;
    }
    
    // *parse + exec command

    int getLen = 0, altLen = 0, numbGet, dontPrompt = 0;
    char stringOut[identifier_max + 50]; //adds some padding bytes for various cmds

    char forceNoNewline = 0;

    while (seashell_userInput[getLen] == ' ' && getLen < seashell_inputLen)
    {
	getLen++;
    }
    while (seashell_userInput[getLen] != ' ' && getLen < seashell_inputLen) {
	seashell_stringGet[altLen] = seashell_userInput[getLen];
        altLen++;getLen++;
    }
    seashell_stringGet[altLen] = 0;
    altLen = 0; //just to be safe

    if (getLen) {
	INTERNAL_appendToIO("\n");
    }

    switch (seashell_stringGet[0]) {
    case 'b':
	if (strcmp(seashell_stringGet,"bargraph") == 0 || strcmp(seashell_stringGet,"bg") == 0) {
	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {
	        INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

		int bytes = 1;
		if (sscanf(seashell_stringGet,"%i",&bytes) == 0 && altLen != 0) {
		    INTERNAL_appendToIO("**Error: cannot read inputted byte value\n");
		} else {
		    if (altLen == 0) {
			bytes = seashell_types[seashell_vars[numbGet].typeUse].sizeTot * seashell_vars[numbGet].amnt;
		    }

		    if (bytes < 2) {
			INTERNAL_appendToIO("*Warning: bargraph doesn't support less than 2 bars, defaulting to 2 bytes\n");
			bytes = 2;
		    }

		    
		    if (bytes > 736) {
			INTERNAL_appendToIO("*Warning: bargraph doesn't support more than 736 bars, defaulting to 736 bytes\n");
			bytes = 736;
		    }

		    INTERNAL_bargraph(seashell_vars[numbGet].pntr,bytes);
		    seashell_renderEvent = 2;
		    seashell_renderVisualMode = 0;
		}
	    }
	} else if (strcmp(seashell_stringGet,"bargraphf") == 0 || strcmp(seashell_stringGet,"bgf") == 0) {
	    
	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {
	        INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

		int passTake = 1, pass;
		if (sscanf(seashell_stringGet,"%i",&passTake) == 0 || altLen == 0) {
		    INTERNAL_appendToIO("**Error: cannot read desired number of bars\n");
		} else {
		    pass = passTake;

		    if (pass > 736) {
			INTERNAL_appendToIO("*Warning: bargraph doesn't support more than 736 bars, defaulting to 736\n");
			pass = 736;
		    }

		    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
		    
		    float scaleTake, scale;
		    if (sscanf(seashell_stringGet,"%f",&scaleTake) == 0 || altLen == 0) {
			INTERNAL_appendToIO("**Error: cannot read desired max scale\n");
		    } else {
			scale = scaleTake;

		        INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
			
			int padTake, pad;
			if (sscanf(seashell_stringGet,"%i",&padTake) == 0 && altLen != 0) {
			    INTERNAL_appendToIO("**Error: cannot read padding\n");
			} else {
			    if (altLen == 0) {
				pad = 0;
			    } else {
				pad = padTake;
			    }
			    
			    INTERNAL_bargraphf(seashell_vars[numbGet].pntr,pass,scale,pad);
			    seashell_renderEvent = 2;
			    seashell_renderVisualMode = 0;
			}
		    }
		}
	    }
	} else if (strcmp(seashell_stringGet,"bargraphd") == 0 || strcmp(seashell_stringGet,"bgd") == 0) {
	    
	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {
	        INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

		int passTake = 1, pass;
		if (sscanf(seashell_stringGet,"%i",&passTake) == 0 || altLen == 0) {
		    INTERNAL_appendToIO("**Error: cannot read desired number of bars\n");
		} else {
		    pass = passTake;

		    if (pass > 736) {
			INTERNAL_appendToIO("*Warning: bargraph doesn't support more than 736 bars, defaulting to 736\n");
			pass = 736;
		    }

		    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
		    
		    float scaleTake, scale;
		    if (sscanf(seashell_stringGet,"%f",&scaleTake) == 0 || altLen == 0) {
			INTERNAL_appendToIO("**Error: cannot read desired max scale\n");
		    } else {
			scale = scaleTake;

		        INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
			
			int padTake, pad;
			if (sscanf(seashell_stringGet,"%i",&padTake) == 0 && altLen != 0) {
			    INTERNAL_appendToIO("**Error: cannot read padding\n");
			} else {
			    if (altLen == 0) {
				pad = 0;
			    } else {
				pad = padTake;
			    }
			    
			    INTERNAL_bargraphd(seashell_vars[numbGet].pntr,pass,(double) scale,pad);
			    seashell_renderEvent = 2;
			    seashell_renderVisualMode = 0;
			}
		    }
		}
	    }
	} else {
	    INTERNAL_appendToIO(cmdUnrecd);
	}
	break;
    case 'c':
	if (strcmp(seashell_stringGet,"call") == 0 || strcmp(seashell_stringGet,"c") == 0) {
	    //TESTING HARDCORE

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getFuncByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO("**Error: function identifier unrecognized\n");
	    } else {
		int varGet, err = 0, pntr;
		
		for (int iter = 0; iter < seashell_funcs[numbGet].args; iter++)
		{
		    altLen = 0;
		    while (seashell_userInput[getLen] == ' ' && getLen < seashell_inputLen)
		    {
			getLen++;
		    }
		    if (seashell_userInput[getLen] == '&' && seashell_funcs[numbGet].argTypes[iter] == INTERNAL_getTypeByName("pointer")) {
			getLen += 1;
			pntr = 1;
		    } else {
			pntr = 0;
		    }
		    while (seashell_userInput[getLen] != ' ' && getLen < seashell_inputLen)
		    {
			seashell_stringGet[altLen] = seashell_userInput[getLen];
			altLen++; getLen++;
		    }
		    seashell_stringGet[altLen] = 0;

		    varGet = INTERNAL_getVarByName(seashell_stringGet);
		    
		    if (varGet == -1 && altLen == 0) {
		        INTERNAL_appendToIO("**Error: missing arguments, call 'desc <func>' to list information.\n");
			err = 1;
			break;
		    } else if (varGet == -1) {
			int point = 0;
		        for (int loc = 0; loc < altLen; loc++)
			{
			    if (seashell_stringGet[loc] == '.') {
				point = 1;
			    }
			}
			
			//repetitive? (not that I really care, people who harp on that are insane)
			
			if (seashell_types[seashell_funcs[numbGet].argTypes[iter]].members != 1) {
			    sprintf(stringOut,"**Error: cannot read argument %i, of array variety (passing arrays is unsupported).\n",iter+1);
			    INTERNAL_appendToIO(stringOut);
			    err = 1;
			    break;
			} else {
			    if (point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 8 &&
				seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
			        float get = 0.0;
				if (sscanf(seashell_stringGet,"%f",&get) == 0) {
				    sprintf(stringOut,"**Error: cannot read float argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    double thing = get;
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&thing,8);
				}
			    } else if (point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 4 &&
				       seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				float get = 0.0;
				if (sscanf(seashell_stringGet,"%f",&get) == 0) {
				    sprintf(stringOut,"**Error: cannot read float argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,4);
				}
			    } else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 8 &&
				       !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				unsigned long long get;
				if (sscanf(seashell_stringGet,"%llu",&get) == 0) {
				    sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,8);
				}
			    } else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == -8 &&
				       !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				long long get;
				if (sscanf(seashell_stringGet,"%lli",&get) == 0) {
				    sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,8);
				}
			    } else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 4 &&
				       !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				unsigned long get;
				if (sscanf(seashell_stringGet,"%lu",&get) == 0) {
				    sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,4);
				}
			    } else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == -4 &&
				       !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				long get;
				if (sscanf(seashell_stringGet,"%li",&get) == 0) {
				    sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,4);
				}
			    } else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 2 &&
				       !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				unsigned short get;
				if (sscanf(seashell_stringGet,"%hu",&get) == 0) {
				    sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,2);
				}
			    } else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == -2 &&
				       !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				short get;
				if (sscanf(seashell_stringGet,"%hi",&get) == 0) {
				    sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,2);
				}
			    } else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 1 &&
				       !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				unsigned short get;
				if (sscanf(seashell_stringGet,"%hu",&get) == 0) { //intentional shorts here
				    sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,1);
				}
			    } else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == -1 &&
				       !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				short get;
				if (sscanf(seashell_stringGet,"%hi",&get) == 0) { //same as above
				    sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,1);
				}
			    } else {
				sprintf(stringOut,"**Error: cannot read argument %i, of unsupported size for constant pass and unrecognized as variable.\n",iter+1);
				INTERNAL_appendToIO(stringOut);
				err = 1;
				break;
			    }
			}
		    } else {
			//typecheck

			if (seashell_vars[varGet].typeUse != seashell_funcs[numbGet].argTypes[iter] &&
			    (!pntr && seashell_funcs[numbGet].argTypes[iter] == INTERNAL_getTypeByName("pointer")) ) {
			    sprintf(stringOut,"**Error: argument %i of incorrect type, should be '%s'\n",iter+1,seashell_types[seashell_funcs[numbGet].argTypes[iter]].name);
			    INTERNAL_appendToIO(stringOut);
			    err = 1;
			    break;
			}
			
			//input

			if (pntr) {
			    memcpy(&seashell_args[iter],&seashell_vars[varGet],sizeof(seashell_var));
			    seashell_args[iter].pntr = seashell_defaultArgs[iter];
			    memcpy(seashell_args[iter].pntr,&(seashell_vars[varGet].pntr),8);
			} else {
			    memcpy(&seashell_args[iter],&seashell_vars[varGet],sizeof(seashell_var));
			    if (seashell_vars[varGet].amnt * seashell_types[seashell_vars[varGet].typeUse].sizeTot <= 8) {
				seashell_args[iter].pntr = seashell_defaultArgs[iter];
				memcpy(seashell_args[iter].pntr,seashell_vars[varGet].pntr,
				       seashell_vars[varGet].amnt * seashell_types[seashell_vars[varGet].typeUse].sizeTot);
			    }
			}
		    }
		}

	        seashell_return.typeUse = seashell_funcs[numbGet].typeReturn;
		
		if (!err) {
		    INTERNAL_funcCall(seashell_vars,seashell_funcs,seashell_types,numbGet,&seashell_return,seashell_args);

		    if (seashell_funcs[numbGet].typeReturn != -1) {
			if (seashell_return.typeUse == INTERNAL_getTypeByName("pointer")) {
			    sprintf(stringOut,"(pointer) %llu\n",((unsigned long long*) seashell_return.pntr));
			    INTERNAL_appendToIO(stringOut);
			} else {
			    INTERNAL_printVar(&seashell_return);
			}
		    }
		}
	    }
	} else if (strcmp(seashell_stringGet,"callret") == 0 || strcmp(seashell_stringGet,"cr") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

	    int retLoc = INTERNAL_getVarByName(seashell_stringGet);

	    if (retLoc == -1) {
		INTERNAL_appendToIO("**Error: return identifier unrecognized\n");	
	    } else {

		INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
		numbGet = INTERNAL_getFuncByName(seashell_stringGet);

		if (numbGet == -1) {
		    INTERNAL_appendToIO("**Error: function identifier unrecognized\n");
		} else if (seashell_vars[retLoc].typeUse != seashell_funcs[numbGet].typeReturn) {
		    sprintf(stringOut,"**Error: return variable is incorrect type, should be '%s'\n",seashell_types[seashell_funcs[numbGet].typeReturn].name);
		    INTERNAL_appendToIO(stringOut);
		} else if (seashell_vars[retLoc].typeUse == -1) {
		    sprintf(stringOut,"**Error: cannot store return from function with void return value\n");
		    INTERNAL_appendToIO(stringOut);
		} else {
		    int varGet, err = 0, pntr;
		
		    for (int iter = 0; iter < seashell_funcs[numbGet].args; iter++)
		    {
			altLen = 0;
			while (seashell_userInput[getLen] == ' ' && getLen < seashell_inputLen)
			{
			    getLen++;
			}
			if (seashell_userInput[getLen] == '&' && seashell_funcs[numbGet].argTypes[iter] == INTERNAL_getTypeByName("pointer")) {
			    getLen += 1;
			    pntr = 1;
			} else {
			    pntr = 0;
			}
			while (seashell_userInput[getLen] != ' ' && getLen < seashell_inputLen)
			{
			    seashell_stringGet[altLen] = seashell_userInput[getLen];
			    altLen++; getLen++;
			}
			seashell_stringGet[altLen] = 0;

			varGet = INTERNAL_getVarByName(seashell_stringGet);
		    
			if (varGet == -1 && altLen == 0) {
			    INTERNAL_appendToIO("**Error: missing arguments, call 'desc <func>' to list information.\n");
			    err = 1;
			    break;
			} else if (varGet == -1) {
			    int point = 0;
			    for (int loc = 0; loc < altLen; loc++)
			    {
				if (seashell_stringGet[loc] == '.') {
				    point = 1;
				}
			    }

			    //repetitive? (not that I really care, people who harp on that are insane)
			
			    if (seashell_types[seashell_funcs[numbGet].argTypes[iter]].members != 1) {
				sprintf(stringOut,"**Error: cannot read argument %i, of array variety (passing arrays is unsupported).\n",iter+1);
				INTERNAL_appendToIO(stringOut);
				err = 1;
				break;
			    } else {
				if (point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 8 &&
				    seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    float get = 0.0;
				    if (sscanf(seashell_stringGet,"%f",&get) == 0) {
					sprintf(stringOut,"**Error: cannot read float argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					double thing = get;
					seashell_args[iter].pntr = seashell_defaultArgs[iter];
					memcpy(seashell_args[iter].pntr,&thing,8);
				    }
				} else if (point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 4 &&
					   seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    float get = 0.0;
				    if (sscanf(seashell_stringGet,"%f",&get) == 0) {
					sprintf(stringOut,"**Error: cannot read float argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					seashell_args[iter].pntr = seashell_defaultArgs[iter];
					memcpy(seashell_args[iter].pntr,&get,4);
				    }
				} else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 8 &&
					   !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    unsigned long long get;
				    if (sscanf(seashell_stringGet,"%llu",&get) == 0) {
					sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					seashell_args[iter].pntr = seashell_defaultArgs[iter];
					memcpy(seashell_args[iter].pntr,&get,8);
				    }
				} else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == -8 &&
					   !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    long long get;
				    if (sscanf(seashell_stringGet,"%lli",&get) == 0) {
					sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					seashell_args[iter].pntr = seashell_defaultArgs[iter];
					memcpy(seashell_args[iter].pntr,&get,8);
				    }
				} else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 4 &&
					   !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    unsigned long get;
				    if (sscanf(seashell_stringGet,"%lu",&get) == 0) {
					sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					seashell_args[iter].pntr = seashell_defaultArgs[iter];
					memcpy(seashell_args[iter].pntr,&get,4);
				    }
				} else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == -4 &&
					   !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    long get;
				    if (sscanf(seashell_stringGet,"%li",&get) == 0) {
					sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					seashell_args[iter].pntr = seashell_defaultArgs[iter];
					memcpy(seashell_args[iter].pntr,&get,4);
				    }
				} else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 2 &&
					   !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    unsigned short get;
				    if (sscanf(seashell_stringGet,"%hu",&get) == 0) {
					sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					seashell_args[iter].pntr = seashell_defaultArgs[iter];
					memcpy(seashell_args[iter].pntr,&get,2);
				    }
				} else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == -2 &&
					   !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    short get;
				    if (sscanf(seashell_stringGet,"%hi",&get) == 0) {
					sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
				    } else {
					seashell_args[iter].pntr = seashell_defaultArgs[iter];
					memcpy(seashell_args[iter].pntr,&get,2);
				    }
				} else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 1 &&
					   !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    unsigned short get;
				    if (sscanf(seashell_stringGet,"%hu",&get) == 0) { //intentional shorts here
					sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					seashell_args[iter].pntr = seashell_defaultArgs[iter];
					memcpy(seashell_args[iter].pntr,&get,1);
				    }
				} else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == -1 &&
					   !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    short get;
				    if (sscanf(seashell_stringGet,"%hi",&get) == 0) { //same as above
					sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					seashell_args[iter].pntr = seashell_defaultArgs[iter];
					memcpy(seashell_args[iter].pntr,&get,1);
				    }
				} else {
				    sprintf(stringOut,"**Error: cannot read argument %i, of unsupported size for constant pass and unrecognized as variable.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				}
			    }
			} else {
			    //typecheck

			    if (seashell_vars[varGet].typeUse != seashell_funcs[numbGet].argTypes[iter] &&
				(!pntr && seashell_funcs[numbGet].argTypes[iter] == INTERNAL_getTypeByName("pointer")) ) {
				sprintf(stringOut,"**Error: argument %i of incorrect type, should be '%s'\n",iter+1,seashell_types[seashell_funcs[numbGet].argTypes[iter]].name);
				INTERNAL_appendToIO(stringOut);
				err = 1;
				break;
			    }
			
			    //input

			    if (pntr) {
				memcpy(&seashell_args[iter],&seashell_vars[varGet],sizeof(seashell_var));
				seashell_args[iter].pntr = seashell_defaultArgs[iter];
				memcpy(seashell_args[iter].pntr,&(seashell_vars[varGet].pntr),8);
			    } else {
				memcpy(&seashell_args[iter],&seashell_vars[varGet],sizeof(seashell_var));
				if (seashell_vars[varGet].amnt * seashell_types[seashell_vars[varGet].typeUse].sizeTot <= 8) {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,seashell_vars[varGet].pntr,
					   seashell_vars[varGet].amnt * seashell_types[seashell_vars[varGet].typeUse].sizeTot);
				}
			    }
			}
		    }

		    seashell_return.typeUse = seashell_funcs[numbGet].typeReturn;
		
		    if (!err) {
			INTERNAL_funcCall(seashell_vars,seashell_funcs,seashell_types,numbGet,&seashell_return,seashell_args);

		        int bytesMeasure = seashell_vars[retLoc].amnt * seashell_types[seashell_vars[retLoc].typeUse].sizeTot;
			int temp = seashell_return.amnt * seashell_types[seashell_return.typeUse].sizeTot;
			    
			if (bytesMeasure > temp) {
			    bytesMeasure = temp;
			}

			if (seashell_return.typeUse == INTERNAL_getTypeByName("pointer")) {
			    memcpy(&seashell_vars[retLoc].pntr,seashell_return.pntr,8);
			} else {
			    memcpy(seashell_vars[retLoc].pntr,seashell_return.pntr,bytesMeasure);
			}
		    }
		}
	    }
	} else if (strcmp(seashell_stringGet,"clear") == 0 || strcmp(seashell_stringGet,"cl") == 0) {
	    memset(seashell_io,0,io_max);
	    seashell_ioLen = 0;
	    seashell_renderVisualMode = -1;
	    seashell_watching = 0;
	} else if (strcmp(seashell_stringGet,"cleargraph") == 0 || strcmp(seashell_stringGet,"clg") == 0) {
	    seashell_renderVisualMode = -1;
	} else if (strcmp(seashell_stringGet,"clearwatch") == 0 || strcmp(seashell_stringGet,"clw") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1 && altLen != 0) {
		INTERNAL_appendToIO(varUnrecd);
	    } else if (altLen == 0) {
		seashell_watching = 0;
	    } else {
		if (INTERNAL_removeWatch(numbGet)) {
		    INTERNAL_appendToIO("**Error: variable not in watch\n");
		}
	    }
	} else if (strcmp(seashell_stringGet,"count") == 0 || strcmp(seashell_stringGet,"cnt") == 0) {
	    
	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {
		INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

		int bytes = 2;
		if (sscanf(seashell_stringGet,"%i",&bytes) == 0 && altLen != 0) {
		    INTERNAL_appendToIO("**Error: cannot read inputted byte value\n");
		} else {
		    if (altLen == 0) {
			bytes = seashell_types[seashell_vars[numbGet].typeUse].sizeTot * seashell_vars[numbGet].amnt;
		    }

		    INTERNAL_count(seashell_vars[numbGet].pntr,bytes);

		    int elipsed = 0;
		    
		    for (int iter = 0; iter < 256; iter++)
		    {
			if (seashell_gCount[iter] != 0) {
			    sprintf(stringOut,"[%02x, %03i]  %llu\n",iter,iter,seashell_gCount[iter]);
			    INTERNAL_appendToIO(stringOut);
			    elipsed = 0;
			} else if (!elipsed) {
			    INTERNAL_appendToIO("...\n");
			    elipsed = 1;
			}
		    }
		    
		    seashell_renderEvent = 2;
		}
	    }
	    
	} else if (strcmp(seashell_stringGet,"countgraph") == 0 || strcmp(seashell_stringGet,"ctg") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {

		INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

		int bytes = 2;
		if (sscanf(seashell_stringGet,"%i",&bytes) == 0 && altLen != 0) {
		    INTERNAL_appendToIO("**Error: cannot read inputted byte value\n");
		} else {
		    if (altLen == 0) {
			bytes = seashell_types[seashell_vars[numbGet].typeUse].sizeTot * seashell_vars[numbGet].amnt;
		    }

		    INTERNAL_countgraph(seashell_vars[numbGet].pntr,bytes);
		    seashell_renderEvent = 2;
		    seashell_renderVisualMode = 0;
		}
	    }
	} else if (strcmp(seashell_stringGet,"copy") == 0 || strcmp(seashell_stringGet,"cp") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {
		seashell_varClipboard = numbGet;
	    }
	} else {
	    INTERNAL_appendToIO(cmdUnrecd);
	}
	break;
    case 'd':
	if (strcmp(seashell_stringGet,"desc") == 0 || strcmp(seashell_stringGet,"dsc") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		numbGet = INTERNAL_getFuncByName(seashell_stringGet);

		if (numbGet == -1) {
		    numbGet = INTERNAL_getTypeByName(seashell_stringGet);

		    if (numbGet == -1) {
			INTERNAL_appendToIO("**Error: unrecognized identifier, cannot find var/func/type\n");
		    } else { //is type
			sprintf(stringOut,"Name: %s\n",seashell_types[numbGet].name);
			INTERNAL_appendToIO(stringOut);
			sprintf(stringOut,"Mbrs: %i\n",seashell_types[numbGet].members);
			INTERNAL_appendToIO(stringOut);

			for (int iter = 0; iter < seashell_types[numbGet].members; iter++)
			{
			    sprintf(stringOut,"-%03i-\n",iter);
			    INTERNAL_appendToIO(stringOut);
			    sprintf(stringOut,"  Size: %i\n",abs(seashell_types[numbGet].sizes[iter]));
			    INTERNAL_appendToIO(stringOut);
			    sprintf(stringOut,"  FltP: %i\n",seashell_types[numbGet].floaty[iter]);
			    INTERNAL_appendToIO(stringOut);
			}
		        sprintf(stringOut,"-end-\n");
			INTERNAL_appendToIO(stringOut);

			if (seashell_types[numbGet].sizeTot != 1) {
			    sprintf(stringOut,"Totl: %i bytes\n",seashell_types[numbGet].sizeTot);
			    INTERNAL_appendToIO(stringOut);
			} else {
			    sprintf(stringOut,"Totl: 1 byte\n");
			    INTERNAL_appendToIO(stringOut);
			}
		    }
		} else { //is func
		    sprintf(stringOut,"Name: %s\n",seashell_funcs[numbGet].name);
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"Args: %i\n",seashell_funcs[numbGet].args);
		    INTERNAL_appendToIO(stringOut);

		    for (int iter = 0; iter < seashell_funcs[numbGet].args; iter++)
		    {
			sprintf(stringOut,"-%03i-\n",iter);
			INTERNAL_appendToIO(stringOut);
			sprintf(stringOut,"  Type: %s\n",seashell_types[seashell_funcs[numbGet].argTypes[iter]].name);
			INTERNAL_appendToIO(stringOut);
			sprintf(stringOut,"  Amnt: %i\n",seashell_funcs[numbGet].argAmnts[iter]);
			INTERNAL_appendToIO(stringOut);
		    }
		    if (seashell_funcs[numbGet].args) {
		        sprintf(stringOut,"-end-\n");
			INTERNAL_appendToIO(stringOut);
		    }

		    if (seashell_funcs[numbGet].typeReturn != -1) {
			sprintf(stringOut,"Rets: %s\n",seashell_types[seashell_funcs[numbGet].typeReturn].name);
			INTERNAL_appendToIO(stringOut);
		    } else {
			sprintf(stringOut,"Rets: void\n");
			INTERNAL_appendToIO(stringOut);
		    }
		    unsigned long long tempPntr;
		    memcpy(&tempPntr,&seashell_funcs[numbGet].pntr,8);
		    sprintf(stringOut,"Locl: %llu\n",tempPntr);
		    INTERNAL_appendToIO(stringOut);
		}
	    } else { //is var
		sprintf(stringOut,"Name: %s\n",seashell_vars[numbGet].name);
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut,"Type: %s\n",seashell_types[ seashell_vars[numbGet].typeUse ].name);
		INTERNAL_appendToIO(stringOut);
		sprintf(stringOut,"Amnt: %i\n",seashell_vars[numbGet].amnt);
		INTERNAL_appendToIO(stringOut);

		unsigned long long tempPntr;
		memcpy(&tempPntr,&seashell_vars[numbGet].pntr,8);
		sprintf(stringOut,"Locl: %llu\n",tempPntr);
		INTERNAL_appendToIO(stringOut);
	    }
	} else {
	    INTERNAL_appendToIO(cmdUnrecd);
	}
	break;
    case 'h': case '?':
	if (strcmp(seashell_stringGet,"holdit") == 0 || strcmp(seashell_stringGet,"hi") == 0) {
	    seashell_holding = 1;
	} else if (strcmp(seashell_stringGet,"help") == 0 || strcmp(seashell_stringGet,"?") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

	    if (altLen == 0) {
		INTERNAL_appendToIO("---Seashell---\n\nAn integrable shell environment for developing for, visualizing, experimenting with, and understanding C/C++ programs at runtime. Jeez that is a mouthful... let's just say it's pretty fricken sweet.\n\nDesigned and Implemented by Grant T (@eternal_salmon)\n\n-Type 'help cmds' for a list of available comands, type 'help <cmd>' for the description of a specific command.\n");
	    } else if (strcmp(seashell_stringGet,"cmds") == 0) {
		INTERNAL_appendToIO("print      scan       prints     scans      printx     scanx      printnm    scannm\n");
		INTERNAL_appendToIO("move       copy       paste      desc       call       callret    timedcall  clear\n");
		INTERNAL_appendToIO("holdit     stepit     timedhold  timedstep  bargraph   bargraphf  bargraphd  countgraph\n");
		INTERNAL_appendToIO("cleargraph watch      clearwatch sys        list       log        output     input\n");
		INTERNAL_appendToIO("modtype    modname    modamnt    hexadd     quit       quitall    count      script\n");
		INTERNAL_appendToIO("ocall      help\n\n");
		INTERNAL_appendToIO("-Type 'help <cmd>' for a more specific description, like args and what it does.\n");
	    } else if (strcmp(seashell_stringGet,"print") == 0 || strcmp(seashell_stringGet,"p") == 0) {
		INTERNAL_appendToIO("print <var>               - prints out value of declared variable <var> with formatting (== p).\n");
	    } else if (strcmp(seashell_stringGet,"scan") == 0 || strcmp(seashell_stringGet,"s") == 0) {
		INTERNAL_appendToIO("scan <var> <val>          - writes in decimal <val> to variable <var>, limited to 8 bytes (== s).\n");
	    } else if (strcmp(seashell_stringGet,"prints") == 0 || strcmp(seashell_stringGet,"ps") == 0) {
		INTERNAL_appendToIO("prints <var>              - prints out <var> with string treatment, as in, ascii (== ps).\n");
	    } else if (strcmp(seashell_stringGet,"scans") == 0 || strcmp(seashell_stringGet,"sst") == 0) {
		INTERNAL_appendToIO("scans <var> <str>         - writes strlen(<str>) bytes into <var>, has C formatting ('\\n') (== sst).\n");
	    } else if (strcmp(seashell_stringGet,"printx") == 0 || strcmp(seashell_stringGet,"px") == 0) {
		INTERNAL_appendToIO("printx <var>              - prints out <var> byte-by-byte in hex format (== px).\n");
	    } else if (strcmp(seashell_stringGet,"scanx") == 0 || strcmp(seashell_stringGet,"sx") == 0) {
		INTERNAL_appendToIO("scanx <var> <off> <val>   - writes hex byte <val> into <var> with hex offset <off> in bytes (== sx).\n");
	    } else if (strcmp(seashell_stringGet,"printnm") == 0 || strcmp(seashell_stringGet,"pnm") == 0) {
		INTERNAL_appendToIO("printnm <v> <n> <m>       - prints out <v>[<n>]'s <m>th member (== pnm).\n");
	    } else if (strcmp(seashell_stringGet,"scannm") == 0 || strcmp(seashell_stringGet,"snm") == 0) {
		INTERNAL_appendToIO("scannm <v> <n> <m> <w>    - writes <w> into <v>[<n>]'s <m>th member, up to 8 bytes (== snm).\n");
	    } else if (strcmp(seashell_stringGet,"move") == 0 || strcmp(seashell_stringGet,"mv") == 0) {
		INTERNAL_appendToIO("move <v0> <v1>            - writes <v1> into <v0>, stopping at the smaller size (== mv).\n");
	    } else if (strcmp(seashell_stringGet,"copy") == 0 || strcmp(seashell_stringGet,"cp") == 0) {
		INTERNAL_appendToIO("copy <var>                - copies <var> into a buffer, 'paste' writes that to other vars (== cp).\n");
	    } else if (strcmp(seashell_stringGet,"paste") == 0 || strcmp(seashell_stringGet,"pst") == 0) {
		INTERNAL_appendToIO("paste <var>               - pastes buffer into <var>, 'copy' modifies said buffer instead (== pst).\n");
	    } else if (strcmp(seashell_stringGet,"desc") == 0 || strcmp(seashell_stringGet,"dsc") == 0) {
		INTERNAL_appendToIO("desc <var/func/type>      - describe the var, type, or func (shows size, arguments, etc.) (== dsc).\n");
	    } else if (strcmp(seashell_stringGet,"call") == 0 || strcmp(seashell_stringGet,"c") == 0) {
		INTERNAL_appendToIO("call <func> <aN> ...      - call <func> with defined arguments/constants (== c).\n");
	    } else if (strcmp(seashell_stringGet,"callret") == 0 || strcmp(seashell_stringGet,"cr") == 0) {
		INTERNAL_appendToIO("callret <var> <func> <aN> - call <func> with defined arguments/constants and store return into <var> (== cr).\n");
	    } else if (strcmp(seashell_stringGet,"timedcall") == 0 || strcmp(seashell_stringGet,"tc") == 0) {
		INTERNAL_appendToIO("timedcall <t> <fn> <a>    - call <fn> with defined arguments/constants <t> times, show timings (== tc).\n");
	    } else if (strcmp(seashell_stringGet,"clear") == 0 || strcmp(seashell_stringGet,"cl") == 0) {
		INTERNAL_appendToIO("clear                     - clear all text, graphs, and watches (== cl).\n");
	    } else if (strcmp(seashell_stringGet,"cleargraph") == 0 || strcmp(seashell_stringGet,"clg") == 0) {
		INTERNAL_appendToIO("cleargraph                - clear the graph at the top from view (== clg).\n");
	    } else if (strcmp(seashell_stringGet,"holdit") == 0 || strcmp(seashell_stringGet,"hi") == 0) {
		INTERNAL_appendToIO("holdit                    - stop the program and do single-main-loops-at-a-time (f11 to loop, f12 to exit) (== hi).\n");
	    } else if (strcmp(seashell_stringGet,"stepit") == 0 || strcmp(seashell_stringGet,"si") == 0) {
		INTERNAL_appendToIO("stepit                    - step through the program, stopping at calls of seashell_step (f11 to step, f12 to exit) (== si).\n");
	    } else if (strcmp(seashell_stringGet,"timedhold") == 0 || strcmp(seashell_stringGet,"th") == 0) {
		INTERNAL_appendToIO("timedhold <t>             - go through the program's main loop <t> times, express timings (== th).\n");
	    } else if (strcmp(seashell_stringGet,"timedstep") == 0 || strcmp(seashell_stringGet,"ts") == 0) {
		INTERNAL_appendToIO("timedstep <t>             - go through the program's main loop <t> times, using steps as flags (== ts).\n");
	    } else if (strcmp(seashell_stringGet,"bargraph") == 0 || strcmp(seashell_stringGet,"bg") == 0) {
		INTERNAL_appendToIO("bargraph <var> <amnt>     - draw a bar graph from the location <var> of <amnt> many bytes (== bg).\n");
	    } else if (strcmp(seashell_stringGet,"bargraphf") == 0 || strcmp(seashell_stringGet,"bgf") == 0) {
		INTERNAL_appendToIO("bargraphf <v> <n> <s> <p> - draw a bar graph of scale <s> starting from <v> with <n> many fp values padded by <p> bytes (== bgf).\n");
	    } else if (strcmp(seashell_stringGet,"bargraphd") == 0 || strcmp(seashell_stringGet,"bgd") == 0) {
		INTERNAL_appendToIO("bargraphd <v> <n> <s> <p> - draw a bar graph of scale <s> starting from <v> with <n> many dp values padded by <p> bytes (== bgd).\n");
	    } else if (strcmp(seashell_stringGet,"count") == 0 || strcmp(seashell_stringGet,"cnt") == 0) {
		INTERNAL_appendToIO("count <var> <amnt>        - count <amnt> many byte occurances from location <var>, print occurances (== ctg).\n");
	    } else if (strcmp(seashell_stringGet,"countgraph") == 0 || strcmp(seashell_stringGet,"ctg") == 0) {
		INTERNAL_appendToIO("countgraph <var> <amnt>   - count <amnt> many byte occurances from location <var>, graph as a bar graph (== ctg).\n");
	    } else if (strcmp(seashell_stringGet,"quit") == 0 || strcmp(seashell_stringGet,"qs") == 0) {
		INTERNAL_appendToIO("quit                      - quit seashell, leave program running & ignore any seashell routines (== qs).\n");
	    } else if (strcmp(seashell_stringGet,"quitall") == 0 || strcmp(seashell_stringGet,"q") == 0) {
		INTERNAL_appendToIO("quitall                   - quit everything, immediately (== q).\n");
	    } else if (strcmp(seashell_stringGet,"sys") == 0) {
		INTERNAL_appendToIO("sys <cmd>                 - execute a system command, literally anything the os can do.\n");
	    } else if (strcmp(seashell_stringGet,"script") == 0 || strcmp(seashell_stringGet,"sc") == 0) {
		INTERNAL_appendToIO("script <file/string>      - execute a script, literally anything seashell can do.\n");
	    } else if (strcmp(seashell_stringGet,"list") == 0 || strcmp(seashell_stringGet,"li") == 0) {
		INTERNAL_appendToIO("list <vrs/typs/fncs/all>  - list all vars/types/funcs/all (== li).\n");
	    } else if (strcmp(seashell_stringGet,"log") == 0 || strcmp(seashell_stringGet,"lg") == 0) {
		INTERNAL_appendToIO("log <filename>            - log the commands/outputs recently executed into a file (== lg).\n");
	    } else if (strcmp(seashell_stringGet,"output") == 0 || strcmp(seashell_stringGet,"out") == 0) {
		INTERNAL_appendToIO("output <var> <filename>   - write out the data of a var directly, no formatting (for saving/loading) (== out).\n");
	    } else if (strcmp(seashell_stringGet,"input") == 0 || strcmp(seashell_stringGet,"in") == 0) {
		INTERNAL_appendToIO("input <var> <filename>    - read in the data of a var directly, no formatting (for saving/loading) (== in).\n");
	    } else if (strcmp(seashell_stringGet,"modtype") == 0 || strcmp(seashell_stringGet,"mt") == 0) {
		INTERNAL_appendToIO("modtype <var> <newtype>   - introduce new type <type> to <var> (does NOT alloc, just redefines) (== mt).\n");
	    } else if (strcmp(seashell_stringGet,"modname") == 0 || strcmp(seashell_stringGet,"mn") == 0) {
		INTERNAL_appendToIO("modname <var> <newname>   - rename <var> with <newname> (== mn).\n");
	    } else if (strcmp(seashell_stringGet,"modamnt") == 0 || strcmp(seashell_stringGet,"ma") == 0) {
		INTERNAL_appendToIO("modamnt <var> <newamnt>   - resize <var>'s array size with <newamnt> (does NOT alloc, just redefines) (== ma).\n");
	    } else if (strcmp(seashell_stringGet,"hexadd") == 0 || strcmp(seashell_stringGet,"ha") == 0) {
		INTERNAL_appendToIO("hexadd <h0> <h1>          - add two hex numbers and output hex and dec, for reading arrays (== ha).\n");
	    } else if (strcmp(seashell_stringGet,"help") == 0 || strcmp(seashell_stringGet,"?") == 0) {
		INTERNAL_appendToIO("help                      - you already know this... I hope (== ?).\n");
	    } else if (strcmp(seashell_stringGet,"watch") == 0 || strcmp(seashell_stringGet,"w") == 0) {
		INTERNAL_appendToIO("watch <var>               - add a var to the watch, printing out their value constantly (== w).\n");
	    } else if (strcmp(seashell_stringGet,"clearwatch") == 0 || strcmp(seashell_stringGet,"clw") == 0) {
		INTERNAL_appendToIO("clearwatch <var>          - remove a var from the watch, pass nothing to clear the whole thing (== clw).\n");
	    } else if (strcmp(seashell_stringGet,"ocall") == 0 || strcmp(seashell_stringGet,"oc") == 0) {
		INTERNAL_appendToIO("ocall <ofunc> <obj> <arg> - call a cpp object function on object with arguments (== oc).\n");
	    } else if (strcmp(seashell_stringGet,"_stats") == 0) {
		INTERNAL_appendToIO("_stats                    - internal stats, compile date + approx mem usage.\n");
	    } else {
		INTERNAL_appendToIO("**Error: Unrecognized command\n-Type 'help cmds' for a list of available comands, type 'help <cmd>' for the description of a specific command.\n");
	    }
	} else if (strcmp(seashell_stringGet,"hexadd") == 0 || strcmp(seashell_stringGet,"ha") == 0) {
	    
	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

	    if (sscanf(seashell_stringGet,"%x",&numbGet) == 0 || altLen == 0) {
		INTERNAL_appendToIO("**Error: Unable to read first number to add\n");
	    } else {
	        INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

		int orig = numbGet;
		int secnd;
		if (sscanf(seashell_stringGet,"%x",&secnd) == 0 || altLen == 0) {
		    INTERNAL_appendToIO("**Error: Unable to read second number to add\n");
		} else {
		    int added = orig + secnd;
		    sprintf(stringOut,"DecVal1: %i\nDecVal2: %i\n\n",orig,secnd);
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"Decimal: %i\nHex:     %x\n",added,added);
		    INTERNAL_appendToIO(stringOut);
		}
	    }
	} else {
	    INTERNAL_appendToIO(cmdUnrecd);
	}
	break;
    case 'i':
	if (strcmp(seashell_stringGet,"input") == 0 || strcmp(seashell_stringGet,"in") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {

		INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

		if (altLen == 0) {
		    INTERNAL_appendToIO("**Error: unable to read filename\n");
		} else {
		    FILE *io = fopen(seashell_stringGet,"rb");
		    if (io == NULL) {
			INTERNAL_appendToIO("**Error: cannot open input file\n");
		    } else {
			fseek(io,0,SEEK_END);
			int size = ftell(io);
			fseek(io,0,SEEK_SET);
			
			int bytesGet = seashell_vars[numbGet].amnt * seashell_types[seashell_vars[numbGet].typeUse].sizeTot;
			if (size < bytesGet) {
			    bytesGet = size;
			}
			
			fread(seashell_vars[numbGet].pntr,1,bytesGet,io);
			fclose(io);
		    }
		}
	    }
	} else {
	    INTERNAL_appendToIO(cmdUnrecd);
	}
	break;
    case 'l':
	if (strcmp(seashell_stringGet,"list") == 0 || strcmp(seashell_stringGet,"li") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

	    if (strcmp(seashell_stringGet,"vars") == 0) {
		int amnt = seashell_varsAmnt/3;
		for (int iter = 0; iter < amnt; iter++)
		{
		    sprintf(stringOut,"%-20s ",seashell_vars[3*iter].name);
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"%-20s ",seashell_vars[3*iter + 1].name);
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"%-20s\n",seashell_vars[3*iter + 2].name);
		    INTERNAL_appendToIO(stringOut);
		}
	        for (int iter = amnt*3; iter < seashell_varsAmnt; iter++)
		{
		    sprintf(stringOut,"%-20s ",seashell_vars[iter].name);
		    INTERNAL_appendToIO(stringOut);
		}
		if (amnt*3 != seashell_varsAmnt) {
		    INTERNAL_appendToIO("\n");
		}
	    } else if (strcmp(seashell_stringGet,"types") == 0) {
		int amnt = seashell_typesAmnt/3;
		for (int iter = 0; iter < amnt; iter++)
		{
		    sprintf(stringOut,"%-20s ",seashell_types[3*iter].name);
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"%-20s ",seashell_types[3*iter + 1].name);
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"%-20s\n",seashell_types[3*iter + 2].name);
		    INTERNAL_appendToIO(stringOut);
		}
	        for (int iter = amnt*3; iter < seashell_typesAmnt; iter++)
		{
		    sprintf(stringOut,"%-20s ",seashell_types[iter].name);
		    INTERNAL_appendToIO(stringOut);
		}
		if (amnt*3 != seashell_typesAmnt) {
		    INTERNAL_appendToIO("\n");
		}
	    } else if (strcmp(seashell_stringGet,"funcs") == 0) {
		int amnt = seashell_funcsAmnt/3;
		for (int iter = 0; iter < amnt; iter++)
		{
		    sprintf(stringOut,"%-20s ",seashell_funcs[3*iter].name);
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"%-20s ",seashell_funcs[3*iter + 1].name);
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"%-20s\n",seashell_funcs[3*iter + 2].name);
		    INTERNAL_appendToIO(stringOut);
		}
	        for (int iter = amnt*3; iter < seashell_funcsAmnt; iter++)
		{
		    sprintf(stringOut,"%-20s ",seashell_funcs[iter].name);
		    INTERNAL_appendToIO(stringOut);
		}
		if (amnt*3 != seashell_funcsAmnt) {
		    INTERNAL_appendToIO("\n");
		}
	    } else if (strcmp(seashell_stringGet,"all") == 0) {
		INTERNAL_appendToIO("--vars--\n");
		int amnt = seashell_varsAmnt/3;
		for (int iter = 0; iter < amnt; iter++)
		{
		    sprintf(stringOut,"%-20s ",seashell_vars[3*iter].name);
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"%-20s ",seashell_vars[3*iter + 1].name);
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"%-20s\n",seashell_vars[3*iter + 2].name);
		    INTERNAL_appendToIO(stringOut);
		}
	        for (int iter = amnt*3; iter < seashell_varsAmnt; iter++)
		{
		    sprintf(stringOut,"%-20s ",seashell_vars[iter].name);
		    INTERNAL_appendToIO(stringOut);
		}
		if (amnt*3 != seashell_varsAmnt) {
		    INTERNAL_appendToIO("\n");
		}
		INTERNAL_appendToIO("\n--types--\n");
		amnt = seashell_typesAmnt/3;
		for (int iter = 0; iter < amnt; iter++)
		{
		    sprintf(stringOut,"%-20s ",seashell_types[3*iter].name);
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"%-20s ",seashell_types[3*iter + 1].name);
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"%-20s\n",seashell_types[3*iter + 2].name);
		    INTERNAL_appendToIO(stringOut);
		}
	        for (int iter = amnt*3; iter < seashell_typesAmnt; iter++)
		{
		    sprintf(stringOut,"%-20s ",seashell_types[iter].name);
		    INTERNAL_appendToIO(stringOut);
		}
		if (amnt*3 != seashell_typesAmnt) {
		    INTERNAL_appendToIO("\n");
		}
		INTERNAL_appendToIO("\n--funcs--\n");
		amnt = seashell_funcsAmnt/3;
		for (int iter = 0; iter < amnt; iter++)
		{
		    sprintf(stringOut,"%-20s ",seashell_funcs[3*iter].name);
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"%-20s ",seashell_funcs[3*iter + 1].name);
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"%-20s\n",seashell_funcs[3*iter + 2].name);
		    INTERNAL_appendToIO(stringOut);
		}
	        for (int iter = amnt*3; iter < seashell_funcsAmnt; iter++)
		{
		    sprintf(stringOut,"%-20s ",seashell_funcs[iter].name);
		    INTERNAL_appendToIO(stringOut);
		}
		if (amnt*3 != seashell_funcsAmnt) {
		    INTERNAL_appendToIO("\n");
		}
	    } else {
		INTERNAL_appendToIO("**Error: list identifier unrecognized\n");
	    }
	} else if (strcmp(seashell_stringGet,"log") == 0 || strcmp(seashell_stringGet,"lg") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

	    if (altLen == 0) {
		INTERNAL_appendToIO("**Error: unable to read filename\n");
	    } else {
		FILE *io = fopen(seashell_stringGet,"w");
		if (io == NULL) {
		    INTERNAL_appendToIO("**Error: cannot open log file\n");
		} else {
		    fwrite(seashell_io,1,seashell_ioLen,io);
		    time_t t = time(NULL);
		    struct tm* tm = localtime(&t);
		    fprintf(io,"\nLogged: %d/%d/%d at %02d:%02d",tm->tm_mon + 1,tm->tm_mday,tm->tm_year + 1900,tm->tm_hour,tm->tm_min);
		    fclose(io);
		}
	    }
	} else {
	    INTERNAL_appendToIO(cmdUnrecd);
	}
	break;
    case 'm':
	if (strcmp(seashell_stringGet,"move") == 0 || strcmp(seashell_stringGet,"mv") == 0) {
	    
	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

	    int secndGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO("**Error: first variable identifier unrecognized\n");
		if (secndGet == -1) {
		    INTERNAL_appendToIO("**Error: second variable identifier unrecognized\n");
		}
	    } else {
		if (secndGet == -1) {
		    INTERNAL_appendToIO("**Error: second variable identifier unrecognized\n");
		} else {
		    int typeOf = seashell_vars[secndGet].typeUse;
		    int sizeOfCopy = seashell_vars[secndGet].amnt * seashell_types[typeOf].sizeTot;

		    int typeGet = seashell_vars[numbGet].typeUse;
		    int sizeGet = seashell_vars[numbGet].amnt * seashell_types[typeGet].sizeTot;

		    if (sizeGet < sizeOfCopy) {
			memcpy(seashell_vars[numbGet].pntr,seashell_vars[secndGet].pntr,sizeGet);
		    } else {
			memcpy(seashell_vars[numbGet].pntr,seashell_vars[secndGet].pntr,sizeOfCopy);
		    }
		}
	    }
	} else if (strcmp(seashell_stringGet,"modtype") == 0 || strcmp(seashell_stringGet,"mt") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {
	        INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

		int typeNew = INTERNAL_getTypeByName(seashell_stringGet);

		if (typeNew == -1) {
		    INTERNAL_appendToIO("**Error: new type not recognized\n");
		} else {
		    seashell_vars[numbGet].typeUse = typeNew;
		}
	    }
	} else if (strcmp(seashell_stringGet,"modname") == 0 || strcmp(seashell_stringGet,"mn") == 0) {
	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {
	        INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

		if (altLen == 0) {
		    INTERNAL_appendToIO("**Error: could not read new name\n");
		} else {
		    memcpy(seashell_vars[numbGet].name,seashell_stringGet,altLen);

		    seashell_sortAll = 1;
		}
	    }
	} else if (strcmp(seashell_stringGet,"modamnt") == 0 || strcmp(seashell_stringGet,"ma") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {
	        INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

		int newAmnt = 0;
		if (sscanf(seashell_stringGet,"%i",&newAmnt) == 0 || altLen == 0) {
		    INTERNAL_appendToIO("**Error: unable to read new amount\n");
		} else {
		    seashell_vars[numbGet].amnt = newAmnt;
		}
	    }
	} else {
	    INTERNAL_appendToIO(cmdUnrecd);
	}
	break;
    case 'o':
	if (strcmp(seashell_stringGet,"output") == 0 || strcmp(seashell_stringGet,"out") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {
	        INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

		if (altLen == 0) {
		    INTERNAL_appendToIO("**Error: unable to read filename\n");
		} else {
		    FILE *io = fopen(seashell_stringGet,"wb");
		    if (io == NULL) {
			INTERNAL_appendToIO("**Error: cannot open output file\n");
		    } else {
			int bytesGet = seashell_vars[numbGet].amnt * seashell_types[seashell_vars[numbGet].typeUse].sizeTot;
			fwrite(seashell_vars[numbGet].pntr,1,bytesGet,io);
			fclose(io);
		    }
		}
	    }
	} else if (strcmp(seashell_stringGet,"ocall") == 0 || strcmp(seashell_stringGet,"oc") == 0) {
	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getFuncByName(seashell_stringGet);
	
	    if (numbGet == -1) {
		INTERNAL_appendToIO("**Error: function identifier unrecognized\n");
	    } else {
		if (seashell_funcs[numbGet].argAmnts[0] == -1) {
		    INTERNAL_appendToIO("**Warning: normal function being called as member function\n");
		}
		
		INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
		int objGet = INTERNAL_getVarByName(seashell_stringGet);
		
		if (objGet == -1) {
		    INTERNAL_appendToIO("**Error: object identifier unrecognized\n");
		} else {
		    //write obj as first argument
		    
		    memcpy(&(seashell_args[0]),&(seashell_vars[objGet]),sizeof(seashell_var));
		    seashell_args[0].pntr = seashell_defaultArgs[0];
		    memcpy(seashell_args[0].pntr,&(seashell_vars[objGet].pntr),8);
		    
		    //main arg getting
		    int varGet, err = 0, pntr;
		
		    for (int iter = 0; iter < seashell_funcs[numbGet].args; iter++)
		    {
			altLen = 0;
			while (seashell_userInput[getLen] == ' ' && getLen < seashell_inputLen)
			{
			    getLen++;
			}
			if (seashell_userInput[getLen] == '&' && seashell_funcs[numbGet].argTypes[iter] == INTERNAL_getTypeByName("pointer")) {
			    getLen += 1;
			    pntr = 1;
			} else {
			    pntr = 0;
			}
			while (seashell_userInput[getLen] != ' ' && getLen < seashell_inputLen)
			{
			    seashell_stringGet[altLen] = seashell_userInput[getLen];
			    altLen++; getLen++;
			}
			seashell_stringGet[altLen] = 0;

			varGet = INTERNAL_getVarByName(seashell_stringGet);
		    
			if (varGet == -1 && altLen == 0) {
			    INTERNAL_appendToIO("**Error: missing arguments, call 'desc <func>' to list information.\n");
			    err = 1;
			    break;
			} else if (varGet == -1) {
			    int point = 0;
			    for (int loc = 0; loc < altLen; loc++)
			    {
				if (seashell_stringGet[loc] == '.') {
				    point = 1;
				}
			    }
			
			    //repetitive? (not that I really care, people who harp on that are insane)
			
			    if (seashell_types[seashell_funcs[numbGet].argTypes[iter]].members != 1) {
				sprintf(stringOut,"**Error: cannot read argument %i, of array variety (passing arrays is unsupported).\n",iter+1);
				INTERNAL_appendToIO(stringOut);
				err = 1;
				break;
			    } else {
				if (point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 8 &&
				    seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    float get = 0.0;
				    if (sscanf(seashell_stringGet,"%f",&get) == 0) {
					sprintf(stringOut,"**Error: cannot read float argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					double thing = get;
					seashell_args[iter+1].pntr = seashell_defaultArgs[iter+1];
					memcpy(seashell_args[iter+1].pntr,&thing,8);
				    }
				} else if (point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 4 &&
					   seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    float get = 0.0;
				    if (sscanf(seashell_stringGet,"%f",&get) == 0) {
					sprintf(stringOut,"**Error: cannot read float argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					seashell_args[iter+1].pntr = seashell_defaultArgs[iter+1];
					memcpy(seashell_args[iter+1].pntr,&get,4);
				    }
				} else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 8 &&
					   !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    unsigned long long get;
				    if (sscanf(seashell_stringGet,"%llu",&get) == 0) {
					sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					seashell_args[iter+1].pntr = seashell_defaultArgs[iter+1];
					memcpy(seashell_args[iter+1].pntr,&get,8);
				    }
				} else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == -8 &&
					   !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    long long get;
				    if (sscanf(seashell_stringGet,"%lli",&get) == 0) {
					sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					seashell_args[iter+1].pntr = seashell_defaultArgs[iter+1];
					memcpy(seashell_args[iter+1].pntr,&get,8);
				    }
				} else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 4 &&
					   !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    unsigned long get;
				    if (sscanf(seashell_stringGet,"%lu",&get) == 0) {
					sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					seashell_args[iter+1].pntr = seashell_defaultArgs[iter+1];
					memcpy(seashell_args[iter+1].pntr,&get,4);
				    }
				} else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == -4 &&
					   !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    long get;
				    if (sscanf(seashell_stringGet,"%li",&get) == 0) {
					sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					seashell_args[iter+1].pntr = seashell_defaultArgs[iter+1];
					memcpy(seashell_args[iter+1].pntr,&get,4);
				    }
				} else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 2 &&
					   !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    unsigned short get;
				    if (sscanf(seashell_stringGet,"%hu",&get) == 0) {
					sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					seashell_args[iter+1].pntr = seashell_defaultArgs[iter+1];
					memcpy(seashell_args[iter+1].pntr,&get,2);
				    }
				} else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == -2 &&
					   !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    short get;
				    if (sscanf(seashell_stringGet,"%hi",&get) == 0) {
					sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					seashell_args[iter+1].pntr = seashell_defaultArgs[iter+1];
					memcpy(seashell_args[iter+1].pntr,&get,2);
				    }
				} else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 1 &&
					   !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    unsigned short get;
				    if (sscanf(seashell_stringGet,"%hu",&get) == 0) { //intentional shorts here
					sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					seashell_args[iter+1].pntr = seashell_defaultArgs[iter+1];
					memcpy(seashell_args[iter+1].pntr,&get,1);
				    }
				} else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == -1 &&
					   !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				    short get;
				    if (sscanf(seashell_stringGet,"%hi",&get) == 0) { //same as above
					sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
					INTERNAL_appendToIO(stringOut);
					err = 1;
					break;
				    } else {
					seashell_args[iter+1].pntr = seashell_defaultArgs[iter+1];
					memcpy(seashell_args[iter+1].pntr,&get,1);
				    }
				} else {
				    sprintf(stringOut,"**Error: cannot read argument %i, of unsupported size for constant pass and unrecognized as variable.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				}
			    }
			} else {
			    //typecheck

			    if (seashell_vars[varGet].typeUse != seashell_funcs[numbGet].argTypes[iter] &&
				(!pntr && seashell_funcs[numbGet].argTypes[iter] == INTERNAL_getTypeByName("pointer")) ) {
				sprintf(stringOut,"**Error: argument %i of incorrect type, should be '%s'\n",iter+1,seashell_types[seashell_funcs[numbGet].argTypes[iter]].name);
				INTERNAL_appendToIO(stringOut);
				err = 1;
				break;
			    }
			
			    //input (modified for ocall)
			    
			    if (pntr) {
				memcpy(&seashell_args[iter+1],&seashell_vars[varGet],sizeof(seashell_var));
				seashell_args[iter+1].pntr = seashell_defaultArgs[iter+1];
				memcpy(seashell_args[iter+1].pntr,&(seashell_vars[varGet].pntr),8);
			    } else {
				memcpy(&seashell_args[iter+1],&seashell_vars[varGet],sizeof(seashell_var));
				if (seashell_vars[varGet].amnt * seashell_types[seashell_vars[varGet].typeUse].sizeTot <= 8) {
				    seashell_args[iter+1].pntr = seashell_defaultArgs[iter+1];
				    memcpy(seashell_args[iter+1].pntr,seashell_vars[varGet].pntr,
					   seashell_vars[varGet].amnt * seashell_types[seashell_vars[varGet].typeUse].sizeTot);
				}
			    }
			}
		    }
		    
		    seashell_return.typeUse = seashell_funcs[numbGet].typeReturn;
		
		    if (!err) {
			//setup
			seashell_funcs[numbGet].args += 1;
			for (int iter = seashell_funcs[numbGet].args-1; iter > 0; iter--)
			{
			    seashell_funcs[numbGet].argTypes[iter] = seashell_funcs[numbGet].argTypes[iter-1];
			}
			seashell_funcs[numbGet].argTypes[0] = INTERNAL_getTypeByName("pointer");
			//call
			INTERNAL_funcCall(seashell_vars,seashell_funcs,seashell_types,numbGet,&seashell_return,seashell_args);
			//reset
			for (int iter = 0; iter < seashell_funcs[numbGet].args-1; iter++)
			{
			    seashell_funcs[numbGet].argTypes[iter] = seashell_funcs[numbGet].argTypes[iter+1];
			}
			seashell_funcs[numbGet].args -= 1;
			
			if (seashell_funcs[numbGet].typeReturn != -1) {
			    if (seashell_return.typeUse == INTERNAL_getTypeByName("pointer")) {
				sprintf(stringOut,"(pointer) %llu\n",((unsigned long long*) seashell_return.pntr));
				INTERNAL_appendToIO(stringOut);
			    } else {
				INTERNAL_printVar(&seashell_return);
			    }
			}
		    }
		}
	    }
	} else {
	    INTERNAL_appendToIO(cmdUnrecd);
	}
	break;
    case 'p':
	if (strcmp(seashell_stringGet,"print") == 0 || strcmp(seashell_stringGet,"p") == 0) {
	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
	        INTERNAL_appendToIO(varUnrecd);
	    } else {
		INTERNAL_printVar(&(seashell_vars[numbGet]));
	    }
	} else if (strcmp(seashell_stringGet,"prints") == 0 || strcmp(seashell_stringGet,"ps") == 0) {
	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {
	        int type = seashell_vars[numbGet].typeUse;
		int bytesPut = seashell_vars[numbGet].amnt * seashell_types[type].sizeTot;
		
		int sizeDiv = bytesPut/(identifier_max-1); //minus one to include null character
		char* pntr = (char*) seashell_vars[numbGet].pntr;
		
		int stopped = 0;
		
		for (int iter = 0; iter < sizeDiv; iter++)
		{
		    for (int iter1 = 0; iter1 < identifier_max-1; iter1++)
		    {
			if (!(*pntr)) { //is the character null (stop if so)
			    stringOut[iter1] = 0;
			    stopped = 1;
			    break;
			}
			
			stringOut[iter1] = *pntr;
			pntr++;
		    }
		    stringOut[identifier_max - 1] = 0; //makes sure strlen gets it right in appendtoio
		    INTERNAL_appendToIO(stringOut);
		    if (stopped) {
			break;
		    }
		}

		if (!stopped) {
		    stringOut[1] = 0; //same as above
		    for (int iter = sizeDiv*(identifier_max-1); iter < bytesPut; iter++)
		    {
			if (!(*pntr)) {
			    break;
			}
			
			stringOut[0] = *pntr;
			pntr++;

			INTERNAL_appendToIO(stringOut);
		    }
		}

	        INTERNAL_appendToIO("\n");
	    }
	} else if (strcmp(seashell_stringGet,"printx") == 0 || strcmp(seashell_stringGet,"px") == 0) {
	    
	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {
		int type = seashell_vars[numbGet].typeUse;
		int bytesPut = seashell_vars[numbGet].amnt * seashell_types[type].sizeTot;

		int sizeDiv = bytesPut/29;
		unsigned char* pntr = seashell_vars[numbGet].pntr;

		strcpy(stringOut,"        00 01 02 03 04 05 06 07 ");
		INTERNAL_appendToIO(stringOut);
		strcpy(stringOut,"08 09 0a 0b 0c 0d 0e 0f 10 11 ");
		INTERNAL_appendToIO(stringOut);
		strcpy(stringOut,"12 13 14 15 16 17 18 19 1a 1b 1c\n\n");
		INTERNAL_appendToIO(stringOut);
		
		for (int iter = 0; iter < sizeDiv; iter++)
		{
		    sprintf(stringOut,"[%04x]  %02x %02x %02x %02x %02x %02x %02x ",
			    (pntr-seashell_vars[numbGet].pntr),*(pntr),*(pntr+1),*(pntr+2),*(pntr+3),*(pntr+4),
			    *(pntr+5),*(pntr+6));
		    INTERNAL_appendToIO(stringOut);
		    pntr += 7;
		    sprintf(stringOut,"%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x ",
			    *(pntr),*(pntr+1),*(pntr+2),*(pntr+3),*(pntr+4),
			    *(pntr+5),*(pntr+6),*(pntr+7),*(pntr+8),*(pntr+9));
		    INTERNAL_appendToIO(stringOut);
		    pntr += 10;
		    sprintf(stringOut,"%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x\n",
			    *(pntr),*(pntr+1),*(pntr+2),*(pntr+3),*(pntr+4),
			    *(pntr+5),*(pntr+6),*(pntr+7),*(pntr+8),*(pntr+9),*(pntr+10),*(pntr+11));
		    INTERNAL_appendToIO(stringOut);
		    pntr += 12;
		}

		if (sizeDiv*29 != bytesPut) {
		    sprintf(stringOut,"[%04x]  ",(pntr-seashell_vars[numbGet].pntr));
		    INTERNAL_appendToIO(stringOut);
		
		    for (int iter = sizeDiv*29; iter < bytesPut; iter++)
		    {
			sprintf(stringOut,"%02x ",*(pntr));
			INTERNAL_appendToIO(stringOut);
			pntr += 1;
		    }

		    INTERNAL_appendToIO("\n");
		}
	    }
	} else if (strcmp(seashell_stringGet,"printnm") == 0 || strcmp(seashell_stringGet,"pnm") == 0) {
	    
	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {
	        INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
		
		int numb, numbStore;
		if (sscanf(seashell_stringGet,"%i",&numb) == 0 || altLen == 0) {
		    INTERNAL_appendToIO("**Error: unable to recognize array number\n");
		} else {
		    numbStore = numb;
		    if (numbStore >= seashell_vars[numbGet].amnt) {
			INTERNAL_appendToIO("**Error: array number outside bounds\n");
		    } else {
			
		        INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

			int memb, membStore;
			if (sscanf(seashell_stringGet,"%i",&memb) == 0 || altLen == 0) {
			    INTERNAL_appendToIO("**Error: unable to recognize member number\n");
			} else {
			    membStore = memb;
			    if (membStore >= seashell_types[seashell_vars[numbGet].typeUse].members) {
				INTERNAL_appendToIO("**Error: member number outside bounds\n");
			    } else {
				seashell_return.amnt = 1;
				
				switch (seashell_types[seashell_vars[numbGet].typeUse].sizes[membStore]) {
				case 1:
				    seashell_return.typeUse = INTERNAL_getTypeByName("unsigned char");
				    break;
				case -1:
				    seashell_return.typeUse = INTERNAL_getTypeByName("char");
				    break;
				case 2:
				    seashell_return.typeUse = INTERNAL_getTypeByName("unsigned short");
				    break;
				case -2:
				    seashell_return.typeUse = INTERNAL_getTypeByName("short");
				    break;
				case 4:
				    if (seashell_types[seashell_vars[numbGet].typeUse].floaty[membStore]) {
					seashell_return.typeUse = INTERNAL_getTypeByName("float");
				    } else {
					seashell_return.typeUse = INTERNAL_getTypeByName("unsigned int");
				    }
				    break;
				case -4:
				    seashell_return.typeUse = INTERNAL_getTypeByName("int");
				    break;
				case 8:
				    if (seashell_types[seashell_vars[numbGet].typeUse].floaty[membStore]) {
					seashell_return.typeUse = INTERNAL_getTypeByName("double");
				    } else {
					seashell_return.typeUse = INTERNAL_getTypeByName("unsigned long long");
				    }
				    break;
				case -8:
				    seashell_return.typeUse = INTERNAL_getTypeByName("long long");
				    break;
				default:
				    seashell_return.typeUse = INTERNAL_getTypeByName("unsigned char");
				    seashell_return.amnt = seashell_types[seashell_vars[numbGet].typeUse].sizes[membStore];
				    break;
				}

			        int adder = seashell_types[seashell_vars[numbGet].typeUse].sizeTot * numbStore;
				for (int iter = 0; iter < membStore; iter++)
				{
				    adder += abs(seashell_types[seashell_vars[numbGet].typeUse].sizes[iter]);
				}

				printf("%i",adder);
				
				memcpy(seashell_return.pntr,seashell_vars[numbGet].pntr + adder,
				       abs(seashell_types[seashell_vars[numbGet].typeUse].sizes[membStore]));
				
				INTERNAL_printVar(&seashell_return);
			    }
			}
		    }
		}
	    }
	} else if (strcmp(seashell_stringGet,"paste") == 0 || strcmp(seashell_stringGet,"pst") == 0) {
	    
	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else if (seashell_varClipboard != -1) {
		int typeOf = seashell_vars[seashell_varClipboard].typeUse;
	        int sizeOfCopy = seashell_vars[seashell_varClipboard].amnt * seashell_types[typeOf].sizeTot;

		int typeGet = seashell_vars[numbGet].typeUse;
		int sizeGet = seashell_vars[numbGet].amnt * seashell_types[typeGet].sizeTot;

		if (sizeGet < sizeOfCopy) {
		    memcpy(seashell_vars[numbGet].pntr,seashell_vars[seashell_varClipboard].pntr,sizeGet);
		} else {
		    memcpy(seashell_vars[numbGet].pntr,seashell_vars[seashell_varClipboard].pntr,sizeOfCopy);
		}
	    } else {
		INTERNAL_appendToIO("**Error: clipboard is empty, cannot write data.\n");
	    }
	} else {
	    INTERNAL_appendToIO(cmdUnrecd);
	}
	break;
    case 'q':
	if (strcmp(seashell_stringGet,"quit") == 0 || strcmp(seashell_stringGet,"qs") == 0) {
	    seashell_quit();
	} else if (strcmp(seashell_stringGet,"quitall") == 0 || strcmp(seashell_stringGet,"q") == 0) {
	    exit(0);
	} else {
	    INTERNAL_appendToIO(cmdUnrecd);
	}
	break;
    case 's':
	if (strcmp(seashell_stringGet,"scan") == 0 || strcmp(seashell_stringGet,"s") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {

		INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

		int sign = seashell_types[seashell_vars[numbGet].typeUse].sizes[0];
	        unsigned long long measureU = 0;
		long long measureS = 0;
		int problems = 0;
		if (seashell_types[seashell_vars[numbGet].typeUse].floaty[0]) {
		    float get;
		    if (sscanf(seashell_stringGet,"%f",&get) == 0 || altLen == 0) {
		        problems = 1;
		    } else {
			if (seashell_types[seashell_vars[numbGet].typeUse].sizeTot == 4) {
			    memcpy(&measureU,&get,4);
			} else {
			    double mod = get;
			    memcpy(&measureU,&mod,8);
			}
		    }
		} else {
		    if (sign < 0) {
			if (sscanf(seashell_stringGet,"%lli",&measureS) == 0 || altLen == 0) {
			    problems = 1;
			}
		    } else if (sign > 0) {
			if (sscanf(seashell_stringGet,"%llu",&measureU) == 0 || altLen == 0) {
			    problems = 1;
			}
		    }
		}

		if (problems) {
		    INTERNAL_appendToIO("**Error: could not read desired value\n");
		} else {
		    int byteGet = seashell_vars[numbGet].amnt;
		    byteGet *= seashell_types[seashell_vars[numbGet].typeUse].sizeTot;

		    if (byteGet > 8) {
			byteGet = 8;
		    }
		
		    //we do the below for consistency: it is only neccessary for signed values
		    //but my ocd didn't allow me to have variance.
		
		    if (byteGet == 1) {
			if (sign < 0) {
			    char this = (char) measureS;
			    memcpy(seashell_vars[numbGet].pntr,&this,1);
			} else if (sign > 0) {
			    unsigned char this = (unsigned char) measureU;
			    memcpy(seashell_vars[numbGet].pntr,&this,1);
			}
		    } else if (byteGet == 2) {
			if (sign < 0) {
			    short this = (short) measureS;
			    memcpy(seashell_vars[numbGet].pntr,&this,2);
			} else if (sign > 0) {
			    unsigned short this = (unsigned short) measureU;
			    memcpy(seashell_vars[numbGet].pntr,&this,2);
			}
		    } else if (byteGet == 4) {
			if (sign < 0) {
			    int this = (int) measureS;
			    memcpy(seashell_vars[numbGet].pntr,&this,4);
			} else if (sign > 0) {
			    if (!seashell_types[seashell_vars[numbGet].typeUse].floaty[0]) {
				unsigned int this = (unsigned int) measureU;
				memcpy(seashell_vars[numbGet].pntr,&this,4);
			    } else {
				memcpy(seashell_vars[numbGet].pntr,&measureU,4);
			    }
			}
		    } else if (byteGet == 8) {
			if (sign < 0) {
			    memcpy(seashell_vars[numbGet].pntr,&measureS,8);
			} else if (sign > 0) {
			    memcpy(seashell_vars[numbGet].pntr,&measureU,8);
			}
		    } else {
			if (sign < 0) {
			    memcpy(seashell_vars[numbGet].pntr,&measureS,byteGet);
			} else if (sign > 0) {
			    memcpy(seashell_vars[numbGet].pntr,&measureU,byteGet);
			}
		    }
		}
	    }
	} else if (strcmp(seashell_stringGet,"scans") == 0 || strcmp(seashell_stringGet,"sst") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {

		//INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
		getLen++;
		memcpy(seashell_stringGet,&(seashell_userInput[getLen]),seashell_inputLen-getLen);
	        altLen = seashell_inputLen-getLen;
	        
		int bytesGet = (seashell_types[seashell_vars[numbGet].typeUse].sizeTot)*(seashell_vars[numbGet].amnt);
		int lenUse, offset;
		
		if (bytesGet < altLen) {
		    lenUse = bytesGet;
		} else {
		    lenUse = altLen;
		}

		char* pntr = (char*) seashell_vars[numbGet].pntr;
	        unsigned char get = 0;
		for (int iter = 0; iter < lenUse; iter++)
		{
		    if (seashell_stringGet[iter] == '\\') {
			switch (seashell_stringGet[iter+1]) {
			case '\\': case '\"': case '\'':
			    *pntr = seashell_stringGet[iter+1];
			    pntr++; iter++;
			    break;
			case 'n':
			    *pntr = '\n';
			    pntr++; iter++;
			    break;
			case 'r':
			    *pntr = '\r';
			    pntr++; iter++;
			    break;
			case 't':
			    *pntr = '\t';
			    pntr++; iter++;
			    break;
			case 'f':
			    *pntr = '\f';
			    pntr++; iter++;
			    break;
			case '0':
			    *pntr = 0;
			    pntr++; iter++;
			    break;
			case 'b':
			    if (pntr > (char*) seashell_vars[numbGet].pntr) {
				pntr--;
			    }
			    iter++;
			    break;
			case 'o':
			    get += (seashell_stringGet[iter+2]-'0')*64;
			    get += (seashell_stringGet[iter+3]-'0')*8;
			    get += (seashell_stringGet[iter+4]-'0');
			    *((unsigned char*) pntr) = get;
			    pntr++;
			    iter += 4;
			    break;
			case 'x':
			    if (seashell_stringGet[iter+2] >= 'a' && seashell_stringGet[iter+2] <= 'f') {
				get += (seashell_stringGet[iter+2]-'a'+10)*16;
			    } else {
				get += (seashell_stringGet[iter+2]-'0')*16;
			    }
			    if (seashell_stringGet[iter+3] >= 'a' && seashell_stringGet[iter+3] <= 'f') {
				get += (seashell_stringGet[iter+3]-'a'+10);
			    } else {
				get += (seashell_stringGet[iter+3]-'0');
			    }
			    *((unsigned char*) pntr) = get;
			    pntr++;
			    iter += 3;
			    break;
			default:
			    *pntr = '\\';
			    pntr++;
			    break;
			}
		    } else {
			*pntr = seashell_stringGet[iter];
			pntr++;
		    }
		}

		*pntr = 0;
		pntr++;
	    }
	} else if (strcmp(seashell_stringGet,"scanx") == 0 || strcmp(seashell_stringGet,"sx") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {

		INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

		unsigned int offset = 0, offsetStore = 0;
		if (sscanf(seashell_stringGet,"%x",&offset) == 0 || altLen == 0) {
		    INTERNAL_appendToIO("**Error: could not read offset\n");
		} else {
		    offsetStore = offset;
		    
		    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

		    unsigned int hex;
		    if (sscanf(seashell_stringGet,"%x",&hex) == 0 || altLen == 0) {
			INTERNAL_appendToIO("**Error: could not read value\n");
		    } else {
			unsigned char* pntr = (unsigned char*) seashell_vars[numbGet].pntr;
			pntr += offsetStore;
		        *pntr = (unsigned char) hex;
		    }
		}
	    }
	} else if (strcmp(seashell_stringGet,"scannm") == 0 || strcmp(seashell_stringGet,"snm") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {

		INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
		
		int numb, numbStore;
		if (sscanf(seashell_stringGet,"%i",&numb) == 0 || altLen == 0) {
		    INTERNAL_appendToIO("**Error: unable to recognize array number\n");
		} else {
		    numbStore = numb;
		    if (numbStore >= seashell_vars[numbGet].amnt) {
			INTERNAL_appendToIO("**Error: array number outside bounds\n");
		    } else {
		        INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

			int memb, membStore;
			if (sscanf(seashell_stringGet,"%i",&memb) == 0 || altLen == 0) {
			    INTERNAL_appendToIO("**Error: unable to recognize member number\n");
			} else {
			    membStore = memb;
			    if (membStore >= seashell_types[seashell_vars[numbGet].typeUse].members) {
				INTERNAL_appendToIO("**Error: member number outside bounds\n");
			    } else {
				unsigned char* pntr = seashell_vars[numbGet].pntr;
				pntr += numbStore*(seashell_types[seashell_vars[numbGet].typeUse].sizeTot);
				for (int iter = 0; iter < membStore; iter++)
				{
				    pntr += abs(seashell_types[seashell_vars[numbGet].typeUse].sizes[iter]);
				}

			        INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

				int sign = seashell_types[seashell_vars[numbGet].typeUse].sizes[membStore];
				unsigned long long measureU = 0;
				long long measureS = 0;
				int problems = 0;
				if (seashell_types[seashell_vars[numbGet].typeUse].floaty[membStore]) {
				    float get;
				    if (sscanf(seashell_stringGet,"%f",&get) == 0 || altLen == 0) {
					problems = 1;
				    } else {
					if (seashell_types[seashell_vars[numbGet].typeUse].sizeTot == 4) {
					    memcpy(&measureU,&get,4);
					} else {
					    double mod = get;
					    memcpy(&measureU,&mod,8);
					}
				    }
				} else {
				    if (sign < 0) {
					if (sscanf(seashell_stringGet,"%lli",&measureS) == 0 || altLen == 0) {
					    problems = 1;
					}
				    } else if (sign > 0) {
					if (sscanf(seashell_stringGet,"%llu",&measureU) == 0 || altLen == 0) {
					    problems = 1;
					}
				    }
				}

				if (problems) {
				    INTERNAL_appendToIO("**Error: could not read desired value\n");
				} else {
				    int byteGet = abs(seashell_types[seashell_vars[numbGet].typeUse].sizes[membStore]);

				    if (byteGet > 8) {
					byteGet = 8;
				    }

				    if (byteGet == 1) {
					if (sign < 0) {
					    char this = (char) measureS;
					    memcpy(pntr,&this,1);
					} else if (sign > 0) {
					    unsigned char this = (unsigned char) measureU;
					    memcpy(pntr,&this,1);
					}
				    } else if (byteGet == 2) {
					if (sign < 0) {
					    short this = (short) measureS;
					    memcpy(pntr,&this,2);
					} else if (sign > 0) {
					    unsigned short this = (unsigned short) measureU;
					    memcpy(pntr,&this,2);
					}
				    } else if (byteGet == 4) {
					if (sign < 0) {
					    int this = (int) measureS;
					    memcpy(pntr,&this,4);
					} else if (sign > 0) {
					    if (!seashell_types[seashell_vars[numbGet].typeUse].floaty[membStore]) {
						unsigned int this = (unsigned int) measureU;
						memcpy(pntr,&this,4);
					    } else {
						memcpy(pntr,&measureU,4);
					    }
					}
				    } else if (byteGet == 8) {
					if (sign < 0) {
					    memcpy(pntr,&measureS,8);
					} else if (sign > 0) {
					    memcpy(pntr,&measureU,8);
					}
				    } else {
					if (sign < 0) {
					    memcpy(pntr,&measureS,byteGet);
					} else if (sign > 0) {
					    memcpy(pntr,&measureU,byteGet);
					}
				    }
				}
			    }
			}
		    }
		}
	    }
	} else if (strcmp(seashell_stringGet,"stepit") == 0 || strcmp(seashell_stringGet,"si") == 0) {
	    seashell_holding = 1;
	    seashell_stepping = 1;
	    seashell_renderEvent = 2;
	} else if (strcmp(seashell_stringGet,"sys") == 0) {
	    while (seashell_userInput[getLen] == ' ' && getLen < seashell_inputLen)
	    {
		getLen++;
	    }
	    memmove(seashell_userInput,&(seashell_userInput[getLen]),seashell_inputLen-getLen);
	    seashell_userInput[seashell_inputLen-getLen] = 0;
	    
	    system(seashell_userInput);
	} else if (strcmp(seashell_stringGet,"script") == 0 || strcmp(seashell_stringGet,"sc") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

	    if (altLen > 0 && seashell_stringGet[altLen-3] == '.' && seashell_stringGet[altLen-2] == 's' &&
		seashell_stringGet[altLen-1] == 'c') {

		FILE* io = fopen(seashell_stringGet,"rb");

		if (io != NULL) {
		    char* text;
		    int size;
		    fseek(io,0,SEEK_END);
		    size = ftell(io);
		    fseek(io,0,SEEK_SET);
		    text = malloc(size);
		    fread(text,1,size,io);
		    
		    int iterText = 0, iterInput = 0;
		    char first = 1;
		    while (iterText < size)
		    {
			if (text[iterText] != '\r' && text[iterText] != '\n') {
			    seashell_userInput[iterInput] = text[iterText];
			    iterInput++; iterText++;
			} else {
			    iterText++;
			}

			if (text[iterText] == '\n' || iterText == size) {
			    seashell_userInput[iterInput] = 0;
			    seashell_inputLen = iterInput;
			    iterInput = 0;
			    //printf("%s : %i\n",seashell_userInput,seashell_inputLen);
			    if (first) {
				INTERNAL_appendToIO("-beginning of script-\n\n");
				first = 0;
			    }
			    if (seashell_inputLen > 0) {
				INTERNAL_enter(1);
			    }
			}
		    }

		    INTERNAL_appendToIO("\n\n-end of script-\n");

		    fclose(io);
		    free(text);
		} else {
		    INTERNAL_appendToIO("**Error: cannot open file\n");
		}
		
	    } else if (altLen > 0) {

		numbGet = INTERNAL_getVarByName(seashell_stringGet);

		if (numbGet != -1) {
		    if (seashell_vars[numbGet].typeUse == INTERNAL_getTypeByName("pointer") ||
			(seashell_vars[numbGet].typeUse == INTERNAL_getTypeByName("char") && seashell_vars[numbGet].amnt != 1)) {

			char* text;
			int size = strlen(seashell_vars[numbGet].pntr);
			text = malloc(size);
			memcpy(text,seashell_vars[numbGet].pntr,size);
		
			int iterText = 0, iterInput = 0;
			while (iterText < size)
			{
			    if (text[iterText] != '\r' && text[iterText] != '\n') {
				seashell_userInput[iterInput] = text[iterText];
				iterInput++; iterText++;
			    } else {
				iterText++;
			    }

			    if (text[iterText] == '\n' || iterText == size) {
				seashell_inputLen = iterInput;
				iterInput = 0;
				if (seashell_inputLen > 0) {
				    INTERNAL_enter(1);
				}
			    }
			}

			INTERNAL_appendToIO("\n\n-end of script-\n");
			
			free(text);
			
		    } else {
			INTERNAL_appendToIO("**Error: cannot get string, incorrect type\n");
		    }
		} else {
		    INTERNAL_appendToIO("**Error: cannot get variable\n");
		}
		
	    } else {
		INTERNAL_appendToIO("**Error: please input a string or file\n");
	    }
	    
	} else {
	    INTERNAL_appendToIO(cmdUnrecd);
	}
	break;
    case 't':
	if (strcmp(seashell_stringGet,"timedhold") == 0 || strcmp(seashell_stringGet,"th") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    if (sscanf(seashell_stringGet,"%i",&numbGet) == 0 || altLen == 0) {
		INTERNAL_appendToIO("**Error: could not read times parameter\n");
	    } else {
		if (numbGet < 30) {
		    INTERNAL_appendToIO("**Error: more please! Lower than 30 is innacurate!\n");
		} else {
		    seashell_holdsLeft = numbGet;
		    seashell_holdsTotal = numbGet;
		    seashell_holdStart = SDL_GetPerformanceCounter();

		    dontPrompt = 1;

		    forceNoNewline = 1;
		}
	    }
	} else if (strcmp(seashell_stringGet,"timedstep") == 0 || strcmp(seashell_stringGet,"ts") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    if (sscanf(seashell_stringGet,"%i",&numbGet) == 0 || altLen == 0) {
		INTERNAL_appendToIO("**Error: could not read times parameter\n");
	    } else {
		if (numbGet < 30) {
		    INTERNAL_appendToIO("**Error: more please! Lower than 30 is innacurate!\n");
		} else {
		    seashell_holdsLeft = numbGet;
		    seashell_holdsTotal = numbGet;
		    seashell_measuringSteps = 1;
		    seashell_stepOn = 0;
		    seashell_ticksTotal = 0;
		    memset(seashell_stepsAdd,0,seashell_stepsAllocd*sizeof(unsigned long long));
		    
		    dontPrompt = 1;

		    forceNoNewline = 1;
		}
	    }
	} else if (strcmp(seashell_stringGet,"timedcall") == 0 || strcmp(seashell_stringGet,"tc") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

	    int numbTimes, storeTimes;
	    sscanf(seashell_stringGet,"%i",&numbTimes);
	    storeTimes = numbTimes;
	    
	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);
	    
	    numbGet = INTERNAL_getFuncByName(seashell_stringGet);
	    
	    if (numbGet == -1) {
		INTERNAL_appendToIO("**Error: function identifier unrecognized\n");
	    } else {
		int varGet, err = 0, pntr;
		
		for (int iter = 0; iter < seashell_funcs[numbGet].args; iter++)
		{
		    altLen = 0;
		    while (seashell_userInput[getLen] == ' ' && getLen < seashell_inputLen)
		    {
			getLen++;
		    }
		    if (seashell_userInput[getLen] == '&' && seashell_funcs[numbGet].argTypes[iter] == INTERNAL_getTypeByName("pointer")) {
			getLen += 1;
			pntr = 1;
		    } else {
			pntr = 0;
		    }
		    while (seashell_userInput[getLen] != ' ' && getLen < seashell_inputLen)
		    {
			seashell_stringGet[altLen] = seashell_userInput[getLen];
			altLen++; getLen++;
		    }
		    seashell_stringGet[altLen] = 0;

		    varGet = INTERNAL_getVarByName(seashell_stringGet);
		    
		    if (varGet == -1 && altLen == 0) {
		        INTERNAL_appendToIO("**Error: missing arguments, call 'desc <func>' to list information.\n");
			err = 1;
			break;
		    } else if (varGet == -1) {
			int point = 0;
		        for (int loc = 0; loc < altLen; loc++)
			{
			    if (seashell_stringGet[loc] == '.') {
				point = 1;
			    }
			}

			//repetitive? (not that I really care, people who harp on that are insane)
			
			if (seashell_types[seashell_funcs[numbGet].argTypes[iter]].members != 1) {
			    sprintf(stringOut,"**Error: cannot read argument %i, of array variety (passing arrays is unsupported).\n",iter+1);
			    INTERNAL_appendToIO(stringOut);
			    err = 1;
			    break;
			} else {
			    if (point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 8 &&
				seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
			        float get = 0.0;
				if (sscanf(seashell_stringGet,"%f",&get) == 0) {
				    sprintf(stringOut,"**Error: cannot read float argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    double new = (double) get;
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&new,8);
				}
			    } else if (point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 4 &&
				       seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				float get = 0.0;
				if (sscanf(seashell_stringGet,"%f",&get) == 0) {
				    sprintf(stringOut,"**Error: cannot read float argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,4);
				}
			    } else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 8 &&
				       !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				unsigned long long get;
				if (sscanf(seashell_stringGet,"%llu",&get) == 0) {
				    sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,8);
				}
			    } else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == -8 &&
				       !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				long long get;
				if (sscanf(seashell_stringGet,"%lli",&get) == 0) {
				    sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,8);
				}
			    } else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 4 &&
				       !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				unsigned long get;
				if (sscanf(seashell_stringGet,"%lu",&get) == 0) {
				    sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,4);
				}
			    } else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == -4 &&
				       !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				long get;
				if (sscanf(seashell_stringGet,"%li",&get) == 0) {
				    sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,4);
				}
			    } else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 2 &&
				       !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				unsigned short get;
				if (sscanf(seashell_stringGet,"%hu",&get) == 0) {
				    sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,2);
				}
			    } else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == -2 &&
				       !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				short get;
				if (sscanf(seashell_stringGet,"%hi",&get) == 0) {
				    sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,2);
				}
			    } else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == 1 &&
				       !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				unsigned short get;
				if (sscanf(seashell_stringGet,"%hu",&get) == 0) { //intentional shorts here
				    sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,1);
				}
			    } else if (!point && seashell_types[seashell_funcs[numbGet].argTypes[iter]].sizes[0] == -1 &&
				       !seashell_types[seashell_funcs[numbGet].argTypes[iter]].floaty[0]) {
				short get;
				if (sscanf(seashell_stringGet,"%hi",&get) == 0) { //same as above
				    sprintf(stringOut,"**Error: cannot read argument %i.\n",iter+1);
				    INTERNAL_appendToIO(stringOut);
				    err = 1;
				    break;
				} else {
				    seashell_args[iter].pntr = seashell_defaultArgs[iter];
				    memcpy(seashell_args[iter].pntr,&get,1);
				}
			    } else {
				sprintf(stringOut,"**Error: cannot read argument %i, of unsupported size for constant pass and unrecognized as variable.\n",iter);
				INTERNAL_appendToIO(stringOut);
				err = 1;
				break;
			    }
			}
		    } else {
			//typecheck

			if (seashell_vars[varGet].typeUse != seashell_funcs[numbGet].argTypes[iter] &&
			    (!pntr && seashell_funcs[numbGet].argTypes[iter] == INTERNAL_getTypeByName("pointer")) ) {
			    sprintf(stringOut,"**Error: argument %i of incorrect type, should be '%s'\n",iter+1,seashell_types[seashell_funcs[numbGet].argTypes[iter]].name);
			    INTERNAL_appendToIO(stringOut);
			    err = 1;
			    break;
			}
			
			//input

			if (pntr) {
			    memcpy(&seashell_args[iter],&seashell_vars[varGet],sizeof(seashell_var));
			    seashell_args[iter].pntr = seashell_defaultArgs[iter];
			    memcpy(seashell_args[iter].pntr,&(seashell_vars[varGet].pntr),8);
			} else {
			    memcpy(&seashell_args[iter],&seashell_vars[varGet],sizeof(seashell_var));
			    if (seashell_vars[varGet].amnt * seashell_types[seashell_vars[varGet].typeUse].sizeTot <= 8) {
				seashell_args[iter].pntr = seashell_defaultArgs[iter];
				memcpy(seashell_args[iter].pntr,seashell_vars[varGet].pntr,
				       seashell_vars[varGet].amnt * seashell_types[seashell_vars[varGet].typeUse].sizeTot);
			    }
			}
		    }
		}

	        seashell_return.typeUse = seashell_funcs[numbGet].typeReturn;
		
		if (!err) {
		    unsigned long long start = SDL_GetPerformanceCounter();
		    for (int iter = 0; iter < storeTimes; iter++)
		    {
			INTERNAL_funcCall(seashell_vars,seashell_funcs,seashell_types,numbGet,&seashell_return,seashell_args);
		    }
		    unsigned long long end = SDL_GetPerformanceCounter();

		    sprintf(stringOut,"Ticks Totl:  %llu\n",end-start);
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"Time Totl:   %.10f\n",(end-start+0.0)/(SDL_GetPerformanceFrequency() + 0.0));
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"Time Avg:    %.10f\n",(end-start+0.0)/((SDL_GetPerformanceFrequency()*storeTimes) + 0.0));
		    INTERNAL_appendToIO(stringOut);
		    sprintf(stringOut,"Execs/Sec:   %.10f\n\n",((SDL_GetPerformanceFrequency()*storeTimes) + 0.0)/(end-start+0.0));
		    INTERNAL_appendToIO(stringOut);
		    
		    if (seashell_funcs[numbGet].typeReturn != -1) {
			if (seashell_return.typeUse == INTERNAL_getTypeByName("pointer")) {
			    unsigned long long temp;
			    memcpy(&temp,&seashell_return.pntr,8);
			    sprintf(stringOut,"(pointer) %08x\n",temp);
			    INTERNAL_appendToIO(stringOut);
			} else {
			    INTERNAL_printVar(&seashell_return);
			}
		    }
		}
	    }
	} else {
	    INTERNAL_appendToIO(cmdUnrecd);
	}
	break;
    case '_':
	if (strcmp(seashell_stringGet,"_stats") == 0) {
	    sprintf(stringOut,"Approxomate Byte Usage: %lu\n\n",seashell_varsAllocd*sizeof(seashell_var) +
		                                               seashell_typesAllocd*sizeof(seashell_type) +
		                                               seashell_funcsAllocd*sizeof(seashell_func) +
		                                               io_max*2 + return_size_max +
		                                               (5000000));
	    INTERNAL_appendToIO(stringOut);
	    sprintf(stringOut,"Version: (infinity SIGMA n=0) 10^-n - 10^-(n+1)\n");
	    INTERNAL_appendToIO(stringOut);
	    sprintf(stringOut,"Compiled on:  %s\n\n",__DATE__);
	    INTERNAL_appendToIO(stringOut);
	    sprintf(stringOut,"-- @eternal_salmon wuz hear\n");
	    INTERNAL_appendToIO(stringOut);
	} else {
	    INTERNAL_appendToIO(cmdUnrecd);
	}
	break;
    case 'w':
	if (strcmp(seashell_stringGet,"watch") == 0 || strcmp(seashell_stringGet,"w") == 0) {

	    INTERNAL_parseWord(seashell_userInput,&getLen,seashell_inputLen,seashell_stringGet,&altLen);

	    numbGet = INTERNAL_getVarByName(seashell_stringGet);

	    if (numbGet == -1) {
		INTERNAL_appendToIO(varUnrecd);
	    } else {
		int returned = INTERNAL_addWatch(numbGet);

		if (returned == 1) {
		    INTERNAL_appendToIO("**Error: watch is full\n");
		} else if (returned == 2) {
		    INTERNAL_appendToIO("*Warning: variable is already in watch\n");
		}
	    }
	} else {
	    INTERNAL_appendToIO(cmdUnrecd);
	}
	break;
    case 0:
	break;
    default:
	INTERNAL_appendToIO(cmdUnrecd);
	break;
    }

    if (getLen && !forceNoNewline && !printThis) {
	INTERNAL_appendToIO("\n");
    }
    
    if (!dontPrompt && !printThis) {
	INTERNAL_appendToIO("@seashell~ ");
    }

    if (seashell_scrolled) {
	seashell_scrolled = 0;
	seashell_renderEvent = 2;
    }
    
    //clear user input + order full render

    memset(seashell_userInput,0,input_max);
    seashell_inputLen = 0;
    seashell_cursorNegative = 0;
    seashell_renderEvent = 2;
}

/*
**Documentation**

This section decompresses memory and makes the font with that.
*/

//works with embedded stuff

SDL_Surface *seashell_icon;
SDL_Surface *seashell_chars;
SDL_Texture *seashell_font;
char seashell_bigFont;

//simple byte-length rle

void INTERNAL_decompress(unsigned char* dest, unsigned char* from, int size)
{
    int loc = 0;

    for (int iter = 0; iter < size; iter += 2)
    {
	for (int iter1 = 0; iter1 < from[iter]; iter1++)
	{
	    dest[loc] = from[iter+1];
	    loc++;
	}
    }
}

void INTERNAL_makeFont()
{
    seashell_asciiData = malloc(seashell_sizeSmall);
    INTERNAL_decompress(seashell_asciiData,seashell_asciiSmall,sizeof(seashell_asciiSmall));
    
    SDL_RWops *io = SDL_RWFromMem(seashell_asciiData,seashell_sizeSmall);
    seashell_chars = SDL_LoadBMP_RW(io,1);
    seashell_font = SDL_CreateTextureFromSurface(seashell_render,seashell_chars);
    seashell_bigFont = 1;

    free(seashell_asciiData);
}

void INTERNAL_makeFontLarge()
{
    seashell_asciiDataLarge = malloc(seashell_sizeLarge);
    INTERNAL_decompress(seashell_asciiDataLarge,seashell_asciiLarge,sizeof(seashell_asciiLarge));
    
    SDL_RWops *io = SDL_RWFromMem(seashell_asciiDataLarge,seashell_sizeLarge);
    seashell_chars = SDL_LoadBMP_RW(io,1);
    seashell_font = SDL_CreateTextureFromSurface(seashell_render,seashell_chars);
    seashell_bigFont = 2;

    free(seashell_asciiDataLarge);
}

// ****USER STUFF****

/*
**Documentation**

User-level declaration funcs. This is the last one I wrote. I don't wanna describe these things anymore.
*/

#define defaultAlloc 64

int __declspec(dllexport) seashell_decl_var(void* data, char* typeUses, char* name)
{
    if (seashell_initted && !seashell_quitted) {
	seashell_sortVars = 1;
	
	if (seashell_varsAmnt >= seashell_varsAllocd) {
	    seashell_varsAllocd += defaultAlloc;
	    seashell_vars = realloc(seashell_vars,sizeof(seashell_var)*seashell_varsAllocd);
	}

	seashell_var fill;
	fill.pntr = data;
	fill.amnt = 1;
    
	fill.typeUse = INTERNAL_getTypeByName(typeUses);
	if (fill.typeUse == -1) {
	    char error[128];
	    sprintf(error,"**Error: variable '%s' declared with unknown type\n\n",name);
	    INTERNAL_appendToIO(error);
	    return 1;
	}
	
	strncpy(fill.name,name,identifier_max);
	if (strlen(name) >= identifier_max) {
	    char error[128];
	    sprintf(error,"*Warning: identifier '%s' too long, storing first %i bytes\n\n",name,identifier_max);
	    INTERNAL_appendToIO(error);
	}

	for (int iter = 0; iter < seashell_varsAmnt; iter++)
	{
	    if (seashell_vars[iter].name[0] == name[0]) {
		if (strcmp(seashell_vars[iter].name,fill.name) == 0) {
		    char error[128];
		    sprintf(error,"*Warning: variable identifier '%s' used twice!\n\n",name);
		    INTERNAL_appendToIO(error);
		    break;
		}
	    }
	}

	seashell_vars[seashell_varsAmnt] = fill;
	seashell_varsAmnt += 1;

	return 0;
    } else {
	return 1;
    }
}

void __declspec(dllexport) seashell_dedecl_var(char* name)
{
    int var = INTERNAL_getVarByName(name);
    if (var != -1) {
	memmove(&(seashell_vars[var]),&(seashell_vars[var+1]),(seashell_varsAllocd*sizeof(seashell_var)) - (seashell_varsAmnt*sizeof(seashell_var)));
	seashell_varsAmnt -= 1;
    }
}

void __declspec(dllexport) seashell_exec(char* command)
{
    seashell_inputLen = strlen(command);
    memcpy(seashell_userInput,command,seashell_inputLen);

    for (int iter = 0; iter < seashell_inputLen; iter++)
    {
	seashell_io[seashell_ioLen] = command[iter];
	seashell_ioLen++;
    }
    
    INTERNAL_enter(0);
}

int __declspec(dllexport) seashell_decl_var_array(void* data, int amnt, char* typeUses, char* name)
{
    if (seashell_initted && !seashell_quitted) {
	seashell_sortVars = 1;
	
	if (seashell_varsAmnt >= seashell_varsAllocd) {
	    seashell_varsAllocd += defaultAlloc;
	    seashell_vars = realloc(seashell_vars,sizeof(seashell_var)*seashell_varsAllocd);
	}

	seashell_var fill;
	fill.pntr = data;
	fill.amnt = amnt;

	fill.typeUse = INTERNAL_getTypeByName(typeUses);
	if (fill.typeUse == -1) {
	    char error[128];
	    sprintf(error,"**Error: variable '%s' declared with unknown type\n\n",name);
	    INTERNAL_appendToIO(error);
	    return 1;
	}
    
	strncpy(fill.name,name,identifier_max);
	if (strlen(name) >= identifier_max) {
	    char error[128];
	    sprintf(error,"*Warning: identifier '%s' too long, storing first %i bytes\n\n",name,identifier_max);
	    INTERNAL_appendToIO(error);
	}

	for (int iter = 0; iter < seashell_varsAmnt; iter++)
	{
	    if (seashell_vars[iter].name[0] == name[0]) {
		if (strcmp(seashell_vars[iter].name,fill.name) == 0) {
		    char error[128];
		    sprintf(error,"*Warning: variable identifier '%s' used twice!\n\n",name);
		    INTERNAL_appendToIO(error);
		    break;
		}
	    }
	}
	
	seashell_vars[seashell_varsAmnt] = fill;
	seashell_varsAmnt += 1;

	return 0;
    } else {
	return 1;
    }
}

void __declspec(dllexport) seashell_decl_func_simple(void* func, char* name)
{
    if (seashell_initted && !seashell_quitted) {
	seashell_sortFuncs = 1;
	
	if (seashell_funcsAmnt >= seashell_funcsAllocd) {
	    seashell_funcsAllocd += defaultAlloc;
	    seashell_funcs = realloc(seashell_funcs,sizeof(seashell_func)*seashell_funcsAllocd);
	}

	seashell_func fill;
	fill.pntr = func;

	strncpy(fill.name,name,identifier_max);
	if (strlen(name) >= identifier_max) {
	    char error[128];
	    sprintf(error,"*Warning: identifier '%s' too long, storing first %i bytes\n\n",name,identifier_max);
	    INTERNAL_appendToIO(error);
	}

	for (int iter = 0; iter < seashell_funcsAmnt; iter++)
	{
	    if (seashell_funcs[iter].name[0] == name[0]) {
		if (strcmp(seashell_funcs[iter].name,fill.name) == 0) {
		    char error[128];
		    sprintf(error,"*Warning: function identifier '%s' used twice!\n\n",name);
		    INTERNAL_appendToIO(error);
		    break;
		}
	    }
	}
	
	fill.args = 0;
	fill.typeReturn = -1;
	fill.argAmnts[0] = -1;

	seashell_funcs[seashell_funcsAmnt] = fill;
	seashell_funcsAmnt += 1;
    }
}

int __declspec(dllexport) seashell_decl_func(void* func, char* argType, char* typeRet, char* name)
{
    if (seashell_initted && !seashell_quitted) {
	seashell_sortFuncs = 1;
	
	if (seashell_funcsAmnt >= seashell_funcsAllocd) {
	    seashell_funcsAllocd += defaultAlloc;
	    seashell_funcs = realloc(seashell_funcs,sizeof(seashell_func)*seashell_funcsAllocd);
	}

	seashell_func fill;
	fill.pntr = func;

	strncpy(fill.name,name,identifier_max);
	if (strlen(name) >= identifier_max) {
	    char error[128];
	    sprintf(error,"*Warning: identifier '%s' too long, storing first %i bytes\n\n",name,identifier_max);
	    INTERNAL_appendToIO(error);
	}

	for (int iter = 0; iter < seashell_funcsAmnt; iter++)
	{
	    if (seashell_funcs[iter].name[0] == name[0]) {
		if (strcmp(seashell_funcs[iter].name,fill.name) == 0) {
		    char error[128];
		    sprintf(error,"*Warning: function identifier '%s' used twice!\n\n",name);
		    INTERNAL_appendToIO(error);
		    break;
		}
	    }
	}
	
        if (strlen(argType) != 0) {
	    char get[identifier_max + 50];
	    int on = 0, alt;
	    int len = strlen(argType);
	    int iter = 0;
	    while (on < strlen(argType))
	    {
		alt = 0;
		while (argType[on] == ' ' && on < len) {
		    on++;
		}
		while (argType[on] != ',' && on < len)
		{
		    get[alt] = argType[on];
		    if (argType[on] == ' ') {
			while (argType[on] == ' ') {
			    on++;
			}
		    } else {
			on++;
		    }
		    alt++;
		}
		if (get[alt-1] == ' ') {
		    get[alt-1] = 0;
		    on++;
		} else {
		    get[alt] = 0;
		    on++;
		}

		int getArgType = INTERNAL_getTypeByName(get);
	    
		if (getArgType == -1) {
		    char error[128];
		    sprintf(error,"**Error: '%s' function argument %i is of unknown type '%s'\n",name,iter,get);
		    INTERNAL_appendToIO(error);
		    return 1;
		} else {
		    fill.argTypes[iter] = getArgType;
		    fill.argAmnts[iter] = 1;
		    iter++;
		}
	    }
	    fill.args = iter;
	} else {
	    fill.args = 0;
	}
	
	int type = INTERNAL_getTypeByName(typeRet);

	if (type == -1 && strcmp(typeRet,"void") == 0) {
	    fill.typeReturn = -1;
	} else if (type == -1) {
	    char error[128];
	    sprintf(error,"**Error: '%s' function return is of unknown type\n\n",name);
	    INTERNAL_appendToIO(error);
	    return 1;
	} else {
	    fill.typeReturn = type;
	}

	fill.argAmnts[0] = -1;
	
	seashell_funcs[seashell_funcsAmnt] = fill;
	seashell_funcsAmnt += 1;
    
	return 0;
    } else {
	return 1;
    }
}
int __declspec(dllexport) seashell_decl_member_func(void* func, char* objType, char* argType, char* typeRet, char* name)
{
    if (seashell_initted && !seashell_quitted) {
	seashell_sortFuncs = 1;
	
	if (seashell_funcsAmnt >= seashell_funcsAllocd) {
	    seashell_funcsAllocd += defaultAlloc;
	    seashell_funcs = realloc(seashell_funcs,sizeof(seashell_func)*seashell_funcsAllocd);
	}

	seashell_func fill;
	fill.pntr = func;

	strncpy(fill.name,name,identifier_max);
	if (strlen(name) >= identifier_max) {
	    char error[128];
	    sprintf(error,"*Warning: identifier '%s' too long, storing first %i bytes\n\n",name,identifier_max);
	    INTERNAL_appendToIO(error);
	}

	for (int iter = 0; iter < seashell_funcsAmnt; iter++)
	{
	    if (seashell_funcs[iter].name[0] == name[0]) {
		if (strcmp(seashell_funcs[iter].name,fill.name) == 0) {
		    char error[128];
		    sprintf(error,"*Warning: function identifier '%s' used twice!\n\n",name);
		    INTERNAL_appendToIO(error);
		    break;
		}
	    }
	}

        fill.argAmnts[0] = INTERNAL_getTypeByName(objType);
	if (fill.argAmnts[0] == -1) {
	    char error[128];
	    sprintf(error,"**Error: '%s' function object is of unknown type '%s'\n",name,objType);
	    INTERNAL_appendToIO(error);
	    return 1;
	}
	
        if (strlen(argType) != 0) {
	    char get[identifier_max + 50];
	    int on = 0, alt;
	    int len = strlen(argType);
	    int iter = 0;
	    while (on < strlen(argType))
	    {
		alt = 0;
		while (argType[on] == ' ' && on < len) {
		    on++;
		}
		while (argType[on] != ',' && on < len)
		{
		    get[alt] = argType[on];
		    if (argType[on] == ' ') {
			while (argType[on] == ' ') {
			    on++;
			}
		    } else {
			on++;
		    }
		    alt++;
		}
		if (get[alt-1] == ' ') {
		    get[alt-1] = 0;
		    on++;
		} else {
		    get[alt] = 0;
		    on++;
		}

		int getArgType = INTERNAL_getTypeByName(get);
	    
		if (getArgType == -1) {
		    char error[128];
		    sprintf(error,"**Error: '%s' function argument %i is of unknown type '%s'\n",name,iter,get);
		    INTERNAL_appendToIO(error);
		    return 1;
		} else {
		    fill.argTypes[iter] = getArgType;
		    fill.argAmnts[iter] = 1;
		    iter++;
		}
	    }
	    fill.args = iter;
	} else {
	    fill.args = 0;
	}
	
	int type = INTERNAL_getTypeByName(typeRet);

	if (type == -1 && strcmp(typeRet,"void") == 0) {
	    fill.typeReturn = -1;
	} else if (type == -1) {
	    char error[128];
	    sprintf(error,"**Error: '%s' function return is of unknown type\n\n",name);
	    INTERNAL_appendToIO(error);
	    return 1;
	} else {
	    fill.typeReturn = type;
	}
    
	seashell_funcs[seashell_funcsAmnt] = fill;
	seashell_funcsAmnt += 1;
    
	return 0;
    } else {
	return 1;
    }
}

void __declspec(dllexport) seashell_decl_type_ints(char members, int* sizes, char* signs, char* name, int sizeTot)
{
    if (seashell_initted && !seashell_quitted) {
	seashell_sortTypes = 1;
	
	if (seashell_typesAmnt >= seashell_typesAllocd) {
	    seashell_typesAllocd += defaultAlloc;
	    seashell_types = realloc(seashell_types,sizeof(seashell_type)*seashell_typesAllocd);
	}

	seashell_type fill;
	fill.members = members;

	if (strlen(signs) != members) {
	    char error[128];
	    sprintf(error,"*Warning: sign definition does not match defined member amount\n\n");
	    INTERNAL_appendToIO(error);
	}
	
	int iter;
	for (iter = 0; iter < members; iter++)
	{
	    fill.sizes[iter] = sizes[iter];
	    if ((signs[iter]-'0' && sizes[iter] > 0) || (!(signs[iter]-'0') && sizes[iter] < 0)) {
		fill.sizes[iter] *= -1;
	    }
	}

	fill.sizeTot = sizeTot;
    
	strncpy(fill.name,name,identifier_max);
	if (strlen(name) > identifier_max) {
	    char error[128];
	    sprintf(error,"*Warning: identifier '%s' too long, storing first %i bytes\n\n",name,identifier_max);
	    INTERNAL_appendToIO(error);
	}

	for (int iter = 0; iter < seashell_typesAmnt; iter++)
	{
	    if (seashell_types[iter].name[0] == name[0]) {
		if (strcmp(seashell_types[iter].name,fill.name) == 0) {
		    char error[128];
		    sprintf(error,"*Warning: type identifier '%s' used twice!\n\n",name);
		    INTERNAL_appendToIO(error);
		    break;
		}
	    }
	}

	memset(fill.floaty,0,members_max);
    
	seashell_types[seashell_typesAmnt] = fill;
	seashell_typesAmnt += 1;
    }
}

void __declspec(dllexport) seashell_decl_type_mixed(char members, int* sizes, char* signs, char* floats, char* name, int sizeTot)
{
    if (seashell_initted && !seashell_quitted) {
	seashell_sortTypes = 1;
	
	if (seashell_typesAmnt >= seashell_typesAllocd) {
	    seashell_typesAllocd += defaultAlloc;
	    seashell_types = realloc(seashell_types,sizeof(seashell_type)*seashell_typesAllocd);
	}

	seashell_type fill;
	fill.members = members;

	if (strlen(signs) != members) {
	    char error[128];
	    sprintf(error,"*Warning: sign definition does not match defined member amount\n\n");
	    INTERNAL_appendToIO(error);
	}
	
	int iter;
	for (iter = 0; iter < members; iter++)
	{
	    fill.sizes[iter] = sizes[iter];
	    if ((signs[iter]-'0' && sizes[iter] > 0) || (!(signs[iter]-'0') && sizes[iter] < 0)) {
		fill.sizes[iter] *= -1;
	    }
	    if (floats[iter]-'0') {
		fill.sizes[iter] = abs(fill.sizes[iter]);
		fill.floaty[iter] = 1;
	    }
	}

	fill.sizeTot = sizeTot;
    
	strncpy(fill.name,name,identifier_max);
	if (strlen(name) > identifier_max) {
	    char error[128];
	    sprintf(error,"*Warning: identifier '%s' too long, storing first %i bytes\n\n",name,identifier_max);
	    INTERNAL_appendToIO(error);
	}

	for (int iter = 0; iter < seashell_typesAmnt; iter++)
	{
	    if (seashell_types[iter].name[0] == name[0]) {
		if (strcmp(seashell_types[iter].name,fill.name) == 0) {
		    char error[128];
		    sprintf(error,"*Warning: type identifier '%s' used twice!\n\n",name);
		    INTERNAL_appendToIO(error);
		    break;
		}
	    }
	}

	seashell_types[seashell_typesAmnt] = fill;
	seashell_typesAmnt += 1;
    }
}

/*
**Documentation**

Initialize the lib, mallocs and makes window.
*/

void __declspec(dllexport) seashell_init(char* id, int monitorSpawn, unsigned int bgColor, int largeMode, int takeInput)
{
    if (!seashell_initted) {
	seashell_initted = 1;
    
	// **internals**
    
	seashell_vars = malloc(sizeof(seashell_var)*defaultAlloc);
	seashell_varsAmnt = 0;
	seashell_varsAllocd = defaultAlloc;

	seashell_funcs = malloc(sizeof(seashell_func)*defaultAlloc);
	seashell_funcsAmnt = 0;
	seashell_funcsAllocd = defaultAlloc;

	seashell_types = malloc(sizeof(seashell_type)*defaultAlloc);
	seashell_typesAmnt = 0;
	seashell_typesAllocd = defaultAlloc;

	seashell_return.amnt = 1;
	seashell_return.pntr = malloc(return_size_max);

	seashell_stepsAllocd = defaultStepsAllocd;
	seashell_stepsAdd = malloc(seashell_stepsAllocd*sizeof(unsigned long long));
	seashell_stepsRec = malloc(seashell_stepsAllocd*sizeof(unsigned long long));
	memset(seashell_stepsAdd,0,seashell_stepsAllocd*sizeof(unsigned long long));
    
	seashell_renderBar.x = 0;
	seashell_renderBar.w = seashell_windowWidth;
	seashell_renderBar.y = seashell_windowHeight-(12*seashell_textHeight);
	seashell_renderBar.h = seashell_textHeight;
	seashell_defBar.x = 0;
	seashell_defBar.w = seashell_windowWidth;
	seashell_defBar.y = 0;
	seashell_defBar.h = seashell_textHeight;

	seashell_identifyBar.x = 0;
	seashell_identifyBar.w = seashell_windowWidth;
	seashell_identifyBar.h = seashell_textHeight;
    
	seashell_gBarsAllocd = 1000;
	seashell_gBars = malloc(seashell_gBarsAllocd*sizeof(SDL_Rect));
	seashell_gBarsNegative = malloc(seashell_gBarsAllocd);

	for (int iter = 0; iter < arguments_max; iter++)
	{
	    seashell_defaultArgs[iter] = malloc(8);
	}

	memset(seashell_recentInput,0,cmds_max*input_max);
    
	// *default types setup*

	int charSizeDecl[1] = {1};
	int shortSizeDecl[1] = {2};
	int intSizeDecl[1] = {4};
	int longSizeDecl[1] = {8};
    
	seashell_decl_type_ints(1,charSizeDecl,"1","char",sizeof(char));
	seashell_decl_type_ints(1,charSizeDecl,"0","unsigned char",sizeof(unsigned char));
	seashell_decl_type_ints(1,shortSizeDecl,"1","short",sizeof(short));
	seashell_decl_type_ints(1,shortSizeDecl,"0","unsigned short",sizeof(unsigned short));
	seashell_decl_type_ints(1,intSizeDecl,"1","int",sizeof(int));
	seashell_decl_type_ints(1,intSizeDecl,"0","unsigned int",sizeof(unsigned int));
	seashell_decl_type_ints(1,longSizeDecl,"1","long long",sizeof(long long));
	seashell_decl_type_ints(1,longSizeDecl,"0","unsigned long long",sizeof(unsigned long long));
	seashell_decl_type_mixed(1,intSizeDecl,"0","1","float",sizeof(float));
	seashell_decl_type_mixed(1,longSizeDecl,"0","1","double",sizeof(double));
	seashell_decl_type_ints(1,longSizeDecl,"0","pointer",8);
    
	// **SDL**

	SDL_Init(SDL_INIT_VIDEO);

	int totalDisplays = SDL_GetNumVideoDisplays(), xDetermine = 50, yDetermine = 50;
	SDL_Rect bounds;
	if (monitorSpawn < totalDisplays) {
	    SDL_GetDisplayBounds(monitorSpawn,&bounds);
	    xDetermine = bounds.x + bounds.w - (seashell_windowWidth + 50);
	}
    
	if (strlen(id) < 200) { //lazy, dont wanna malloc math
	    sprintf(seashell_title,"seashell | %s",id);
	} else {
	    sprintf(seashell_title,"seashell | think of a shorter title, weirdo. Unless you did that just to see this... OH WAIT THIS IS... just fine in length, you! STOP WORRYING!");
	}

	if (largeMode) {
	    seashell_window = SDL_CreateWindow(seashell_title,xDetermine,yDetermine,
					       2*seashell_windowWidth,2*seashell_windowHeight,0);
	} else {
	    seashell_window = SDL_CreateWindow(seashell_title,xDetermine,yDetermine,
					       seashell_windowWidth,seashell_windowHeight,0);
	}
	seashell_render = SDL_CreateRenderer(seashell_window,-1,0);

	if (largeMode) {
	    SDL_RenderSetScale(seashell_render,2.0,2.0);
	}
    
	if (bgColor == 0) {
	    bgColor = 0x216697f0;
	}
    
	unsigned char opacByte = bgColor;
	SDL_SetWindowOpacity(seashell_window,opacByte/255.0);

	seashell_keystate = SDL_GetKeyboardState(NULL);

	//extract color
    
	seashell_bgR = bgColor >> 24;
	seashell_bgG = bgColor >> 16;
	seashell_bgB = bgColor >> 8;

	if (seashell_bgR >= 205 || seashell_bgG >= 205 || seashell_bgB >= 205) {
	    INTERNAL_appendToIO("*Warning: Colors may be too bright to render certain things!\n");
	}

	//make icon + font

	SDL_RWops *io = SDL_RWFromMem(seashell_iconData,sizeof(seashell_iconData));
	seashell_icon = SDL_LoadBMP_RW(io,1);
	SDL_SetWindowIcon(seashell_window,seashell_icon);

	if (largeMode) {
	    INTERNAL_makeFontLarge();
	} else {
	    INTERNAL_makeFont();
	}

	seashell_fontLoc.x = 0;
	seashell_fontLoc.w = seashell_textWidth;
	seashell_fontLoc.y = 0;
	seashell_fontLoc.h = seashell_textHeight;
    
	seashell_fontGet.x = 0;
	seashell_fontGet.w = seashell_bigFont*seashell_textWidth;
	seashell_fontGet.y = 0;
	seashell_fontGet.h = seashell_bigFont*seashell_textHeight;

	// input

	seashell_hasInput = takeInput;
	
	// **startup**

	seashell_promptOnUpdate = 1;
	INTERNAL_appendToIO("\n");
    }
}

/*
**Documentation**

Force quit seashell. Equivalent to calling 'quit'.
*/

void __declspec(dllexport) seashell_quit()
{
    if (seashell_initted && !seashell_quitted) {
	seashell_quitted = 1;
	SDL_DestroyRenderer(seashell_render);
	SDL_DestroyWindow(seashell_window);
	free(seashell_vars);
	free(seashell_funcs);
	free(seashell_types);
    }
}

/*
**Documentation**

Called every frame to make seashell work. Does a bit more than I can describe.
*/

void __declspec(dllexport) seashell_update()
{
    if (seashell_initted && !seashell_quitted && seashell_holdsLeft <= 1) {
	if (seashell_sortAll) {
	    qsort(seashell_types,seashell_typesAmnt,sizeof(seashell_type),INTERNAL_compareTypes);
	    seashell_sortTypes = 0;
	    qsort(seashell_funcs,seashell_funcsAmnt,sizeof(seashell_func),INTERNAL_compareFuncs);
	    seashell_sortFuncs = 0;
	    qsort(seashell_vars,seashell_varsAmnt,sizeof(seashell_var),INTERNAL_compareFuncs);
	    seashell_sortVars = 0;
	    
	    seashell_sortAll = 0;
	}
	
	if (seashell_holdsLeft == 1 && !seashell_measuringSteps) {
	    seashell_holdsLeft--;
	    seashell_holdEnd = SDL_GetPerformanceCounter();
	    
	    char writeOut[256];
	    sprintf(writeOut,"Total Ticks:         %u\n",seashell_holdEnd-seashell_holdStart);
	    INTERNAL_appendToIO(writeOut);
	    sprintf(writeOut,"Ticks Per Second:    %u\n",SDL_GetPerformanceFrequency());
	    INTERNAL_appendToIO(writeOut);
	    sprintf(writeOut,"Total Seconds:       %f\n\n",(seashell_holdEnd-seashell_holdStart + 0.0) / (SDL_GetPerformanceFrequency() + 0.0));
	    INTERNAL_appendToIO(writeOut);
	    sprintf(writeOut,"Avg Seconds / Loop:  %.10f\n",((seashell_holdEnd-seashell_holdStart + 0.0) / (SDL_GetPerformanceFrequency() + 0.0)) / (seashell_holdsTotal + 0.0));
	    INTERNAL_appendToIO(writeOut);
	    sprintf(writeOut,"Avg Loops / Second:  %.10f\n\n",(seashell_holdsTotal + 0.0) / ((seashell_holdEnd-seashell_holdStart + 0.0) / (SDL_GetPerformanceFrequency() + 0.0)));
	    INTERNAL_appendToIO(writeOut);

	    seashell_holdsTotal = 1;
	    seashell_renderEvent = 2;
	    seashell_promptOnUpdate = 1;
	} else if (seashell_holdsLeft == 1 && seashell_measuringSteps) {
	    seashell_measuringSteps = 0;
	    seashell_holdsLeft--;
	    seashell_renderEvent = 2;
	    seashell_promptOnUpdate = 1;
	}
	
	if (seashell_promptOnUpdate) {
	    INTERNAL_appendToIO("@seashell~ ");
	    seashell_promptOnUpdate = 0;
	}
	
	if ((seashell_stepping && !seashell_calledFromStepping) || (!seashell_stepping && seashell_holding)) {
	    if (!seashell_stepping) {
		INTERNAL_deferToIO("loop");
	    } else {
		INTERNAL_deferToIO("-loop-");
	    }
	    seashell_renderEvent = 2;
        }
	
        seashell_oper = seashell_holding;
	
	do {
	    unsigned int start = SDL_GetTicks();
	    
	    // **input** (note internal feed functions: we filter keyboard input based upon window inputted into, whatever is left is placed back upon the event que)
	    
	    seashell_shifting = seashell_keystate[SDL_SCANCODE_LSHIFT] || seashell_keystate[SDL_SCANCODE_RSHIFT];
	    seashell_controlling = seashell_keystate[SDL_SCANCODE_LCTRL] || seashell_keystate[SDL_SCANCODE_RCTRL];
	    INTERNAL_cleanFeed();
	    INTERNAL_cleanFixer();
	    INTERNAL_cleanLinuxFixer();

	    while (SDL_PollEvent(&seashell_event))
	    {
		INTERNAL_feedEvent(seashell_event);
	        switch (seashell_event.type) {
		case SDL_WINDOWEVENT:
		    switch (seashell_event.window.event) {
		    case SDL_WINDOWEVENT_CLOSE:
		        exit(0);
		    }
		    break;
		case SDL_KEYDOWN:
		    break;
		}
	    }
	    
	    if (seashell_hasInput) {
		INTERNAL_useShiftFixer();
		INTERNAL_doLinuxFix();
	    }

	    if (seashell_keystate[SDL_GetScancodeFromKey(SDLK_F10)]) {
		if (!seashell_recentInputMod) {
		    seashell_hasInput = !seashell_hasInput;
		    seashell_recentInputMod = 1;
		    seashell_renderEvent = 2;
		}
	    } else {
		seashell_recentInputMod = 0;
	    }
	    
	    // **render**

	    int lineOn = 0, linesToRender, locOn, xCur, yCur, widthChars = seashell_windowWidth/seashell_textWidth, tempAdd, tempStore;
	    int first = 1, linesGot;

	    if (seashell_renderEvent == 1) {
		locOn = seashell_ioLen-1;
		while (seashell_io[locOn] != '\n' && locOn >= 0) {
		    locOn--;
		}
		tempStore = locOn - 1;
		locOn++;
	    
		if (seashell_linesRecent != INTERNAL_calculateLines(locOn,&xCur)) {
		    seashell_linesRecent = INTERNAL_calculateLines(locOn,&xCur);
		    seashell_renderEvent = 2;
		}
	    }

	    //standard renderevent draws
	    
	    if (seashell_renderEvent == 1) { //re-render just bottom text (maybe put into function due to repeating code?)
		if (seashell_renderVisualMode >= 0) {
		    linesToRender = 12;
		} else {
		    linesToRender = 47;
		}

		linesGot = INTERNAL_calculateLines(locOn, &xCur);
		lineOn += linesGot;

		if (first) {
		    seashell_fontLoc.w = seashell_windowWidth;
		    seashell_fontLoc.h = seashell_textHeight*linesGot + 1;
		    seashell_fontLoc.x = 0;
		    seashell_fontLoc.y = seashell_windowHeight - seashell_textHeight*linesGot - 1;
		    if (seashell_bgR <= 235 && seashell_bgG <= 235 && seashell_bgB <= 235) { //fix?: calculate only once
			SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 20,seashell_bgG + 20,seashell_bgB + 20,255);
		    }
		    SDL_RenderFillRect(seashell_render,&seashell_fontLoc);
		    seashell_fontLoc.w = seashell_textWidth;
		    seashell_fontLoc.h = seashell_textHeight;

		    if (seashell_bgR <= 215 && seashell_bgG <= 215 && seashell_bgB <= 215) { //fix?: calculate only once
			SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 40,seashell_bgG + 40,seashell_bgB + 40,255);
		    } else {
			SDL_SetRenderDrawColor(seashell_render,seashell_grayDraw,seashell_grayDraw,seashell_grayDraw,255);
		    }

		    //we need to add one to linesGot here to do the initial offset from windowheight
		    
		    seashell_fontLoc.y = seashell_windowHeight - seashell_textHeight*(linesGot+1) + 
			seashell_textHeight*INTERNAL_calculateLinesDefinedStop(locOn,seashell_ioLen - seashell_cursorNegative,&xCur);
		    seashell_fontLoc.x = (xCur)*seashell_textWidth;

		    SDL_RenderFillRect(seashell_render,&seashell_fontLoc);
		    first = 0;
		}
		
	        yCur = -lineOn;
		xCur = 0;
		
		while (seashell_io[locOn] != '\n' && locOn < seashell_ioLen) {
		    if (xCur >= widthChars) {
			yCur += 1;
			xCur = 0;
		    } else if (xCur > widthChars-identifier_max && seashell_io[locOn] == ' ' && seashell_io[locOn+1] != ' ') {
			tempAdd = 1;
			while (seashell_io[locOn+tempAdd] != ' ' && seashell_io[locOn+tempAdd] != '\n' && locOn+tempAdd < seashell_ioLen) {
			    tempAdd += 1;
			}
			if (xCur+tempAdd-1 >= widthChars) {
			    yCur += 1;
			    xCur = 0;
			} else {
			    xCur += 1;
			}
		    } else {
			if (seashell_io[locOn] >= '!' && seashell_io[locOn] <= '~') {
			    seashell_fontLoc.x = xCur*seashell_textWidth;
			    seashell_fontLoc.y = yCur*seashell_textHeight + seashell_windowHeight;
			    seashell_fontGet.x = seashell_bigFont*seashell_textWidth*(seashell_io[locOn] - '!');
			    SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
			}
			xCur++;
		    }
		    locOn++;
		}
		
		SDL_RenderPresent(seashell_render);
		
		seashell_renderEvent = 0;
	    } else if (seashell_renderEvent == 2) { //re-render everything
		SDL_SetRenderDrawColor(seashell_render,seashell_bgR,seashell_bgG,seashell_bgB,255);
		SDL_RenderClear(seashell_render);

		// *text*
	
		if (seashell_renderVisualMode >= 0) {
		    linesToRender = 12;
		} else {
		    linesToRender = 47;
		}

		locOn = seashell_ioLen-1;

		int scrollMeasure = 0;
		while (scrollMeasure < seashell_scrolled) {
		    while (seashell_io[locOn] != '\n' && locOn >= 0) {
			locOn--;
		    }
		    locOn--;
		    scrollMeasure++;
		    
		    if (locOn < 0) {
			seashell_scrolled--;
			locOn = 0;
			while (seashell_io[locOn] != '\n' && locOn < seashell_ioLen) {
			    locOn++;
			}
			break;
		    }
		}
		
		while (lineOn < linesToRender) {
		    while (seashell_io[locOn] != '\n' && locOn >= 0) {
			locOn--;
		    }
		    tempStore = locOn - 1;
		    locOn++;

		    linesGot = INTERNAL_calculateLines(locOn, &xCur);
		    lineOn += linesGot;
		    
		    if (!first || seashell_scrolled) { //MAGIC COLORS!
			seashell_identifyBar.y = seashell_windowHeight - seashell_textHeight*lineOn;
			seashell_identifyBar.h = linesGot*seashell_textHeight;
			if (seashell_io[locOn] == '@' && !(seashell_io[locOn+1] == 's' && seashell_io[locOn+2] == 'e' && seashell_io[locOn+3] == 'a' && seashell_io[locOn+4] == 's' && seashell_io[locOn+5] == 'h' && seashell_io[locOn+6] == 'e' && seashell_io[locOn+7] == 'l' && seashell_io[locOn+8] == 'l' && seashell_io[locOn+9] == '~' && seashell_io[locOn+10] == ' ' && seashell_io[locOn+11] == '\n')) {
			    if (seashell_bgR <= 243 && seashell_bgG <= 243 && seashell_bgB <= 243) {
				SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 12,seashell_bgG + 12,seashell_bgB + 12,255);
			    }
			    SDL_RenderFillRect(seashell_render,&seashell_identifyBar);
			} else if (seashell_io[locOn] == '*' && seashell_io[locOn+1] != '*') {
			    if (seashell_bgB >= seashell_bgG && seashell_bgB >= seashell_bgR) {
				if (seashell_bgR <= 240) {
				    SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 15,seashell_bgG,seashell_bgB,255);
				}
			    } else {
				if (seashell_bgB <= 233) {
				    SDL_SetRenderDrawColor(seashell_render,seashell_bgR,seashell_bgG,seashell_bgB + 22,255);
				}
			    }
			    SDL_RenderFillRect(seashell_render,&seashell_identifyBar);
			} else if (seashell_io[locOn] == '*' && seashell_io[locOn+1] == '*') {
			    if (seashell_bgB >= seashell_bgG && seashell_bgB >= seashell_bgR) {
				if (seashell_bgR <= 235) {
				    SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 20,seashell_bgG,seashell_bgB,255);
				}
			    } else {
				if (seashell_bgB <= 225) {
				    SDL_SetRenderDrawColor(seashell_render,seashell_bgR,seashell_bgG,seashell_bgB + 30,255);
				}
			    }
			    SDL_RenderFillRect(seashell_render,&seashell_identifyBar);
			} else if (seashell_io[locOn] == '-' && seashell_io[locOn+1] != '-' && seashell_io[locOn+2] != '-') {
			    if (abs(seashell_bgR - seashell_bgG) < 30 || (seashell_bgB >= seashell_bgG && seashell_bgB >= seashell_bgR)) {
				if (seashell_bgG <= 240) {
				    SDL_SetRenderDrawColor(seashell_render,seashell_bgR,seashell_bgG + 15,seashell_bgB,255);
				}
			    } else {
				if (seashell_bgR <= 247 && seashell_bgG <= 247 && seashell_bgB <= 240) {
				    SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 8,seashell_bgG + 8,seashell_bgB + 15,255);
				}
			    }
			    SDL_RenderFillRect(seashell_render,&seashell_identifyBar);
			} else if (seashell_io[locOn] == '-' && seashell_io[locOn+1] == '-' && seashell_io[locOn+2] != '-') {
			    if (abs(seashell_bgR - seashell_bgG) < 30 || (seashell_bgB >= seashell_bgG && seashell_bgB >= seashell_bgR)) {
				if (seashell_bgG <= 233) {
				    SDL_SetRenderDrawColor(seashell_render,seashell_bgR,seashell_bgG + 22,seashell_bgB,255);
				}
			    } else {
				if (seashell_bgR <= 245 && seashell_bgG <= 245 && seashell_bgB <= 235) {
				    SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 10,seashell_bgG + 10,seashell_bgB + 20,255);
				}
			    }
			    SDL_RenderFillRect(seashell_render,&seashell_identifyBar);
			} else if (seashell_io[locOn] == '-' && seashell_io[locOn+1] == '-' && seashell_io[locOn+2] == '-') {
			    if (abs(seashell_bgR - seashell_bgG) < 30 || (seashell_bgB >= seashell_bgG && seashell_bgB >= seashell_bgR)) {
				if (seashell_bgG <= 225) {
				    SDL_SetRenderDrawColor(seashell_render,seashell_bgR,seashell_bgG + 30,seashell_bgB,255);
				}
			    } else {
				if (seashell_bgR <= 238 && seashell_bgG <= 238 && seashell_bgB <= 225) {
				    SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 17,seashell_bgG + 17,seashell_bgB + 30,255);
				}
			    }
			    SDL_RenderFillRect(seashell_render,&seashell_identifyBar);
			}
		    }
		    
		    if (first && !seashell_scrolled) {
			seashell_fontLoc.w = seashell_windowWidth;
			seashell_fontLoc.h = seashell_textHeight*linesGot + 1;
			seashell_fontLoc.x = 0;
			seashell_fontLoc.y = seashell_windowHeight - seashell_textHeight*linesGot - 1;
			if (seashell_bgR <= 235 && seashell_bgG <= 235 && seashell_bgB <= 235) { //fix?: calculate only once
			    SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 20,seashell_bgG + 20,seashell_bgB + 20,255);
			}
			SDL_RenderFillRect(seashell_render,&seashell_fontLoc);
			seashell_fontLoc.w = seashell_textWidth;
			seashell_fontLoc.h = seashell_textHeight;

			if (seashell_bgR <= 215 && seashell_bgG <= 215 && seashell_bgB <= 215) { //fix?: calculate only once
			    SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 40,seashell_bgG + 40,seashell_bgB + 40,255);
			} else {
			    SDL_SetRenderDrawColor(seashell_render,seashell_grayDraw,seashell_grayDraw,seashell_grayDraw,255);
			}
			
		        seashell_fontLoc.y = seashell_windowHeight - seashell_textHeight*(linesGot+1) + 
			seashell_textHeight*INTERNAL_calculateLinesDefinedStop(locOn,seashell_ioLen - seashell_cursorNegative,&xCur);
			seashell_fontLoc.x = (xCur)*seashell_textWidth;
			
			SDL_RenderFillRect(seashell_render,&seashell_fontLoc);
			first = 0;
		    }
		    
		    if (lineOn < linesToRender) {
			yCur = -lineOn;
			xCur = 0;
			
			while (seashell_io[locOn] != '\n' && locOn < seashell_ioLen) {
			    if (xCur >= widthChars) {
				yCur += 1;
				xCur = 0;
			    } else if (xCur > widthChars-identifier_max && seashell_io[locOn] == ' ' && seashell_io[locOn+1] != ' ') {
				tempAdd = 1;
				while (seashell_io[locOn+tempAdd] != ' ' && seashell_io[locOn+tempAdd] != '\n' && locOn+tempAdd < seashell_ioLen) {
				    tempAdd += 1;
				}
				if (xCur+tempAdd-1 >= widthChars) {
				    yCur += 1;
				    xCur = 0;
				} else {
				    xCur += 1;
				}
			    } else {
				if (seashell_io[locOn] >= '!' && seashell_io[locOn] <= '~') {
				    seashell_fontLoc.x = xCur*seashell_textWidth;
				    seashell_fontLoc.y = yCur*seashell_textHeight + seashell_windowHeight;
				    seashell_fontGet.x = seashell_bigFont*seashell_textWidth*(seashell_io[locOn] - '!');
				    SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
				}
				xCur++;
			    }
			    locOn++;
			}
		    }

		    locOn = tempStore;
		
		    if (locOn <= 0) {
			break;
		    }
		}
	
		// *graphs*
	
		if (seashell_renderVisualMode >= 0) { //we had more stuff, once upon a tim ;(
		    if (seashell_bgR <= 235 && seashell_bgG <= 235 && seashell_bgB <= 235) { //fix?: calculate only once
			SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 20,seashell_bgG + 20,seashell_bgB + 20,255);
		    }
		    SDL_RenderFillRect(seashell_render,&seashell_renderBar);

		    if (seashell_bgR <= 205 && seashell_bgG <= 205 && seashell_bgB <= 205) { //same as above?
			SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 50,seashell_bgG + 50,seashell_bgB + 50,255);
		    }
		    SDL_RenderDrawLine(seashell_render,0,seashell_windowHeight-(12*seashell_textHeight)+8,
				       seashell_windowWidth,seashell_windowHeight-(12*seashell_textHeight)+8);
		} else {
		    if (seashell_bgR <= 235 && seashell_bgG <= 235 && seashell_bgB <= 235) { //fix?: calculate only once
			SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 20,seashell_bgG + 20,seashell_bgB + 20,255);
		    }
		    SDL_RenderFillRect(seashell_render,&seashell_defBar);

		    if (seashell_bgR <= 205 && seashell_bgG <= 205 && seashell_bgB <= 205) { //same as above?
			SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 50,seashell_bgG + 50,seashell_bgB + 50,255);
		    }
		    SDL_RenderDrawLine(seashell_render,0,8,seashell_windowWidth,8);
		}
        
	        if (seashell_renderVisualMode == 0) {
		    if (seashell_bgR <= 235 && seashell_bgG <= 235 && seashell_bgB <= 235) {
			SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 20,seashell_bgG + 20,seashell_bgB + 20,255);
		    } else {
			SDL_SetRenderDrawColor(seashell_render,seashell_dgrayDraw,seashell_dgrayDraw,seashell_dgrayDraw,255);
		    }

		    for (int iter = 0; iter < 16; iter++)
		    {
			SDL_RenderDrawLine(seashell_render,2*seashell_textWidth,seashell_textHeight + (iter*32),
					   seashell_windowWidth - 2*seashell_textWidth,seashell_textHeight + (iter*32));
			SDL_RenderDrawLine(seashell_render,2*seashell_textWidth + 46 + (iter*46),seashell_textHeight,
					   2*seashell_textWidth + 46 + (iter*46),seashell_windowHeight-(14*seashell_textHeight));
		    }

		    for (int iter = 0; iter < seashell_gBarsDraw; iter++)
		    {
			if (seashell_gBarsNegative[iter]) {
			    if (seashell_bgR <= 215 && seashell_bgG <= 215 && seashell_bgB <= 215) {
				SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 40,
						       seashell_bgG + 5*(3-(iter % 4)),
						       seashell_bgB + 5*(3-(iter % 4)),255);
				SDL_RenderFillRect(seashell_render,&(seashell_gBars[iter]));
			    } else {
				SDL_SetRenderDrawColor(seashell_render,seashell_dgrayDraw + 40,seashell_dgrayDraw,seashell_dgrayDraw,255);
				SDL_RenderFillRect(seashell_render,&(seashell_gBars[iter]));
			    }
			} else {
			    if (seashell_bgR <= 215 && seashell_bgG <= 215 && seashell_bgB <= 215) {
				SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 10 + 10*(3-(iter % 4)),
						       seashell_bgG + 10 + 10*(3-(iter % 4)),
						       seashell_bgB + 10 + 10*(3-(iter % 4)),255);
				SDL_RenderFillRect(seashell_render,&(seashell_gBars[iter]));
			    } else {
				SDL_SetRenderDrawColor(seashell_render,seashell_dgrayDraw,seashell_dgrayDraw,seashell_dgrayDraw,255);
				SDL_RenderFillRect(seashell_render,&(seashell_gBars[iter]));
			    }
			}
		    }
		    
		    SDL_SetRenderDrawColor(seashell_render,seashell_grayDraw,seashell_grayDraw,seashell_grayDraw,255);
		    SDL_RenderDrawLine(seashell_render,2*seashell_textWidth,seashell_textHeight,
				       2*seashell_textWidth,seashell_windowHeight-(14*seashell_textHeight));
		    SDL_RenderDrawLine(seashell_render,2*seashell_textWidth,seashell_windowHeight-(14*seashell_textHeight),
				       seashell_windowWidth - 2*seashell_textWidth,seashell_windowHeight-(14*seashell_textHeight));

		    //text

		    char numb[32];
		    sprintf(numb,"%0.2f",seashell_gBarsHVal);
		    int len = strlen(numb);

		    if (len > 8) {
			for (int sub = 0; sub < 3; sub++)
			{
			    numb[5+sub] = '.';
		        }
			len = 8;
		    }

		    if (seashell_gBarsHeight < 528) { // we actually have something with height, fixes zeroing error
			seashell_fontLoc.y = seashell_gBarsHeight - ((len/2.0)*seashell_textHeight);
			seashell_fontLoc.x = 4;
			for (int sub = 0; sub < len; sub++)
			{
			    seashell_fontGet.x = seashell_bigFont*seashell_textWidth*(numb[sub] - '!');
			    SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
			    seashell_fontLoc.y += seashell_textHeight;
			}

			float new = seashell_gBarsHVal/2.0;
			sprintf(numb,"%0.2f",new);
			len = strlen(numb);

			if (len > 8) {
			    for (int sub = 0; sub < 3; sub++)
			    {
				numb[5+sub] = '.';
			    }
			    len = 8;
			}
		    
			seashell_fontLoc.y = seashell_gBarsHeightHalf - ((len/2.0)*seashell_textHeight);
			seashell_fontLoc.x = 4;
			for (int sub = 0; sub < len; sub++)
			{
			    seashell_fontGet.x = seashell_bigFont*seashell_textWidth*(numb[sub] - '!');
			    SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
			    seashell_fontLoc.y += seashell_textHeight;
			}
		    }

		    sprintf(numb,"%03i",seashell_gBarsDraw);

		    seashell_fontLoc.y = 532;
		    seashell_fontLoc.x = seashell_gBarsWidth - (1.5*seashell_textWidth);
		    seashell_fontGet.x = seashell_bigFont*seashell_textWidth*(numb[0] - '!');
		    SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
		    seashell_fontLoc.x += seashell_textWidth;
		    seashell_fontGet.x = seashell_bigFont*seashell_textWidth*(numb[1] - '!');
		    SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
		    seashell_fontLoc.x += seashell_textWidth;
		    seashell_fontGet.x = seashell_bigFont*seashell_textWidth*(numb[2] - '!');
		    SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);

		    int newi = (seashell_gBarsDraw)/(2);
		    if (newi != 0) { //if there is more than one bar (case of countgraph presents the issue)
			sprintf(numb,"%03i",newi);

			seashell_fontLoc.y = 532;
			seashell_fontLoc.x = seashell_gBarsWidthHalf - (1.5*seashell_textWidth);
			seashell_fontGet.x = seashell_bigFont*seashell_textWidth*(numb[0] - '!');
			SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
			seashell_fontLoc.x += seashell_textWidth;
			seashell_fontGet.x = seashell_bigFont*seashell_textWidth*(numb[1] - '!');
			SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
			seashell_fontLoc.x += seashell_textWidth;
			seashell_fontGet.x = seashell_bigFont*seashell_textWidth*(numb[2] - '!');
			SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
		    }
			
		    seashell_fontLoc.y = 532;
		    seashell_fontLoc.x = 4;
		    seashell_fontGet.x = seashell_bigFont*seashell_textWidth*('0' - '!');
		    SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
		}		

		if (!seashell_hasInput) { //draws border to show if we have input
		    SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 50,seashell_bgG + 50,seashell_bgB + 50,255);
		    SDL_RenderDrawLine(seashell_render,0,0,seashell_windowWidth,0);
		    SDL_RenderDrawLine(seashell_render,0,1,seashell_windowWidth,1);
		    SDL_RenderDrawLine(seashell_render,0,2,seashell_windowWidth,2);
		    SDL_RenderDrawLine(seashell_render,0,3,seashell_windowWidth,3);
		    SDL_RenderDrawLine(seashell_render,0,0,0,seashell_windowHeight);
		    SDL_RenderDrawLine(seashell_render,1,0,1,seashell_windowHeight);
		    SDL_RenderDrawLine(seashell_render,2,0,2,seashell_windowHeight);
		    SDL_RenderDrawLine(seashell_render,3,0,3,seashell_windowHeight);
		    SDL_RenderDrawLine(seashell_render,0,seashell_windowHeight-1,seashell_windowWidth,seashell_windowHeight-1);
		    SDL_RenderDrawLine(seashell_render,0,seashell_windowHeight-2,seashell_windowWidth,seashell_windowHeight-2);
		    SDL_RenderDrawLine(seashell_render,0,seashell_windowHeight-3,seashell_windowWidth,seashell_windowHeight-3);
		    SDL_RenderDrawLine(seashell_render,0,seashell_windowHeight-4,seashell_windowWidth,seashell_windowHeight-4);
		    SDL_RenderDrawLine(seashell_render,seashell_windowWidth-1,0,seashell_windowWidth-1,seashell_windowHeight);
		    SDL_RenderDrawLine(seashell_render,seashell_windowWidth-2,0,seashell_windowWidth-2,seashell_windowHeight);
		    SDL_RenderDrawLine(seashell_render,seashell_windowWidth-3,0,seashell_windowWidth-3,seashell_windowHeight);
		    SDL_RenderDrawLine(seashell_render,seashell_windowWidth-4,0,seashell_windowWidth-4,seashell_windowHeight);
		}
		
		SDL_RenderPresent(seashell_render);

		seashell_renderEvent = 0;
	    }

	    if (seashell_watching) {
	        int lenTotal = seashell_watchTxtLen + seashell_watchBytLen + 5;
		int heightMod = (seashell_renderVisualMode >= 0)*seashell_textHeight*35;

		//first highlight
		
		if (seashell_bgR <= 250 && seashell_bgG <= 250 && seashell_bgB <= 250) {
		    SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 5,seashell_bgG + 5,seashell_bgB + 5,255);
		}
	        seashell_watchBar.x = seashell_windowWidth - (lenTotal-1)*seashell_textWidth;
		seashell_watchBar.w = (lenTotal-2)*seashell_textWidth;
		seashell_watchBar.y = heightMod + seashell_textHeight;
		seashell_watchBar.h = seashell_textHeight;
		SDL_RenderFillRect(seashell_render,&seashell_watchBar);
		seashell_watchBar.x = seashell_windowWidth - (lenTotal-1)*seashell_textWidth;
		seashell_watchBar.w = (lenTotal-2)*seashell_textWidth;
		seashell_watchBar.y = heightMod + seashell_textHeight*4;
		seashell_watchBar.h = seashell_textHeight;
		SDL_RenderFillRect(seashell_render,&seashell_watchBar);

		//second highlight

		if (seashell_bgR <= 245 && seashell_bgG <= 245 && seashell_bgB <= 245) {
		    SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 10,seashell_bgG + 10,seashell_bgB + 10,255);
		}
	        seashell_watchBar.x = seashell_windowWidth - (lenTotal-1)*seashell_textWidth;
		seashell_watchBar.w = (lenTotal-2)*seashell_textWidth;
		seashell_watchBar.y = heightMod + seashell_textHeight*2;
		seashell_watchBar.h = seashell_textHeight;
		SDL_RenderFillRect(seashell_render,&seashell_watchBar);
		seashell_watchBar.x = seashell_windowWidth - (lenTotal-1)*seashell_textWidth;
		seashell_watchBar.w = (lenTotal-2)*seashell_textWidth;
		seashell_watchBar.y = heightMod + seashell_textHeight*5;
		seashell_watchBar.h = seashell_textHeight;
		SDL_RenderFillRect(seashell_render,&seashell_watchBar);
		
		//third highlight

		if (seashell_bgR <= 240 && seashell_bgG <= 240 && seashell_bgB <= 240) {
		    SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 15,seashell_bgG + 15,seashell_bgB + 15,255);
		}
	        seashell_watchBar.x = seashell_windowWidth - (lenTotal-1)*seashell_textWidth;
		seashell_watchBar.w = (lenTotal-2)*seashell_textWidth;
		seashell_watchBar.y = heightMod + seashell_textHeight*3;
		seashell_watchBar.h = seashell_textHeight;
		SDL_RenderFillRect(seashell_render,&seashell_watchBar);
		seashell_watchBar.x = seashell_windowWidth - (lenTotal-1)*seashell_textWidth;
		seashell_watchBar.w = (lenTotal-2)*seashell_textWidth;
		seashell_watchBar.y = heightMod + seashell_textHeight*6;
		seashell_watchBar.h = seashell_textHeight;
		SDL_RenderFillRect(seashell_render,&seashell_watchBar);
		
		//surround
		
		if (seashell_bgR <= 230 && seashell_bgG <= 230 && seashell_bgB <= 230) {
		    SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 25,seashell_bgG + 25,seashell_bgB + 25,255);
		}
		
		seashell_watchBar.x = seashell_windowWidth - lenTotal*seashell_textWidth;
		seashell_watchBar.w = lenTotal*seashell_textWidth;
		seashell_watchBar.y = heightMod;
		seashell_watchBar.h = seashell_textHeight;
		SDL_RenderFillRect(seashell_render,&seashell_watchBar);

		seashell_watchBar.x = seashell_windowWidth - lenTotal*seashell_textWidth;
		seashell_watchBar.w = seashell_textWidth;
		seashell_watchBar.y = heightMod;
		seashell_watchBar.h = seashell_textHeight*8;
		SDL_RenderFillRect(seashell_render,&seashell_watchBar);

		seashell_watchBar.x = seashell_windowWidth - seashell_textWidth;
		seashell_watchBar.w = seashell_textWidth;
		seashell_watchBar.y = heightMod;
		seashell_watchBar.h = seashell_textHeight*8;
		SDL_RenderFillRect(seashell_render,&seashell_watchBar);

		seashell_watchBar.x = seashell_windowWidth - lenTotal*seashell_textWidth;
		seashell_watchBar.w = lenTotal*seashell_textWidth;
		seashell_watchBar.y = heightMod + seashell_textHeight*7;
		seashell_watchBar.h = seashell_textHeight;
		SDL_RenderFillRect(seashell_render,&seashell_watchBar);

		if (seashell_bgR <= 205 && seashell_bgG <= 205 && seashell_bgB <= 205) {
		    SDL_SetRenderDrawColor(seashell_render,seashell_bgR + 50,seashell_bgG + 50,seashell_bgB + 50,255);
		}
		
		SDL_RenderDrawLine(seashell_render,
				   seashell_windowWidth - lenTotal*seashell_textWidth + 4,
				   heightMod + seashell_textHeight/2,
				   seashell_windowWidth - lenTotal*seashell_textWidth + 4,
				   heightMod + seashell_textHeight*8 - seashell_textHeight/2);
		SDL_RenderDrawLine(seashell_render,
				   seashell_windowWidth - lenTotal*seashell_textWidth,
				   heightMod + seashell_textHeight/2,
				   seashell_windowWidth - 6*seashell_textWidth - 4,
				   heightMod + seashell_textHeight/2);
		SDL_RenderDrawLine(seashell_render,
				   seashell_windowWidth - lenTotal*seashell_textWidth + 4,
				   heightMod + seashell_textHeight*8 - seashell_textHeight/2,
				   seashell_windowWidth - 4,
				   heightMod + seashell_textHeight*8 - seashell_textHeight/2);
		SDL_RenderDrawLine(seashell_render,
				   seashell_windowWidth - 4,
				   heightMod + seashell_textHeight/2,
				   seashell_windowWidth - 4,
				   heightMod + seashell_textHeight*8 - seashell_textHeight/2);

		seashell_fontLoc.y = heightMod;
		seashell_fontLoc.x = seashell_windowWidth - 6*seashell_textWidth;
		seashell_fontGet.x = seashell_bigFont*seashell_textWidth*('w' - '!');
		SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
		seashell_fontLoc.x += seashell_textWidth;
		seashell_fontGet.x = seashell_bigFont*seashell_textWidth*('a' - '!');
		SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
		seashell_fontLoc.x += seashell_textWidth;
		seashell_fontGet.x = seashell_bigFont*seashell_textWidth*('t' - '!');
		SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
		seashell_fontLoc.x += seashell_textWidth;
		seashell_fontGet.x = seashell_bigFont*seashell_textWidth*('c' - '!');
		SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
		seashell_fontLoc.x += seashell_textWidth;
		seashell_fontGet.x = seashell_bigFont*seashell_textWidth*('h' - '!');
		SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
		seashell_fontLoc.x += seashell_textWidth;
		
		//watches
		
		for (int iter = 0; iter < seashell_watching; iter++)
		{
		    int elipse = 0;
		    int len = strlen(seashell_vars[seashell_watch[iter]].name);
		    if (len > 17) {
			elipse = 1;
			len = 17;
		    }

		    seashell_fontLoc.y = heightMod + seashell_textHeight*iter + seashell_textHeight;
		    
		    for (int on = 0; on < len; on++)
		    {
			seashell_fontLoc.x = seashell_windowWidth - lenTotal*seashell_textWidth + seashell_textWidth + seashell_textWidth*on;
		        seashell_fontGet.x = seashell_bigFont*seashell_textWidth*(seashell_vars[seashell_watch[iter]].name[on] - '!');
			SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
		    }

		    if (elipse) {
		        seashell_fontGet.x = seashell_textWidth*('.' - '!');

			seashell_fontLoc.x += seashell_textWidth;
			SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
			seashell_fontLoc.x += seashell_textWidth;
			SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
			seashell_fontLoc.x += seashell_textWidth;
			SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
		    }

		    seashell_fontLoc.x = seashell_windowWidth - lenTotal*seashell_textWidth + seashell_textWidth + seashell_textWidth*21;

		    seashell_fontGet.x = seashell_bigFont*seashell_textWidth*('[' - '!');
		    SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
		    seashell_fontLoc.x += seashell_textWidth;

		    int bytes = seashell_vars[seashell_watch[iter]].amnt * seashell_types[seashell_vars[seashell_watch[iter]].typeUse].sizeTot;
		    if (bytes > 8) {
			bytes = 8;
		    }

		    unsigned long long get = 0;
		    memcpy(&get,seashell_vars[seashell_watch[iter]].pntr,bytes);

		    if (get > seashell_watchRec[iter]) {
			seashell_fontGet.x = seashell_bigFont*seashell_textWidth*('+' - '!');
			SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
			seashell_fontLoc.x += seashell_textWidth;
		    } else if (get < seashell_watchRec[iter]) {
			seashell_fontGet.x = seashell_bigFont*seashell_textWidth*('-' - '!');
			SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
			seashell_fontLoc.x += seashell_textWidth;
		    } else {
			seashell_fontGet.x = seashell_bigFont*seashell_textWidth*('=' - '!');
			SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
			seashell_fontLoc.x += seashell_textWidth;
		    }

		    seashell_watchRec[iter] = get;
			
		    seashell_fontGet.x = seashell_bigFont*seashell_textWidth*(']' - '!');
		    SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
		    seashell_fontLoc.x += seashell_textWidth;

		    seashell_fontLoc.x += seashell_textWidth;
		    
		    sprintf(seashell_watchStrings,"%08x",get);

		    for (int on = 0; on < 8; on++)
		    {
			seashell_fontGet.x = seashell_bigFont*seashell_textWidth*(seashell_watchStrings[on] - '!');
			SDL_RenderCopy(seashell_render,seashell_font,&seashell_fontGet,&seashell_fontLoc);
			seashell_fontLoc.x += seashell_textWidth;
		    }
		}
		
		SDL_RenderPresent(seashell_render);
	    }

	    // *clean events!*
	    if (seashell_hasInput) {
		while (SDL_PollEvent(&seashell_event)) {}
	    } else {
		INTERNAL_pushFeed();
	    }
	    // done at the end because input may be recieved during our stuff!
	    
	    if (seashell_holding) {
		unsigned int end = SDL_GetTicks();
		if (end-start < 16) {
		    SDL_Delay(abs(16-(end-start)));
		}
	    }
	} while (seashell_oper);
    } else if (seashell_initted && seashell_holdsLeft) {
	seashell_holdsLeft--;

	if (seashell_measuringSteps) {
	    seashell_stepOn = 1;
	    seashell_stepsRec[0] = SDL_GetPerformanceCounter();
	}
    }
}

/*
**Documentation**

stepping call, holds into update with specific details.
*/

void __declspec(dllexport) seashell_step(char* stepTitle)
{
    if (seashell_initted && !seashell_quitted) {
	if (seashell_measuringSteps == 0) {
	    if (seashell_stepping) {
		if (stepTitle != NULL) {
		    INTERNAL_deferToIO(stepTitle);
		    seashell_renderEvent = 2;
		}
		seashell_calledFromStepping = 1;
		seashell_update();
		seashell_calledFromStepping = 0;
	    }
	} else {
	    if (seashell_holdsLeft >= 2) {
		while (seashell_stepOn >= seashell_stepsAllocd) {
		    seashell_stepsAllocd += defaultStepsAllocd;
		    seashell_stepsAdd = realloc(seashell_stepsAdd,seashell_stepsAllocd*sizeof(unsigned long long));
		    seashell_stepsRec = realloc(seashell_stepsRec,seashell_stepsAllocd*sizeof(unsigned long long));
		    memset(seashell_stepsAdd + ((seashell_stepsAllocd-defaultStepsAllocd)*sizeof(unsigned long long)),0,
			   defaultStepsAllocd*sizeof(unsigned long long));
		}
		
		unsigned long long temp = seashell_stepsRec[seashell_stepOn-1];
		seashell_stepsRec[seashell_stepOn] = SDL_GetPerformanceCounter();
		seashell_stepsAdd[seashell_stepOn] += (seashell_stepsRec[seashell_stepOn]-temp);

		if (seashell_holdsLeft == 2) {
		    seashell_ticksTotal += seashell_stepsAdd[seashell_stepOn];
		}
		
		seashell_stepOn++;
	    } else {
		char writeOut[256];
		sprintf(writeOut,"--%s--\n",stepTitle);
		INTERNAL_appendToIO(writeOut);
		sprintf(writeOut,"Total Ticks:   %u\n",seashell_stepsAdd[seashell_stepOn]);
		INTERNAL_appendToIO(writeOut);
		sprintf(writeOut,"Average Time:  %.10f\n",(seashell_stepsAdd[seashell_stepOn] + 0.0)/(seashell_holdsTotal*SDL_GetPerformanceFrequency() + 0.0));
		INTERNAL_appendToIO(writeOut);
		sprintf(writeOut,"Execs / Sec:   %.10f\n",(seashell_holdsTotal*SDL_GetPerformanceFrequency() + 0.0)/(seashell_stepsAdd[seashell_stepOn] + 0.0));
		INTERNAL_appendToIO(writeOut);
		sprintf(writeOut,"*Percentage*:  %.6f\n\n",100*(seashell_stepsAdd[seashell_stepOn] + 0.0)/(seashell_ticksTotal + 0.0));
		INTERNAL_appendToIO(writeOut);
		seashell_stepOn++;
	    }
	}
    }
}

/*
**Documentation**

print a string to the stream
*/

void __declspec(dllexport) seashell_print(char* string)
{
    if (seashell_initted && !seashell_quitted) {
	INTERNAL_deferToIO(string);
	seashell_renderEvent = 2;
    }
}

/*
**Documentation**

force stepping in the c code.
*/

void __declspec(dllexport) seashell_stepit()
{
    if (seashell_stepping == 0) {
	seashell_holding = 1;
	seashell_stepping = 1;
	if (seashell_initted && !seashell_quitted) {
	    INTERNAL_deferToIO("*Entering stepping"); //expresses to user + fixes issue of newlines
	}
    }
}

/*
**Documentation**

force frame controlling in the c code.
*/

void __declspec(dllexport) seashell_holdit()
{
    if (seashell_holding == 0) {
	seashell_holding = 1;
	if (seashell_initted && !seashell_quitted) {
	    INTERNAL_deferToIO("*Entering holding"); //expresses to user + fixes issue of newlines
	}
    }
}

/*
**Documentation**

force-exit controlling in the c code.
*/

void __declspec(dllexport) seashell_exitcontrol()
{
    if (seashell_holding) {
	seashell_holding = 0;
	seashell_stepping = 0;
	if (seashell_initted && !seashell_quitted) {
	    INTERNAL_deferToIO("*Forcing exit of control");
	}
    }
}

/*
**Documentation**

simply, are we taking/giving input?
*/

char __declspec(dllexport) seashell_taking_input()
{
    return seashell_hasInput;
}

char __declspec(dllexport) seashell_giving_input()
{
    return !seashell_hasInput;
}

/*
**Documentation**

set new amount for variable, pointer for reallocing
*/

void __declspec(dllexport) seashell_var_amnt_set(char* var, int amnt, void* newPntr)
{
    if (seashell_initted && !seashell_quitted) {
	int numbGet = INTERNAL_getVarByName(var);

	if (numbGet == -1) {
	    char error[256];
	    sprintf(error,"**Error: unable to recognize variable '%s' from called var_amnt_set\n",var);
	    INTERNAL_deferToIO(error);
	} else {
	    seashell_vars[numbGet].amnt = amnt;

	    if (newPntr != NULL) {
		seashell_vars[numbGet].pntr = newPntr;
	    }
	}
    }
}
